<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\MVC\Controller\AdminController;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Language\Text;

class RseventsproControllerEvents extends AdminController
{
	protected $text_prefix = 'COM_RSEVENTSPRO_EVENTS';
	
	public function __construct($config = array()) {
		parent::__construct($config);
		
		$this->registerTask('unfeatured',	'featured');
	}
	
	public function getModel($name = 'Event', $prefix = 'RseventsproModel', $config = array('ignore_request' => true)) {
		return parent::getModel($name, $prefix, $config);
	}
	
	public function exportical() {
		$pks = $this->input->get('cid', array(0), 'array');
		
		// Force array elements to be integers
		$pks = array_map('intval',$pks);

		// Export events
		try {
			$this->getModel()->exportical($pks);
			exit();
		} catch (Exception $e) {
			$this->setMessage($e->getMessage());
			$this->setRedirect('index.php?option=com_rseventspro&view=events');
		}
	}
	
	/**
	 * Method to export events to CSV format.
	 *
	 * @return	.csv file
	 */
	public function exportcsv() {
		$pks = $this->input->get('cid', array(0), 'array');
		
		// Force array elements to be integers
		$pks = array_map('intval',$pks);
		
		// Export events
		try {
			$this->getModel()->exportcsv($pks);
		} catch (Exception $e) {
			$this->setMessage($e->getMessage());
			$this->setRedirect('index.php?option=com_rseventspro&view=events');
		}
	}
	
	/**
	 * Method to clear event rating.
	 *
	 * @return	void
	 */
	public function rating() {
		// Get the selected items
		$pks = $this->input->get('cid', array(0), 'array');
		
		// Force array elements to be integers
		$pks = array_map('intval',$pks);
		
		try {
			$this->getModel()->rating($pks);
		} catch (Exception $e) {
			$this->setMessage($e->getMessage());
		}

		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}

	/**
	 * Method to clear event hits.
	 *
	 * @return	void
	 */
	public function hits() {
		// Get the selected items
		$pks = $this->input->get('cid', array(0), 'array');

		// Force array elements to be integers
		$pks = array_map('intval',$pks);

		try {
			$this->getModel()->hits($pks);
		} catch (Exception $e) {
			$this->setMessage($e->getMessage());
		}

		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}

	/**
	 * Method to copy events.
	 *
	 * @return	void
	 */
	public function copy() {
		// Get the selected items
		$pks = $this->input->get('cid', array(0), 'array');
		
		// Force array elements to be integers
		$pks = array_map('intval',$pks);
		
		// Copy events
		try {
			$this->getModel()->copy($pks);
			$this->setMessage(Text::_('COM_RSEVENTSPRO_EVENTS_COPIED'));
		} catch (Exception $e) {
			$this->setMessage($e->getMessage());
		}
		
		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}
	
	/**
	 * Method to copy events.
	 *
	 * @return	void
	 */
	public function deletereports() {
		// Get the selected items
		$pks = $this->input->get('cid', array(0), 'array');
		
		$model = $this->getModel();
		
		// Force array elements to be integers
		$pks = array_map('intval',$pks);
		
		// Delete reports
		$model->deletereports($pks);
		
		$this->setRedirect('index.php?option=com_rseventspro&view=events&layout=report&id='.$this->input->getInt('ide', 0), Text::_('COM_RSEVENTSPRO_REPORTS_DELETED'));
	}
	
	/**
	 * Method to toggle the featured setting of a list of events.
	 *
	 * @return  void
	 * @since   1.6
	 */
	public function featured() {
		$ids    = $this->input->get('cid', array(), 'array');
		$values = array('featured' => 1, 'unfeatured' => 0);
		$task   = $this->getTask();
		$value  = ArrayHelper::getValue($values, $task, 0, 'int');

		try {
			$this->getModel()->featured($ids, $value);
		} catch (Exception $e) {
			$this->setMessage($e->getMessage(), 'error');
		}

		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}
	
	/**
	 * Method to run batch operations.
	 *
	 * @param   object  $model  The model.
	 * @return  boolean   True if successful, false otherwise and internal error is set.
	 * @since   1.6
	 */
	public function batch() {
		try {
			$this->getModel()->batchProcess($this->input->get('cid', array(), 'array'));
			$this->setMessage(Text::_('COM_RSEVENTSPRO_BATCH_COMPLETED'));
		} catch (Exception $e) {
			$this->setMessage($e->getMessage(), 'error');
		}

		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}
	
	/**
	 * Method to sync event dates.
	 */
	public function sync() {
		$this->getModel()->sync();

		$this->setMessage(Text::_('COM_RSEVENTSPRO_SYNC_COMPLETED'));
		
		// Preset the redirect
		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}
	
	/**
	 * Method to cancel events.
	 */
	public function cancel() {
		$this->getModel()->cancelevent($this->input->get('cid', array(), 'array'));

		$this->setMessage(Text::_('COM_RSEVENTSPRO_EVENTS_CANCELED'));

		// Preset the redirect
		$this->setRedirect('index.php?option=com_rseventspro&view=events');
	}

	public function addNew() {
		$id = $this->getModel()->saveNew($this->input->get('jform', array(), 'array'));

		$this->setMessage(Text::_('COM_RSEVENTSPRO_CREATE_NEW_EVENT_SAVED'));

		// Preset the redirect
		$this->setRedirect('index.php?option=com_rseventspro&view=event&layout=edit&id='.$id);
	}
}