<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2015 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\Filesystem\Folder;
use Joomla\Filesystem\File;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Factory;

class RseventsproModelEvent extends AdminModel
{
	protected $text_prefix = 'COM_RSEVENTSPRO';

	public function getTable($type = 'Event', $prefix = 'RseventsproTable', $config = array()) {
		return Table::getInstance($type, $prefix, $config);
	}

	public function getItem($pk = null) {
		if ($item = parent::getItem($pk)) {
			$db = Factory::getDbo();
			
			if (empty($item->location))
				$item->location = 0;
			
			$item->locationname = $this->getLocationName($item->location);
			
			// Convert image properties
			try {
				$registry = new Registry;
				$registry->loadString($item->properties);
				$item->properties = $registry->toArray();
			} catch (Exception $e) {
				$item->properties = array();
			}
			
			if (empty($item->start) || $item->start == $db->getNullDate()) {
				$item->start = Factory::getDate()->toSql();
			}
			
			if (empty($item->end) || $item->end == $db->getNullDate()) {
				$end = Factory::getDate();
				$end->modify('+2 hours');
				$item->end = $end->toSql();
			}
			
			if (empty($item->id)) 
				$item->published = 1;
			
			if (empty($item->URL) && empty($item->id)) 
				$item->URL = 'http://';
			
			if (empty($item->itemid)) 
				$item->itemid = '';
			
			if (empty($item->rsvp_quota)) 
				$item->rsvp_quota = '';
			
			if (empty($item->repeat_end) || $item->repeat_end == $db->getNullDate()) 
				$item->repeat_end = '';
			
			$item->repeat_end = str_replace(' 00:00:00','',$item->repeat_end);
			
			if (empty($item->start_registration) || $item->start_registration == $db->getNullDate()) 
				$item->start_registration = '';
			
			if (empty($item->end_registration) || $item->end_registration == $db->getNullDate()) 
				$item->end_registration = '';
				
			if (empty($item->unsubscribe_date) || $item->unsubscribe_date == $db->getNullDate()) 
				$item->unsubscribe_date = '';
				
			if (empty($item->early_fee_end) || $item->early_fee_end == $db->getNullDate()) 
				$item->early_fee_end = '';
				
			if (empty($item->late_fee_start) || $item->late_fee_start == $db->getNullDate()) 
				$item->late_fee_start = '';
				
			if (empty($item->rsvp_start) || $item->rsvp_start == $db->getNullDate()) 
				$item->rsvp_start = '';
			
			if (empty($item->rsvp_end) || $item->rsvp_end == $db->getNullDate()) 
				$item->rsvp_end = '';
			
			if (empty($item->waitinglist_limit)) 
				$item->waitinglist_limit = '';
				
			if (empty($item->repeat_type)) 
				$item->repeat_type = 1;
				
			if (empty($item->repeat_interval)) 
				$item->repeat_interval = 0;	
				
			if (empty($item->owner)) 
				$item->owner = Factory::getUser()->get('id');
				
			if (empty($item->invoice_padding)) 
				$item->invoice_padding = '';
			
			$item->waitinglist_time = rseventsproHelper::secondsToTime($item->waitinglist_time);
			
			if (empty($item->tickets_amount))
				$item->tickets_amount = '';
			
			if (empty($item->overbooking_amount))
				$item->overbooking_amount = '';
			
			if (empty($item->max_tickets_amount))
				$item->max_tickets_amount = '';
			
			if (!empty($item->rsm_lists)) {
				try {
					$registry = new Registry;
					$registry->loadString($item->rsm_lists);
					$item->rsm_lists = $registry->toArray();
				} catch (Exception $e) {
					$item->rsm_lists = array();
				}
			}
			
			if (!empty($item->repeat_also)) {
				try {
					$registry = new Registry;
					$registry->loadString($item->repeat_also);
					$item->repeat_also = $registry->toArray();
				} catch (Exception $e) {
					$item->repeat_also = array();
				}
			}
			
			if (!empty($item->exclude_dates)) {
				try {
					$registry = new Registry;
					$registry->loadString($item->exclude_dates);
					$item->exclude_dates = $registry->toArray();
				} catch (Exception $e) {
					$item->exclude_dates = array();
				}
			}
			
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
			$event	= RSEvent::getInstance($item->id);
			
			$item->options = $event->getEventOptions();
			
			RSEventsProTranslations::overrideValues($item, 'event');
		}
		
		return $item;
	}

	public function getForm($data = array(), $loadData = true) {
		// Get the form.
		$form = $this->loadForm('com_rseventspro.event', JPATH_ADMINISTRATOR.'/components/com_rseventspro/models/forms/event.xml', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		if (rseventsproHelper::isJ4()) {
			$form->setFieldAttribute('payments', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('sharing', 'layout', 'joomla.form.field.list-fancy-select', 'options');
			$form->setFieldAttribute('rsm_lists', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('gallery_tags', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('repeat_also', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('repeat_also', 'class', 'repeat_also_fancy');
			$form->setFieldAttribute('exclude_dates', 'layout', 'joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('exclude_dates', 'class', 'exclude_dates_fancy');
		} else {
			$form->setFieldAttribute('name', 'class', 'span10');
			$form->setFieldAttribute('small_description', 'class', 'span10');
			$form->setFieldAttribute('URL', 'class', 'span10');
			$form->setFieldAttribute('phone', 'class', 'span10');
			$form->setFieldAttribute('email', 'class', 'span10');
			$form->setFieldAttribute('metaname', 'class', 'span10');
			$form->setFieldAttribute('metakeywords', 'class', 'span10');
			$form->setFieldAttribute('metadescription', 'class', 'span10');
			$form->setFieldAttribute('event_ended', 'class', 'span10');
			$form->setFieldAttribute('event_full', 'class', 'span10');
			$form->setFieldAttribute('repeat_on_day_order', 'class', 'input-small');
			$form->setFieldAttribute('repeat_on_day_type', 'class', 'input-small');
			$form->setFieldAttribute('repeat_interval', 'class', 'input-small');
			$form->setFieldAttribute('repeat_type', 'class', 'input-small');
			$form->setFieldAttribute('repeat_on_day', 'class', 'input-small');
			$form->setFieldAttribute('early_fee', 'class', 'input-small');
			$form->setFieldAttribute('early_fee_type', 'class', 'input-small');
			$form->setFieldAttribute('late_fee', 'class', 'input-small');
			$form->setFieldAttribute('late_fee_type', 'class', 'input-small');
			$form->setFieldAttribute('notify_me_emails', 'class', 'span10');
			$form->setFieldAttribute('notify_me_paid_emails', 'class', 'span10');
			$form->setFieldAttribute('notify_me_unsubscribe_emails', 'class', 'span10');
		}
		
		if ($form->getValue('allday') == 1) {
			$form->setFieldAttribute('start', 'allday', 'true');
			$form->setFieldAttribute('end', 'allday', 'true');
		}
		
		if ($form->getValue('rsvp') == 1) {
			$form->setFieldAttribute('registration', 'muted', 'true');
			$form->setFieldAttribute('registration', 'disabled', 'true');
		}
		
		if ($form->getValue('registration') == 1) {
			$form->setFieldAttribute('rsvp', 'muted', 'true');
			$form->setFieldAttribute('rsvp', 'disabled', 'true');
		}
		
		if (Factory::getApplication()->isClient('site')) {
			$form->setFieldAttribute('description', 'buttons', rseventsproHelper::getConfig('enable_buttons','bool') ? 'true' : 'false');
		}
		
		RSEventsProTranslations::setTranslateIcon($form, 'event');
		
		return $form;
	}
	
	public function getFormDependencies() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$id		= Factory::getApplication()->input->getInt('id',0);
		$event	= RSEvent::getInstance($id);
		
		return $event->getEventFormDependencies();
	}
	
	public function getFormTickets($id = null) {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$id		= is_null($id) ? Factory::getApplication()->input->getInt('id',0) : $id;
		$event	= RSEvent::getInstance($id);
		
		return $event->getEventFormTickets();
	}
	
	public function getFormCoupons() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$id		= Factory::getApplication()->input->getInt('id',0);
		$event	= RSEvent::getInstance($id);
		
		return $event->getEventFormCoupons();
	}
	
	public function getFormFeedback() {
		$id		= Factory::getApplication()->input->getInt('id',0);
		$form	= Form::getInstance('com_rseventspro.feedback', 'feedback',  array('control' => 'feedback', 'load_data' => false), false, false);
		
		$form->setValue('ide', null, $id);
		
		RSEventsProTranslations::setTranslateIcon($form, 'feedback');
		
		return $form;
	}
	
	public function getFormFaq() {
		$id		= Factory::getApplication()->input->getInt('id',0);
		$form	= Form::getInstance('com_rseventspro.faq', 'faq',  array('control' => 'faq', 'load_data' => false), false, false);
		
		$form->setValue('ide', null, $id);
		
		RSEventsProTranslations::setTranslateIcon($form, 'faq');
		
		return $form;
	}
	
	public function getFormItinerary() {
		$id		= Factory::getApplication()->input->getInt('id',0);
		$form	= Form::getInstance('com_rseventspro.itinerary', 'itinerary',  array('control' => 'itinerary', 'load_data' => false), false, false);
		
		$form->setValue('ide', null, $id);
		
		RSEventsProTranslations::setTranslateIcon($form, 'itinerary');
		
		return $form;
	}
	
	public function getFormTimeline() {
		$form	= Form::getInstance('com_rseventspro.itinerarytimeline', 'itinerarytimeline',  array('control' => 'timeline', 'load_data' => false), false, false);
		
		RSEventsProTranslations::setTranslateIcon($form, 'timeline');
		
		return $form;
	}

	protected function loadFormData() {
		if (Factory::getApplication()->input->get('layout') == 'invoice') {
			return $this->getInvoice();
		}
		
		// Check the session for previously entered form data.
		$data = Factory::getApplication()->getUserState('com_rseventspro.edit.event.data', array());

		if (empty($data))
			$data = $this->getItem();

		return $data;
	}

	public function save($data) {
		$table		= $this->getTable();
		$pk 		= (!empty($data['id'])) ? $data['id'] : (int) $this->getState($this->getName() . '.id');
		$isNew 		= true;
		$status		= false;
		
		// Load the row if saving an existing tag.
		if ($pk > 0) {
			$table->load($pk);
			$isNew  = false;
			$status = $table->published;
		}
		
		// Verify data
		if (!$table->verify($data)) {
			$this->setError($table->getError());
			return false;
		}
		
		// Bind the data.
		if (!$table->bind($data)) {
			$this->setError($table->getError());
			return false;
		}
		
		if (isset($data['from']))
			$table->from = $data['from'];
		
		// Check the data.
		if (!$table->check()) {
			$this->setError($table->getError());
			return false;
		}
		
		Factory::getApplication()->triggerEvent('onrsepro_beforeEventStore',array(array('data'=>&$table)));
		
		// Store the data.
		if (!$table->store()) {
			$this->setError($table->getError());
			return false;
		}
		
		// After store
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		$event = RSEvent::getInstance($table->id);
		$event->save($table, $isNew);
		
		Factory::getApplication()->triggerEvent('onrsepro_afterEventStore',array(array('data'=>&$table, 'event' => $event)));
		
		// Check for cancel event, and send emails
		if ($status !== false) {
			if ($status != $data['published'] && $data['published'] == 3) {
				rseventsproEmails::cancel($table->id);
			}
		}
		
		// Replace translations
		if ($translation = RSEventsProTranslations::getTranslation('event', $table->id, 'name')) {
			$table->name = $translation;
		}
		
		$this->setState($this->getName() . '.id', $table->id);
		$this->setState($this->getName() . '.name', $table->name);
		
		return true;
	}

	public function exportical($pks) {
		if (!empty($pks)) {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/ical.php';
			$ical = RSEventsProiCal::getInstance($pks);
			
			$ical->toIcal();
		} else {
			throw new Exception(Text::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
		}
	}
	
	public function exportcsv($pks) {
		if (!empty($pks)) {
			$db     = Factory::getDbo();
			$query  = $db->getQuery(true);
			$lines  = array();
			$csv    = '';
			
			$query->clear()
				->select('e.*')
				->select($db->qn('l.name','locationname'))->select($db->qn('l.address'))
				->from($db->qn('#__rseventspro_events','e'))
				->join('left', $db->qn('#__rseventspro_locations','l').' ON '.$db->qn('l.id').' = '.$db->qn('e.location'))
				->where($db->qn('e.id').' IN ('.implode(',',$pks).')');
			
			$db->setQuery($query);
			if ($events = $db->loadObjectList()) {
				foreach ($events as $event) {
					$line = array();

					$line[] = strip_tags($event->name);
					$line[] = Factory::getDate($event->start)->toSql();
					$line[] = Factory::getDate($event->end)->toSql();
					$line[] = strip_tags($event->description);
					$line[] = strip_tags($event->URL);
					$line[] = strip_tags($event->email);
					$line[] = strip_tags($event->phone);
					$line[] = strip_tags($event->locationname);
					$line[] = strip_tags($event->address);

					// Escape CSV line
					rseventsproHelper::csvEscape($line);

					$lines[] = $line;
				}

				foreach ($lines as $line) {
					$csv .= rseventsproHelper::csvLine($line)."\n";
				}
				
				$file = 'Events.csv';
				header("Content-type: text/csv; charset=UTF-8");
				header("Content-Disposition: attachment; filename=$file");
				echo rtrim($csv,"\n");

				exit();
			} else {
				throw new Exception(Text::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
			}
		} else {
			throw new Exception(Text::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
		}
	}
	
	public function rating($pks) {
		if (!empty($pks)) {
			$db = Factory::getDbo();
			$query = $db->getQuery(true);
			
			$query->clear()
				->delete()
				->from($db->qn('#__rseventspro_rating'))
				->where($db->qn('ide').' IN ('.implode(',',$pks).')');
			
			$db->setQuery($query);
			$db->execute();
		} else {
			throw new Exception(Text::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
		}
	}

	public function hits($pks) {
		if (!empty($pks)) {
			$db = Factory::getDbo();
			$query = $db->getQuery(true);

			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('hits').' = '.$db->q('0'))
				->where($db->qn('id').' IN ('.implode(',',$pks).')');

			$db->setQuery($query);
			$db->execute();
		} else {
			throw new Exception(Text::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
		}
	}

	public function copy($pks) {
		if (!empty($pks)) {
			foreach ($pks as $pk) {
				rseventsproHelper::copy($pk,0);
			}
		} else {
			throw new Exception(Text::_('JLIB_DATABASE_ERROR_NO_ROWS_SELECTED'));
		}
	}

	public function removeticket() {
		$id = Factory::getApplication()->input->getInt('id');
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		$query->clear()
			->delete()
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$response = $db->execute();
		
		RSEventsProTranslations::removeTranslations('ticket', $id);
		
		if ($response)
			Factory::getApplication()->triggerEvent('onrsepro_afterDeleteTicket', array(array('id' => $id)));
		
		return $response;
	}

	public function removecoupon() {
		$id = Factory::getApplication()->input->getInt('id');
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		$query->clear()
			->delete()
			->from($db->qn('#__rseventspro_coupons'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($db->execute()) {
			$query->clear()
				->delete()
				->from($db->qn('#__rseventspro_coupon_codes'))
				->where($db->qn('idc').' = '.$id);
			$db->setQuery($query);
			$db->execute();
			return true;
		} else return false;
	}

	public function upload() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$input	= Factory::getApplication()->input;
		$id		= $input->getInt('id');
		$icon	= $input->files->get('icon',array(),'array');
		$local	= $input->getString('local');
		$path	= JPATH_SITE.'/components/com_rseventspro/assets/images/events/';
		$empty  = empty($local) && $icon['size'] == 0;
		$type	= '';
		$config = rseventsproHelper::getConfig();
		
		if ($icon['size'] > 0 && $icon['error'] == 0) {
			$type = 'upload';
		} elseif (!empty($local)) {
			$type = 'local';
		}
		
		if (!$empty || empty($type)) {
			// Check for file extension
			$ext = rseventsproHelper::getExt($type == 'upload' ? $icon['name'] : $local);
			if (!in_array(strtolower($ext),array('jpg','png','jpeg'))) {
				throw new Exception(Text::_('COM_RSEVENTSPRO_WRONG_FILE_TYPE'));
			}
			
			// Remove old icon
			$query->clear()
				->select($db->qn('icon'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.$id);
			
			$db->setQuery($query);
			if ($eventicon = $db->loadResult()) {
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$eventicon)) {
					File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$eventicon);
				}
				
				$extension	= rseventsproHelper::getExt($eventicon);
				$name		= File::stripExt($eventicon);
				
				// Delete small icon
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_small_width.'/'.md5($config->icon_small_width.$name).'.'.$extension)) {
					File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_small_width.'/'.md5($config->icon_small_width.$name).'.'.$extension);
				}
				
				// Delete big icon
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_big_width.'/'.md5($config->icon_big_width.$name).'.'.$extension)) {
					File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/'.$config->icon_big_width.'/'.md5($config->icon_big_width.$name).'.'.$extension);
				}
				
				// Delete event listing icon from backend
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/70/'.md5('70'.$name).'.'.$extension)) {
					File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/70/'.md5('70'.$name).'.'.$extension);
				}
				
				// Delete event edit icon
				if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/188/'.md5('188'.$name).'.'.$extension)) {
					File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/188/'.md5('188'.$name).'.'.$extension);
				}
			}
			
			// Copy or upload the new icon
			if ($type == 'upload') {
				// Upload file
				if ($icon['error'] == 0) {
					$file		= File::makeSafe($icon['name']);
					$filename	= basename(File::stripExt($file));
					
					while(file_exists($path.$filename.'.'.$ext)) {
						$filename .= rand(1,999);
					}
						
					if (File::upload($icon['tmp_name'],$path.$filename.'.'.$ext)) {
						$query->clear()
							->update($db->qn('#__rseventspro_events'))
							->set($db->qn('icon').' = '.$db->q($filename.'.'.$ext))
							->set($db->qn('properties').' = '.$db->q(''))
							->where($db->qn('id').' = '.$id);
						
						$db->setQuery($query);
						$db->execute();
						
						$this->setState('com_rseventspro.edit.icon', $filename.'.'.$ext);
					} else { 
						throw new Exception(Text::_('COM_RSEVENTSPRO_UPLOAD_ERROR'));
					}
				} else {
					throw new Exception(Text::_('COM_RSEVENTSPRO_FILE_ERROR'));
				}
			} else {
				// Copy local file
				$params 	= ComponentHelper::getParams('com_media');
				$root		= JPATH_ROOT . '/' . $params->get('image_path', 'images');
				$root		= str_replace(DIRECTORY_SEPARATOR, '/', $root . '/');
				$file		= $root.$local;
				$filename	= basename(File::stripExt($file));
				
				while(file_exists($path.$filename.'.'.$ext)) {
					$filename .= rand(1,999);
				}
				
				if (File::copy($file,$path.$filename.'.'.$ext)) {
					$query->clear()
						->update($db->qn('#__rseventspro_events'))
						->set($db->qn('icon').' = '.$db->q($filename.'.'.$ext))
						->set($db->qn('properties').' = '.$db->q(''))
						->where($db->qn('id').' = '.$id);
					
					$db->setQuery($query);
					$db->execute();
					
					$this->setState('com_rseventspro.edit.icon', $filename.'.'.$ext);
				} else { 
					throw new Exception(Text::_('COM_RSEVENTSPRO_COPY_ERROR'));
				}
			}
		} else {
			throw new Exception(Text::_('COM_RSEVENTSPRO_NO_FILE_SELECTED'));
		}
	}
	
	public function getIcon() {
		if ($icon = Factory::getApplication()->input->getString('icon','')) {
			return base64_decode($icon);
		}
		
		return false;
	}

	public function deleteicon() {
		$id		= Factory::getApplication()->input->getInt('id');
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('icon'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($icon = $db->loadResult()) {
			if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$icon))
				File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/events/'.$icon);
			
			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('icon').' = '.$db->q(''))
				->set($db->qn('properties').' = '.$db->q(''))
				->where($db->qn('id').' = '.$id);
			
			$db->setQuery($query);
			$db->execute();
		}

		return true;
	}

	public function crop() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$input	= Factory::getApplication()->input;
		$id		= $input->getInt('id');
		$thumbs = JPATH_SITE.'/components/com_rseventspro/assets/images/events/thumbs/';
		
		$query->clear()
			->select($db->qn('icon'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$icon = $db->loadResult();
		
		$this->setState('com_rseventspro.crop.icon', $icon);
		
		$left	= $input->getInt('x1');
		$top	= $input->getInt('y1');
		$width	= $input->getInt('width');
		$height	= $input->getInt('height');
		
		$properties = array('left' => $left, 'top' => $top, 'width' => $width, 'height' => $height);
		$registry = new Registry;
		$registry->loadArray($properties);
		$properties = $registry->toString();
		
		$query->clear()
			->update($db->qn('#__rseventspro_events'))
			->set($db->qn('properties').' = '.$db->q($properties))
			->set($db->qn('aspectratio').' = '.$db->q($input->getInt('aspectratio',0)))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$db->execute();
		
		// Get file extension
		$extension	= rseventsproHelper::getExt($icon);
		// Strip extension
		$name		= File::stripExt($icon);
		
		if ($folders = Folder::folders($thumbs)) {
			$folders = array_map('intval',$folders);
			
			foreach ($folders as $folder) {
				if (file_exists($thumbs.$folder.'/'.md5($folder.$name).'.'.$extension)) {
					File::delete($thumbs.$folder.'/'.md5($folder.$name).'.'.$extension);
				}
			}
		}
		
		return true;
	}

	public function savefile() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$jinput	= Factory::getApplication()->input->post;
		$id		= $jinput->getInt('id');
		$permissions = '';
		
		$fp0 = $jinput->get('fp0');
		$fp1 = $jinput->get('fp1');
		$fp2 = $jinput->get('fp2');
		$fp3 = $jinput->get('fp3');
		$fp4 = $jinput->get('fp4');
		$fp5 = $jinput->get('fp5');
		
		if (isset($fp0) && $fp0 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp1) && $fp1 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp2) && $fp2 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp3) && $fp3 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp4) && $fp4 == 1) $permissions .= '1'; else $permissions .= '0';
		if (isset($fp5) && $fp5 == 1) $permissions .= '1'; else $permissions .= '0';
		
		$query->clear()
			->update($db->qn('#__rseventspro_files'))
			->set($db->qn('name').' = '.$db->q($jinput->getString('name')))
			->set($db->qn('permissions').' = '.$db->q($permissions))
			->where($db->qn('id').' = '.$db->q($id));
		
		$db->setQuery($query);
		$db->execute();
		
		$this->setState('com_rseventspro.file.id',$id);
		$this->setState('com_rseventspro.file.name',$jinput->getString('name'));
		
		return true;
	}

	public function removefile() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id');
		
		$query->clear()
			->select($db->qn('location'))
			->from($db->qn('#__rseventspro_files'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($file = $db->loadResult()) {
			if (file_exists(JPATH_SITE.'/components/com_rseventspro/assets/images/files/'.$file)) {
				if (File::delete(JPATH_SITE.'/components/com_rseventspro/assets/images/files/'.$file)) {
					$query->clear()
						->delete()
						->from($db->qn('#__rseventspro_files'))
						->where($db->qn('id').' = '.$id);
						
					$db->setQuery($query);
					$db->execute();
					return true;
				}
			}
		}
		
		return false;
	}

	public function tickets() {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$input		= Factory::getApplication()->input;
		$params		= $input->get('params',array(),'array');
		
		if (!empty($params)) {
			foreach ($params as $i => $param) {
				$registry = new Registry;
				$registry->loadArray($param);
				$position = $registry->toString();
				
				$query->clear()
					->update($db->qn('#__rseventspro_tickets'))
					->set($db->qn('position').' = '.$db->q($position))
					->where($db->qn('id').' = '.(int) $i);
				
				$db->setQuery($query);
				$db->execute();
			}
		}
	}

	public function deletereports($pks) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete()
			->from($db->qn('#__rseventspro_reports'))
			->where($db->qn('id').' IN ('.implode(',',$pks).')');
		$db->setQuery($query);
		$db->execute();
	}

	public function featured($pks, $value = 0) {
		// Sanitize the ids.
		$pks = (array) $pks;
		$pks = array_map('intval',$pks);

		if (empty($pks)) {
			throw new Exception(Text::_('JERROR_NO_ITEMS_SELECTED'));
		}
		
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->update($db->qn('#__rseventspro_events'))
			->set($db->qn('featured').' = '.(int) $value)
			->where($db->qn('id').' IN ('.implode(',', $pks).')');
		
		$db->setQuery($query);
		$db->execute();
	}
	
	public function batchProcess($pks) {
		// Sanitize the ids.
		$pks = (array) $pks;
		$pks = array_map('intval',$pks);
		
		$batch	 = Factory::getApplication()->input->get('batch',array(),'array');
		$all	 = isset($batch['all']) ? $batch['all'] : 0;
		$options = isset($batch['options']) ? $batch['options'] : array();
		$enable	 = isset($batch['enable_options']) ? 1 : 0;
		
		$categories	= isset($batch['categories']) ? $batch['categories'] : array();
		$tags		= isset($batch['tags']) ? $batch['tags'] : array();
		$groups		= isset($batch['groups']) ? $batch['groups'] : array();
		$sponsors	= isset($batch['sponsors']) ? $batch['sponsors'] : array();
		$speakers	= isset($batch['speakers']) ? $batch['speakers'] : array();
		$location	= isset($batch['locations']) ? $batch['locations'] : 0;
		$itemid		= isset($batch['itemid']) ? $batch['itemid'] : '';
		$type		= isset($batch['type']) ? $batch['type'] : 'r';
		
		$registration 	= isset($batch['registration']) ? $batch['registration'] : '';
		$activation 	= isset($batch['activation']) ? $batch['activation'] : '';
		$denied			= isset($batch['denied']) ? $batch['denied'] : '';
		$refund 		= isset($batch['refund']) ? $batch['refund'] : '';
		
		if (empty($pks) && !$all) {
			throw new Exception(Text::_('JERROR_NO_ITEMS_SELECTED'));
		}

		$db		 = Factory::getDbo();
		$query	 = $db->getQuery(true);

		if ($enable) {
			$defaults = rseventsproHelper::getDefaultOptions();

			try {
				$registry = new Registry;
				$registry->loadString($defaults);
				$defaults = $registry->toArray();
			} catch (Exception $e) {}

			foreach ($defaults as $name => $value) {
				if (!isset($options[$name]))
					$options[$name] = 0;
			}

			$registry = new Registry;
			$registry->loadArray($options);

			$query->update($db->qn('#__rseventspro_events'))->set($db->qn('options').' = '.$db->q($registry->toString()));

			if ($location) {
				$query->set($db->qn('location').' = '.$db->q($location));
			}

			if ($itemid) {
				$query->set($db->qn('itemid').' = '.$db->q($itemid));
			}

			if ($registration != '') {
				$query->set($db->qn('disable_registration').' = '.$db->q($registration));
			}

			if ($activation != '') {
				$query->set($db->qn('disable_activation').' = '.$db->q($activation));
			}

			if ($denied != '') {
				$query->set($db->qn('disable_denied').' = '.$db->q($denied));
			}

			if ($refund != '') {
				$query->set($db->qn('disable_refund').' = '.$db->q($refund));
			}

			if (!$all) {
				$query->where($db->qn('id').' IN ('.implode(',', $pks).')');
			}

			$db->setQuery($query);
			$db->execute();
		} else {
			if ($location || $itemid || $registration != '' || $activation != '' || $denied != '' || $refund != '') {
				$query->update($db->qn('#__rseventspro_events'));

				if ($location) {
					$query->set($db->qn('location').' = '.$db->q($location));
				}

				if ($itemid) {
					$query->set($db->qn('itemid').' = '.$db->q($itemid));
				}

				if ($registration != '') {
					$query->set($db->qn('disable_registration').' = '.$db->q($registration));
				}

				if ($activation != '') {
					$query->set($db->qn('disable_activation').' = '.$db->q($activation));
				}

				if ($denied != '') {
					$query->set($db->qn('disable_denied').' = '.$db->q($denied));
				}

				if ($refund != '') {
					$query->set($db->qn('disable_refund').' = '.$db->q($refund));
				}

				if (!$all) {
					$query->where($db->qn('id').' IN ('.implode(',', $pks).')');
				}

				$db->setQuery($query);
				$db->execute();
			}
		}

		// Update categories
		$this->updateBatchOptions($categories, 'category', $pks, $all, $type);
		// Update groups
		$this->updateBatchOptions($groups, 'groups', $pks, $all, $type);
		// Update sponsors
		$this->updateBatchOptions($sponsors, 'sponsor', $pks, $all, $type);
		// Update speakers
		$this->updateBatchOptions($speakers, 'speaker', $pks, $all, $type);

		// Update tags
		if (!empty($tags)) {
			// Get the tags ids
			$ids = array();
			foreach ($tags as $tag) {
				$tag = trim($tag);
				$query->clear()
					->select($db->qn('id'))
					->from($db->qn('#__rseventspro_tags'))
					->where($db->qn('name').' = '.$db->q($tag));
				$db->setQuery($query);
				if ($tid = (int) $db->loadResult()) {
					$ids[] = $tid;
				} else {
					$query->clear()
						->insert($db->qn('#__rseventspro_tags'))
						->set($db->qn('name').' = '.$db->q($tag))
						->set($db->qn('published').' = '.$db->q(1));

					$db->setQuery($query);
					$db->execute();
					$tid = $db->insertid();
					$ids[] = $tid;
				}
			}

			if ($all) {
				if ($type == 'r') {
					$query->clear()
						->delete($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q('tag'));
					$db->setQuery($query);
					$db->execute();
				}

				$query->clear()
					->select($db->qn('id'))
					->from($db->qn('#__rseventspro_events'));
				$db->setQuery($query);
				if ($events = $db->loadColumn()) {
					foreach ($events as $id) {
						foreach ($ids as $idt) {
							$query->clear()
								->select($db->qn('id'))
								->from($db->qn('#__rseventspro_taxonomy'))
								->where($db->qn('type').' = '.$db->q('tag'))
								->where($db->qn('ide').' = '.$db->q($id))
								->where($db->qn('id').' = '.$db->q($idt));
							$db->setQuery($query);
							$txid = (int) $db->loadResult();

							if (!$txid) {
								$query->clear()
									->insert($db->qn('#__rseventspro_taxonomy'))
									->set($db->qn('type').' = '.$db->q('tag'))
									->set($db->qn('ide').' = '.$db->q($id))
									->set($db->qn('id').' = '.$db->q($idt));
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
				}
			} else {
				if ($type == 'r') {
					$query->clear()
						->delete($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q('tag'))
						->where($db->qn('ide').' IN ('.implode(',', $pks).')');
					$db->setQuery($query);
					$db->execute();
				}

				foreach ($pks as $id) {
					foreach ($ids as $idt) {
						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_taxonomy'))
							->where($db->qn('type').' = '.$db->q('tag'))
							->where($db->qn('ide').' = '.$db->q($id))
							->where($db->qn('id').' = '.$db->q($idt));
						$db->setQuery($query);
						$txid = (int) $db->loadResult();

						if (!$txid) {
							$query->clear()
								->insert($db->qn('#__rseventspro_taxonomy'))
								->set($db->qn('type').' = '.$db->q('tag'))
								->set($db->qn('ide').' = '.$db->q($id))
								->set($db->qn('id').' = '.$db->q($idt));
							$db->setQuery($query);
							$db->execute();
						}
					}
				}
			}
		}
	}
	
	public function sync() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$config	= Factory::getConfig();
		$offset	= $config->get('offset');
		
		$query->select($db->qn('id'))
			->select($db->qn('start'))->select($db->qn('timezone'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('allday').' = 1');
		
		$db->setQuery($query);
		if ($events = $db->loadObjectList()) {
			foreach ($events as $event) {
				if ($event->timezone == $offset) {
					continue;
				}
				
				if (empty($event->timezone)) {
					$event->timezone = 'UTC';
				}
				
				$date = new DateTime($event->start, new DateTimezone('UTC'));
				$date->setTimezone(new DateTimezone($event->timezone));
				
				$date = new DateTime($date->format('Y-m-d H:i:s'), new DateTimezone($offset));
				$date->setTimezone(new DateTimezone('UTC'));
				
				$start = $date->format('Y-m-d H:i:s');
				
				$query->clear()
					->update($db->qn('#__rseventspro_events'))
					->set($db->qn('start').' = '.$db->q($start))
					->set($db->qn('timezone').' = '.$db->q($offset))
					->where($db->qn('id').' = '.$db->q($event->id));
				$db->setQuery($query);
				$db->execute();
			}
		}
		
		return true;
	}
	
	public function ticketsorder() {
		$db		 = Factory::getDbo();
		$query	 = $db->getQuery(true);
		$input	 = Factory::getApplication()->input;
		$id		 = $input->getInt('id',0);
		$tickets = $input->get('ticket', array(), 'array');
		
		foreach ($tickets as $i => $ticket) {
			$query->clear()
				->update($db->qn('#__rseventspro_tickets'))
				->set($db->qn('order').' = '.$db->q($i))
				->where($db->qn('id').' = '.$db->q($ticket))
				->where($db->qn('ide').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}
	}
	
	public function subscribeUsers() {
		if (!rseventsproHelper::isRsmail()) {
			return;
		}
		
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$app 	= Factory::getApplication();
		$id		= $app->input->getInt('id',0);
		$lists 	= $app->input->get('lists', array(), 'array');
		$status	= $app->input->get('status', array(), 'array');
		
		if ($lists) {
			$query->select('DISTINCT '.$db->qn('u.email'))
				->select($db->qn('u.id'))
				->from($db->qn('#__rseventspro_users','u'))
				->where($db->qn('u.ide').' = '.$id);
			
			if ($status) {
				$status = array_map('intval', $status);
				$query->where($db->qn('u.state').' IN ('.implode(',', $status).')');
			}
			
			$app->triggerEvent('onrsepro_subscriptionsQuery',array(array('query'=>&$query, 'rule' => 'u.ide')));
			
			$db->setQuery($query);
			if ($subscribers = $db->loadObjectList()) {
				require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';
				$rsmail = new rsmHelper;
				
				foreach ($subscribers as $subscriber) {
					foreach ($lists as $listID) {
						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_rsmail'))
							->where($db->qn('ids').' = '.$db->q($subscriber->id))
							->where($db->qn('idl').' = '.$db->q($listID));
						$db->setQuery($query);
						if (!$db->loadResult()) {
							$list = (object) array('id' => $listID);
							if ($rsmail->subscribe($subscriber->email, $list, 1, 0)) {
								$query->clear()
									->insert($db->qn('#__rseventspro_rsmail'))
									->set($db->qn('ids').' = '.$db->q($subscriber->id))
									->set($db->qn('idl').' = '.$db->q($listID));
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
				}
			}
		}
	}
	
	public function duplicateticket() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$app	= Factory::getApplication();
		$id		= $app->input->getInt('id', 0);
		$cart	= false;
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$ticket = $db->loadObject();
		
		$ticket->id = 0;
		$ticket->name = Text::sprintf('COM_RSEVENTSPRO_DUPLICATE_TICKET_NAME', $ticket->name);
		
		$query->clear()
			->select('MAX('.$db->qn('order').')')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('ide').' = '.$ticket->ide);
		$db->setQuery($query);
		$ticket->order = (int) $db->loadResult() + 1;
		
		// Save ticket
		$db->insertObject('#__rseventspro_tickets', $ticket, 'id');
		
		$app->triggerEvent('onrsepro_isCart',array(array('cart'=>&$cart)));
		
		if ($cart) {
			$query->clear()
				->select('*')
				->from($db->qn('#__rseventspro_fields'))
				->where($db->qn('idt').' = '.$db->q($id));
			$db->setQuery($query);
			if ($fields = $db->loadObjectList()) {
				foreach ($fields as $field) {
					$field->id = 0;
					$field->idt = $ticket->id;
					$db->insertObject('#__rseventspro_fields', $field);
				}
			}
		}
		
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/events.php';
		
		$eventClass = RSEvent::getInstance($ticket->ide);
		
		$app->input->set('format', 'raw');
		
		$view = new HtmlView(array(
			'name' => 'event',
			'layout' => 'edit',
			'base_path' => JPATH_ADMINISTRATOR.'/components/com_rseventspro'
		));
		
		$view->eventClass 	= $eventClass;
		$view->tickets 		= $eventClass->getTickets($ticket->id);
		$view->ticketsform	= $this->getFormTickets($ticket->ide);
		
		$response = new stdClass();
		$response->id = $ticket->id;
		$response->name = $ticket->name;
		$response->html = $view->loadTemplate('tickets');
		
		return $response;
	}
	
	public function save2copy($id) {
		return rseventsproHelper::copy($id,0);
	}
	
	public function cancelevent($pks) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		// Sanitize the ids.
		$pks = (array) $pks;
		$pks = array_map('intval',$pks);
		
		if ($pks) {
			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('published').' = '.$db->q(3))
				->where($db->qn('id').' IN ('.implode(',',$pks).')');
			$db->setQuery($query);
			$db->execute();
			
			// Send emails
			foreach ($pks as $id) {
				rseventsproEmails::cancel($id);
			}
		}
		
		return true;
	}
	
	protected function getLocationName($id) {
		static $locations = array();
		
		if (!isset($locations[$id])) {
			$db = Factory::getDbo();
			$query = $db->getQuery(true);
			
			$query->clear()
				->select($db->qn('name'))
				->from($db->qn('#__rseventspro_locations'))
				->where($db->qn('id').' = '.$id);
			$db->setQuery($query);
			$name = $db->loadResult();
			
			if (Factory::getApplication()->isClient('site')) {
				// Replace translations
				if ($translation = RSEventsProTranslations::getTranslation('location', $id, 'name')) {
					$name = $translation;
				}
			}
			
			$locations[$id] = $name;
		}
		
		return $locations[$id];
	}
	
	protected function updateBatchOptions($array, $taxonomy, $pks, $all, $type) {
		if (!empty($array)) {
			$db		= Factory::getDbo();
			$query	= $db->getQuery(true);
			$array	= array_map('intval',$array);
			
			if ($all) {
				if ($type == 'r') {
					$query->clear()
						->delete($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q($taxonomy));
					$db->setQuery($query);
					$db->execute();
				}
				
				$query->clear()
					->select($db->qn('id'))
					->from($db->qn('#__rseventspro_events'));
				$db->setQuery($query);
				if ($events = $db->loadColumn()) {
					foreach ($events as $id) {
						foreach ($array as $arrayID) {
							$query->clear()
								->select($db->qn('id'))
								->from($db->qn('#__rseventspro_taxonomy'))
								->where($db->qn('type').' = '.$db->q($taxonomy))
								->where($db->qn('ide').' = '.$db->q($id))
								->where($db->qn('id').' = '.$db->q($arrayID));
							$db->setQuery($query);
							$txid = (int) $db->loadResult();
							
							if (!$txid) {
								$query->clear()
									->insert($db->qn('#__rseventspro_taxonomy'))
									->set($db->qn('type').' = '.$db->q($taxonomy))
									->set($db->qn('ide').' = '.$db->q($id))
									->set($db->qn('id').' = '.$db->q($arrayID));
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
				}
			} else {
				if ($type == 'r') {
					$query->clear()
						->delete($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q($taxonomy))
						->where($db->qn('ide').' IN ('.implode(',', $pks).')');
					$db->setQuery($query);
					$db->execute();
				}
				
				foreach ($pks as $id) {
					foreach ($array as $arrayID) {
						$query->clear()
							->select($db->qn('id'))
							->from($db->qn('#__rseventspro_taxonomy'))
							->where($db->qn('type').' = '.$db->q($taxonomy))
							->where($db->qn('ide').' = '.$db->q($id))
							->where($db->qn('id').' = '.$db->q($arrayID));
						$db->setQuery($query);
						$txid = (int) $db->loadResult();
						
						if (!$txid) {
							$query->clear()
								->insert($db->qn('#__rseventspro_taxonomy'))
								->set($db->qn('type').' = '.$db->q($taxonomy))
								->set($db->qn('ide').' = '.$db->q($id))
								->set($db->qn('id').' = '.$db->q($arrayID));
							$db->setQuery($query);
							$db->execute();
						}
					}
				}
			}
		}
	}
	
	public function getTicketForm() {
		$form = Form::getInstance('ticketoptions', JPATH_ADMINISTRATOR.'/components/com_rseventspro/models/forms/event_ticketoptions.xml', array('control' => 'jform'));
		$data = $this->getTicket();
		
		$form->bind($data);
		
		return $form;
	}
	
	public function getTicket() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id', 0);
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$ticket = $db->loadObject();
		
		if (empty($ticket->start)) $ticket->start = 1;
		if (empty($ticket->disabled)) $ticket->disabled = '';
		if (empty($ticket->available)) $ticket->available = 1;
		
		try {
			$registry = new Registry;
			$registry->loadString($ticket->position);
			$ticket->position = $registry->toArray();
		} catch (Exception $e) {
			$ticket->position = array();
		}
		
		try {
			$registry = new Registry;
			$registry->loadString($ticket->manual_disabled);
			$ticket->manual_disabled = $registry->toArray();
		} catch (Exception $e) {
			$ticket->manual_disabled = array();
		}
		
		return $ticket;
	}
	
	public function getPurchasedTickets() {
		$id	= Factory::getApplication()->input->getInt('id', 0);
		
		return $this->ticketCount($id);
	}
	
	public function saveTicketOptions($data) {
		$data['start'] = (int) $data['start'];
		$data['disabled'] = (int) $data['disabled'];
		$data['available'] = (int) $data['available'];
		$reset = (int) $data['reset'];
		
		unset($data['reset']);
		
		if (isset($data['manual_disabled']) && is_array($data['manual_disabled'])) {
			$registry = new Registry;
			$registry->loadArray($data['manual_disabled']);
			$data['manual_disabled'] = $registry->toString();
		} else {
			$data['manual_disabled'] = '';
		}
		
		if ($reset) {
			$data['disabled'] = 0;
			$data['available'] = 1;
			$data['manual_disabled'] = '';
			unset($data['start']);
		} else {
			if ($this->ticketCount($data['id'])) {
				unset($data['start'], $data['disabled_type'], $data['disabled'], $data['available'], $data['disabled_available'], $data['manual_disabled']);
			}
		}
		
		$ticket = (object) $data;
		
		return Factory::getDbo()->updateObject('#__rseventspro_tickets', $ticket, 'id');
	}
	
	protected function ticketCount($id) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select('COUNT('.$db->qn('ut.id').')')
			->from($db->qn('#__rseventspro_user_tickets', 'ut'))
			->join('RIGHT', $db->qn('#__rseventspro_users', 'u').' ON '.$db->qn('u.id').' = '.$db->qn('ut.ids'))
			->where($db->qn('u.state').' IN (0,1,2)')
			->where($db->qn('ut.idt').' = '.$db->q($id));
		$db->setQuery($query);
		return (int) $db->loadResult();
	}
	
	public function getInvoice() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery();
		$id		= Factory::getApplication()->input->getInt('id',0);
		$lang	= $this->getLanguage();
		
		$query->clear()
			->select($db->qn('title'))->select($db->qn('layout'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('ide'). ' = '.$db->q($id))
			->where($db->qn('lang'). ' = '.$db->q($lang));
		$db->setQuery($query);
		$invoice = $db->loadObject();
		
		if (!$invoice) {
			$query->clear()
				->select($db->qn('title'))->select($db->qn('layout'))
				->from($db->qn('#__rseventspro_invoice'))
				->where($db->qn('ide'). ' = '.$db->q($id))
				->where($db->qn('lang'). ' = '.$db->q('en-GB'));
			$db->setQuery($query);
			$invoice = $db->loadObject();
			
			if (!$invoice) {
				$invoice = new stdClass();
				$invoice->title = '';
				$invoice->layout = '';
			}
		}
		
		return array('invoice_language' => $lang, 'invoice_title' => $invoice->title, 'invoice_layout' => $invoice->layout);
	}
	
	public function getLanguage() {
		$jform	= Factory::getApplication()->input->get('jform',array(),'array');
		return !empty($jform['invoice_language']) ? $jform['invoice_language'] : Factory::getLanguage()->getTag();
	}
	
	public function saveinvoice($data) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery();
		$ide		= isset($data['id']) ? $data['id'] : 0;
		$language	= isset($data['invoice_language']) ? $data['invoice_language'] : $this->getLanguage();
		$title		= isset($data['invoice_title']) ? $data['invoice_title'] : 'Invoice';
		$layout		= isset($data['invoice_layout']) ? $data['invoice_layout'] : '';
		
		if (empty($ide)) {
			return false;
		}
		
		$query->clear()
			->select($db->qn('id'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('lang').' = '.$db->q($language))
			->where($db->qn('ide').' = '.$db->q($ide));
		
		$db->setQuery($query);
		if ($id = $db->loadResult()) {
			$query->clear()
				->update($db->qn('#__rseventspro_invoice'))
				->set($db->qn('title').' = '.$db->q($title))
				->set($db->qn('layout').' = '.$db->q($layout))
				->where($db->qn('id').' = '.$db->q($id));
		} else {
			$query->clear()
				->insert($db->qn('#__rseventspro_invoice'))
				->set($db->qn('layout').' = '.$db->q($layout))
				->set($db->qn('title').' = '.$db->q($title))
				->set($db->qn('lang').' = '.$db->q($language))
				->set($db->qn('ide').' = '.$db->q($ide));
		}
		
		$db->setQuery($query);
		$db->execute();
		
		return true;
	}
	
	public function getFeedback($id) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery();
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_feedbacks'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$feedback = $db->loadObject();
		
		// Replace translations
		if ($translation = RSEventsProTranslations::getTranslation('feedback', $feedback->id, 'question', RSEventsProTranslations::getLanguage('event', $feedback->ide))) {
			$feedback->question = $translation;
		}
		
		return $feedback;
	}
	
	public function saveFeedback($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$data	= (object) $data;
		
		if ($data->id) {
			$query->clear()
				->select($db->qn('language'))
				->from($db->qn('#__rseventspro_feedbacks'))
				->where($db->qn('id').' = '.$db->q($data->id));
			$db->setQuery($query);
			$data->language = $db->loadResult();
			
			$lang = RSEventsProTranslations::getLanguage('event', $data->ide);
			RSEventsProTranslations::saveTranslations($data, 'feedback', $lang);
			
			$db->updateObject('#__rseventspro_feedbacks', $data, 'id');
		} else {
			$query->clear()
				->select('MAX('.$db->qn('ordering').')')
				->from($db->qn('#__rseventspro_feedbacks'))
				->where($db->qn('ide').' = '.$db->q($data->ide));

			$db->setQuery($query);
			$max = (int) $db->loadResult();
			$data->ordering = $max + 1;
			
			$data->language = Factory::getLanguage()->getTag();
			
			$db->insertObject('#__rseventspro_feedbacks', $data, 'id');
		}
		
		return $data;
	}
	
	public function removeFeedback($id) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete($db->qn('#__rseventspro_feedbacks'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	public function saveOrderFeedbackAjax() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$input	= Factory::getApplication()->input;
		$cids 	= $input->get('cid', array(),'array');
		$order 	= $input->get('order',array(),'array');

		for ($i=0; $i < count($cids); $i++) {
			if (is_numeric($cids[$i])) {
				$query->clear()
					->select($db->qn('ordering'))
					->from($db->qn('#__rseventspro_feedbacks'))
					->where($db->qn('id').' = '.$db->q($cids[$i]));
				$db->setQuery($query);
				$ordering = $db->loadResult();

				// Set the new ordering only if different
				if ($ordering != $order[$i]) {
					$query->clear()
						->update($db->qn('#__rseventspro_feedbacks'))
						->set($db->qn('ordering').' = '.$db->q($order[$i]))
						->where($db->qn('id').' = '.$db->q($cids[$i]));
					$db->setQuery($query);
					$db->execute();
				}
			}
		}
	}
	
	public function getFaq($id) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery();
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_faq'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$faq = $db->loadObject();
		
		// Replace translations
		if ($translations = RSEventsProTranslations::getTranslations('faq', $faq->id, RSEventsProTranslations::getLanguage('event', $faq->ide))) {
			foreach ($translations as $field => $translation) {
				if (isset($faq->$field)) {
					$faq->$field = $translation;
				}
			}
		}
		
		return $faq;
	}
	
	public function saveFaq($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$data	= (object) $data;
		
		if ($data->id) {
			$query->clear()
				->select($db->qn('language'))
				->from($db->qn('#__rseventspro_faq'))
				->where($db->qn('id').' = '.$db->q($data->id));
			$db->setQuery($query);
			$data->language = $db->loadResult();
			
			$lang = RSEventsProTranslations::getLanguage('event', $data->ide);
			RSEventsProTranslations::saveTranslations($data, 'faq', $lang);
			
			$db->updateObject('#__rseventspro_faq', $data, 'id');
		} else {
			$query->clear()
				->select('MAX('.$db->qn('ordering').')')
				->from($db->qn('#__rseventspro_faq'))
				->where($db->qn('ide').' = '.$db->q($data->ide));

			$db->setQuery($query);
			$max = (int) $db->loadResult();
			$data->ordering = $max + 1;
			
			$data->language = Factory::getLanguage()->getTag();
			
			$db->insertObject('#__rseventspro_faq', $data, 'id');
		}
		
		return $data;
	}
	
	public function removeFaq($id) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete($db->qn('#__rseventspro_faq'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	public function saveOrderFaqAjax() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$input	= Factory::getApplication()->input;
		$cids 	= $input->get('cid', array(),'array');
		$order 	= $input->get('order',array(),'array');

		for ($i=0; $i < count($cids); $i++) {
			if (is_numeric($cids[$i])) {
				$query->clear()
					->select($db->qn('ordering'))
					->from($db->qn('#__rseventspro_faq'))
					->where($db->qn('id').' = '.$db->q($cids[$i]));
				$db->setQuery($query);
				$ordering = $db->loadResult();

				// Set the new ordering only if different
				if ($ordering != $order[$i]) {
					$query->clear()
						->update($db->qn('#__rseventspro_faq'))
						->set($db->qn('ordering').' = '.$db->q($order[$i]))
						->where($db->qn('id').' = '.$db->q($cids[$i]));
					$db->setQuery($query);
					$db->execute();
				}
			}
		}
	}
	
	public function getItinerary($id) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery();
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_itinerary'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$itinerary = $db->loadObject();
		
		// Replace translations
		if ($translations = RSEventsProTranslations::getTranslations('itinerary', $itinerary->id, RSEventsProTranslations::getLanguage('event', $itinerary->ide))) {
			foreach ($translations as $field => $translation) {
				if (isset($itinerary->$field)) {
					$itinerary->$field = $translation;
				}
			}
		}
		
		return $itinerary;
	}
	
	public function saveItinerary($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$data	= (object) $data;
		
		if ($data->id) {
			$query->clear()
				->select($db->qn('language'))
				->from($db->qn('#__rseventspro_itinerary'))
				->where($db->qn('id').' = '.$db->q($data->id));
			$db->setQuery($query);
			$data->language = $db->loadResult();
			
			$lang = RSEventsProTranslations::getLanguage('event', $data->ide);
			RSEventsProTranslations::saveTranslations($data, 'itinerary', $lang);
			
			$db->updateObject('#__rseventspro_itinerary', $data, 'id');
		} else {
			$data->language = Factory::getLanguage()->getTag();
			$db->insertObject('#__rseventspro_itinerary', $data, 'id');
			$data->pill = RSEventsproAdapterItinerary::addPill($data->id, $data->name);
			$data->content = RSEventsproAdapterItinerary::addPillContent($data->id, '');
		}
		
		return $data;
	}
	
	public function removeItinerary($id) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete($db->qn('#__rseventspro_itinerary'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
		
		$query->clear()
			->delete($db->qn('#__rseventspro_itinerary_timeline'))
			->where($db->qn('idi').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	public function getTimeline($id) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery();
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_itinerary_timeline'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$timeline = $db->loadObject();
		
		// Get event ID
		$eventID = Factory::getApplication()->input->getInt('ide', 0);
		
		// Replace translations
		if ($translations = RSEventsProTranslations::getTranslations('timeline', $timeline->id, RSEventsProTranslations::getLanguage('event', $eventID))) {
			foreach ($translations as $field => $translation) {
				if (isset($timeline->$field)) {
					$timeline->$field = $translation;
				}
			}
		}
		
		return $timeline;
	}
	
	public function saveTimeline($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$data	= (object) $data;
		
		if ($data->id) {
			$query->clear()
				->select($db->qn('language'))
				->from($db->qn('#__rseventspro_itinerary_timeline'))
				->where($db->qn('id').' = '.$db->q($data->id));
			$db->setQuery($query);
			$data->language = $db->loadResult();
			
			// Get event ID
			$eventID = Factory::getApplication()->input->getInt('ide', 0);
			
			$lang = RSEventsProTranslations::getLanguage('event', $eventID);
			RSEventsProTranslations::saveTranslations($data, 'timeline', $lang);
			
			$db->updateObject('#__rseventspro_itinerary_timeline', $data, 'id');
		} else {
			$query->clear()
				->select('MAX('.$db->qn('ordering').')')
				->from($db->qn('#__rseventspro_itinerary_timeline'))
				->where($db->qn('idi').' = '.$db->q($data->idi));

			$db->setQuery($query);
			$max = (int) $db->loadResult();
			$data->ordering = $max + 1;
			
			$data->language = Factory::getLanguage()->getTag();
			
			$db->insertObject('#__rseventspro_itinerary_timeline', $data, 'id');
		}
		
		// Get what is already saved in the database for the base language
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_itinerary_timeline'))
			->where($db->qn('id').' = '.$db->q($data->id));
		$db->setQuery($query);
		$timeline = $db->loadObject();
		
		$data->timeline = RSEventsproAdapterItinerary::addTimeline($timeline);
		
		return $data;
	}
	
	public function removeTimeline($id) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->delete($db->qn('#__rseventspro_itinerary_timeline'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	public function saveOrderTimelineAjax() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$input	= Factory::getApplication()->input;
		$id		= $input->getInt('id',0);
		$items	= $input->get('items', array(), 'array');
		
		foreach ($items as $i => $item) {
			$query->clear()
				->update($db->qn('#__rseventspro_itinerary_timeline'))
				->set($db->qn('ordering').' = '.$db->q($i))
				->where($db->qn('id').' = '.$db->q($item))
				->where($db->qn('idi').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}
	}

	public function saveNew($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$input	= Factory::getApplication()->input;
		$start  = $data['start'];
		$end    = $data['end'];

		$tzoffset   = Factory::getConfig()->get('offset');
		$categories = isset($data['categories']) ? $data['categories'] : array();
		$tags       = isset($data['tags']) ? $data['tags'] : array();
		$allday     = isset($data['allday']) ? $data['allday'] : 0;

		if ($data['registration_type'] == 1) {
			$data['registration'] = 1;
		} elseif ($data['registration_type'] == 2) {
			$data['rsvp'] = 1;
		}

		unset($data['categories'], $data['tags'], $data['registration_type']);

		if ($allday) {
			$start = Factory::getDate($start, $tzoffset);
			$start->setTimezone(new DateTimezone('UTC'));
			$startDate = $start->toSql();
			$endDate   = $db->getNullDate();
		} else {
			$start = Factory::getDate($start, $tzoffset);
			$start->setTimezone(new DateTimezone('UTC'));

			if ($start->format('I')) {
				$start->modify('-1 hours');
			}

			$startDate = $start->toSql();

			$end = Factory::getDate($end, $tzoffset);
			$end->setTimezone(new DateTimezone('UTC'));

			if ($start > $end) {
				$end->modify('+2 hours');
			}

			if ($end->format('I')) {
				$end->modify('-1 hours');
			}

			$endDate = $end->toSql();
		}

		$table = $this->getTable();
		$table->verify($data);
		$table->bind($data);
		$table->check();
		$table->start = $startDate;
		$table->end = $endDate;
		$table->options = rseventsproHelper::getDefaultOptions();
		$table->completed = 1;
		$table->store();
		$id = $table->id;

		foreach ($categories as $category) {
			$query->clear()
				->insert($db->qn('#__rseventspro_taxonomy'))
				->set($db->qn('type').' = '.$db->q('category'))
				->set($db->qn('ide').' = '.(int) $id)
				->set($db->qn('id').' = '.(int) $category);

			$db->setQuery($query);
			$db->execute();
		}

		foreach ($tags as $tag) {
			$tag = trim($tag);

			$query->clear()
				->select($db->qn('id'))
				->from($db->qn('#__rseventspro_tags'))
				->where($db->qn('name').' = '.$db->q($tag));

			$db->setQuery($query);
			$tid = $db->loadResult();

			if (empty($tid)) {
				$query->clear()
					->insert($db->qn('#__rseventspro_tags'))
					->set($db->qn('name').' = '.$db->q($tag))
					->set($db->qn('published').' = '.$db->q(1));

				$db->setQuery($query);
				$db->execute();
				$tid = $db->insertid();
			}

			$query->clear()
				->insert($db->qn('#__rseventspro_taxonomy'))
				->set($db->qn('type').' = '.$db->q('tag'))
				->set($db->qn('ide').' = '.(int) $id)
				->set($db->qn('id').' = '.(int) $tid);

			$db->setQuery($query);
			$db->execute();
		}

		if ($table->registration) {
			$ticketName = $input->get('ticketName', array(), 'array');
			$ticketPrice  = $input->get('ticketPrice', array(), 'array');
			$ticketQuantity = $input->get('ticketQuantity', array(), 'array');

			if ($ticketName) {
				for($i=0; $i<count($ticketName); $i++) {
					$name = trim($ticketName[$i]);

					if ($name) {
						$price = isset($ticketPrice[$i]) ? $ticketPrice[$i] : 0;
						$quantity = isset($ticketQuantity[$i]) ? $ticketQuantity[$i] : 0;

						$query->clear()
							->insert($db->qn('#__rseventspro_tickets'))
							->set($db->qn('ide').' = '.$db->q($id))
							->set($db->qn('name').' = '.$db->q($name))
							->set($db->qn('description').' = '.$db->q(''))
							->set($db->qn('position').' = '.$db->q(''))
							->set($db->qn('groups').' = '.$db->q(''))
							->set($db->qn('layout').' = '.$db->q(''))
							->set($db->qn('ticketinfolayout').' = '.$db->q(''))
							->set($db->qn('manual_disabled').' = '.$db->q(''))
							->set($db->qn('price').' = '.$db->q((float) $price))
							->set($db->qn('seats').' = '.$db->q((int) $quantity));

						$db->setQuery($query);
						$db->execute();
					}
				}
			}
		}

		return $id;
	}
}