<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license     GNU General Public License version 2 or later; see LICENSE
*/

defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Form\FormHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

FormHelper::loadFieldClass('category');

JLoader::registerAlias('JFormFieldCategory', '\\Joomla\\CMS\\Form\\Field\\CategoryField');

class JFormFieldRSCategories extends JFormFieldCategory
{
	public $type = 'RSCategories';
	
	public function __construct() {
		if (!class_exists('rseventsproHelper')) {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/rseventspro.php';
		}
		
		if (rseventsproHelper::isJ4()) {
			Text::script('JGLOBAL_SELECT_NO_RESULTS_MATCH');
			Text::script('JGLOBAL_SELECT_PRESS_TO_SELECT');

			Factory::getDocument()->getWebAssetManager()->usePreset('choicesjs')->useScript('webcomponent.field-fancy-select');
		}
	}
	
	public function getInput() {
		if (rseventsproHelper::isJ4()) {
			$attr		= '';
			$attr2		= '';
			$options	= $this->getOptions();

			$attr .= !empty($this->element['size']) ? ' size="' . $this->element['size'] . '"' : '';
			$attr .= $this->element['multiple'] ? ' multiple' : '';

			$attr2 .= !empty($this->element['class']) ? ' class="' . $this->element['class'] . '"' : '';
			$attr2 .= ' placeholder="' . Text::_('JGLOBAL_TYPE_OR_SELECT_SOME_OPTIONS',true) . '" ';
			$attr2 .= ' allow-custom';
			
			return '<joomla-field-fancy-select '.trim($attr2).'>'.HTMLHelper::_('select.genericlist', $options, $this->name, trim($attr), 'value', 'text', $this->value, $this->id).'</joomla-field-fancy-select>';
		} else {
			return parent::getInput();
		}
	}
	
	protected function getOptions() {
		$options = parent::getOptions();
		
		if (Factory::getApplication()->isClient('site')) {
			if ($options = rseventsproHelper::getCategoriesOptions()) {
				foreach ($options as $option) {
					// Replace translations
					if ($translation = RSEventsProTranslations::getTranslation('category', $option->value, 'title')) {
						$length = strlen($translation) + 2 * substr_count($option->text, '- ');
						$option->text = str_pad($translation, $length, '- ', STR_PAD_LEFT);
					}
				}
			}
			
			return $options;
		}
		
		return $options;
	}
}