<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license     GNU General Public License version 2 or later; see LICENSE
*/

defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Form\FormHelper;
use Joomla\CMS\HTML\HTMLHelper;

FormHelper::loadFieldClass('list');

JLoader::registerAlias('JFormFieldList', '\\Joomla\\CMS\\Form\\Field\\ListField');

class JFormFieldRSCombobox extends JFormFieldList
{
	public $type = 'RSCombobox';

	protected function getInput() {
		// Initialize variables.
		$html = array();
		$attr = '';

		// Initialize some field attributes.
		$attr .= $this->element['class'] ? ' class="rscombobox ' . (string) $this->element['class'] . '"' : ' class="rscombobox"';
		$attr .= ((string) $this->element['readonly'] == 'true') ? ' readonly="readonly"' : '';
		$attr .= ((string) $this->element['disabled'] == 'true') ? ' disabled="disabled"' : '';
		$attr .= $this->element['size'] ? ' size="' . (int) $this->element['size'] . '"' : '';

		// Initialize JavaScript field attributes.
		$attr .= $this->element['onchange'] ? ' onchange="' . (string) $this->element['onchange'] . '"' : '';

		// Get the field options.
		$options = $this->getOptions();
		
		if (rseventsproHelper::isJ4()) $html[] = '<div id="rsepro-combo-j4">';
		
		// Build the list for the combo box.
		$html[] = '<select size="1" id="rscombobox-' . $this->id . '" class="rscomboboxoptions custom-select" onchange="document.getElementById(\''.$this->id.'\').value = this.value;">';
		foreach ($options as $option) {
			$selected = trim($this->value) == $option->value ? ' selected="selected"' : '';
			$html[] = '<option value="'. $option->value .'"'.$selected.'>' . $option->text . '</option>';
		}
		$html[] = '</select>';
		
		// Build the input for the combo box.
		$html[] = '<input type="text" name="' . $this->name . '" id="' . $this->id . '"' . ' value="'. htmlspecialchars($this->value, ENT_COMPAT, 'UTF-8') . '"' . $attr . '/>';
		if (rseventsproHelper::isJ4()) $html[] = '</div>';

		return implode($html);
	}
	
	protected function getOptions() {
		$found	 = false;
		$options = parent::getOptions();
		
		foreach ($options as $option) {
			if (trim($this->value) == $option->value) {
				$found = true;
			}
		}
		
		if (!$found) {
			array_unshift($options, HTMLHelper::_('select.option', $this->value, $this->value));
		}
		
		return $options;
	}
}
