<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license     GNU General Public License version 2 or later; see LICENSE
*/

defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Form\FormHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

FormHelper::loadFieldClass('list');

JLoader::registerAlias('JFormFieldList', '\\Joomla\\CMS\\Form\\Field\\ListField');

class JFormFieldRSResolution extends JFormFieldList
{
	public $type = 'RSResolution';

	public function __construct($parent = null) {
		static $added;
		parent::__construct($parent);
		
		if (!$added) {
			// add javascript
			Factory::getDocument()->addScriptDeclaration("function rsepro_change_other(id, value) {
				var value = value == 'h' ? '".Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_WIDTH', true)."' : '".Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_HEIGHT', true)."';
				document.getElementById('rsepro_other_' + id).innerHTML = value;
			}");
			
			$added = true;
		}
	}
	
	public function getInput() {
		$html 		= array();
		
		$name  		= $this->name;
		$fieldname 	= $this->fieldname;
		$value 		= $this->value;
		
		$size 		= isset($this->element['size']) ? 'size="'.$this->element['size'].'"' : '';
		$value		= is_array($value) ? $value : explode(',', $value);		
		if (!isset($value[1])) $value[1] = '0';
		
		$options = array(
			HTMLHelper::_('select.option', 'w', Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_WIDTH')),
			HTMLHelper::_('select.option', 'h', Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_HEIGHT'))
		);
		
		$select = HTMLHelper::_('select.genericlist', $options, $name.'[]', 'onchange="rsepro_change_other(\''.addslashes($fieldname).'\', this.value)" class="custom-select"', 'value', 'text', $value[0], $fieldname.'_w');
		$input	= '<input class="form-control" type="text" name="'.$name.'[]" id="'.$fieldname.'_res" value="'.(int) $value[1].'" '.$size.' />';
		$other	= $value[0] == 'h' ? Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_WIDTH') : Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_HEIGHT');
		
		$words = explode(' ', Text::_('COM_RSEVENTSPRO_GALLERY_PARAM_SIZE_ADJUST'));
		
		foreach ($words as $word) {
			// found replacement, wrap it in a <td>
			if ($word[0] == '%') {
				if (strstr($word, '%3$s'))
					$html[] = '<span'.(strstr($word, '%3$s') ? ' id="rsepro_other_'.$fieldname.'"' : '').' class="rsextra">'.$word.'</span>';
				else
					$html[] = $word;
			}
			else
				$html[] = '<span class="rsextra">'.$word.'</span>';
		}
		
		return sprintf(implode("\r\n", $html), $select, $input, $other);
	}
}