<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Factory;

class RseventsproModelGroup extends AdminModel
{
	protected $text_prefix = 'COM_RSEVENTSPRO';

	public function getTable($type = 'Group', $prefix = 'RseventsproTable', $config = array()) {
		return Table::getInstance($type, $prefix, $config);
	}

	public function getItem($pk = null) {
		if ($item = parent::getItem($pk)) {
			// Convert the Joomla groups field to an array.
			try {
				$registry = new Registry;
				$registry->loadString($item->jgroups ? $item->jgroups : '');
				$item->jgroups = $registry->toArray();
			} catch (Exception $e) {
				$item->jgroups = array();
			}
			
			// Convert the Joomla users field to an array.
			try {
				$registry = new Registry;
				$registry->loadString($item->jusers ? $item->jusers : '');
				$item->jusers = $registry->toArray();
			} catch (Exception $e) {
				$item->jusers = array();
			}
			
			// Convert the event options.
			try {
				$registry = new Registry;
				$registry->loadString($item->event ? $item->event : '');
				$item->event = $registry->toArray();
			} catch (Exception $e) {
				$item->event = array();
			}
			
			// Convert the restricted categories.
			try {
				$registry = new Registry;
				$registry->loadString($item->restricted_categories ? $item->restricted_categories : '');
				$item->restricted_categories = $registry->toArray();
			} catch (Exception $e) {
				$item->restricted_categories = array();
			}
			
			// Convert the default categories.
			try {
				$registry = new Registry;
				$registry->loadString($item->default_category ? $item->default_category : '');
				$item->default_category = $registry->toArray();
			} catch (Exception $e) {
				$item->default_category = array();
			}
		}
		
		return $item;
	}

	public function getForm($data = array(), $loadData = true) {
		// Get the form.
		$form = $this->loadForm('com_rseventspro.group', 'group', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		if (rseventsproHelper::isJ4()) {
			$form->setFieldAttribute('jgroups','layout','joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('restricted_categories','layout','joomla.form.field.list-fancy-select');
		}
		
		if ($used = $this->getUsed()) {
			$form->setFieldAttribute('jgroups', 'used', $used);
		}
		
		return $form;
	}

	protected function loadFormData() {
		// Check the session for previously entered form data.
		$data = Factory::getApplication()->getUserState('com_rseventspro.edit.group.data', array());

		if (empty($data))
			$data = $this->getItem();

		return $data;
	}

	public function getUsed() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$used = array();
		$jinput = Factory::getApplication()->input;
		
		$query->clear();
		$query->select($db->qn('jgroups'))
			->from($db->qn('#__rseventspro_groups'))
			->where($db->qn('jgroups').' <> '.$db->q(''))
			->where($db->qn('id').' <> '.$db->q($jinput->getInt('id',0)));
		
		$db->setQuery($query);
		if ($options = $db->loadColumn()) {
			foreach ($options as $option) {
				try {
					$registry = new Registry;
					$registry->loadString($option);
					$option = $registry->toArray();
				} catch (Exception $e) {
					$option = array();
				}
				
				$option = array_map('intval',$option);
				$used = array_merge($used, $option);
			}
		}
		
		$used = array_unique($used);
		return !empty($used) ? implode(',',$used) : '';
	}

	public function getTabs() {
		$tabs = new RSEventsproAdapterTabs('groups');
		return $tabs;
	}
}