<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Application\ApplicationHelper;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\String\StringHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\Filesystem\File;
use Joomla\CMS\Factory;

class RseventsproModelImports extends BaseDatabaseModel
{
	protected $_tz = 0;

	public function __construct() {
		parent::__construct();
		$this->_tz = Factory::getApplication()->input->getInt('offset',0) * 3600;
	}

	public function getItems() {
		$db = Factory::getDbo();
		
		$items = array('rsevents' => false, 'jevents' => false, 'jcalpro' => false,
			'ohanah' => false, 'eventlist' => false, 'eventlistbeta' => false);
		
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_rsevents/rsevents.php'))
			$items['rsevents'] = true;
		
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_jevents/jevents.php'))
			$items['jevents'] = true;
		
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_jcalpro/jcalpro.xml')) {
			$xml = file_get_contents(JPATH_ADMINISTRATOR.'/components/com_jcalpro/jcalpro.xml');
			
			if ($this->checkVersion($xml, '4.3.18', '<=')) {
				$items['jcalpro'] = true;
			}
		}
		
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_ohanah/ohanah.php'))
			$items['ohanah'] = true;
		
		$beta = false;
		$elist = file_exists(JPATH_ADMINISTRATOR.'/components/com_eventlist/admin.eventlist.php');
		if ($elist) {
			$db->setQuery("DESCRIBE #__eventlist_cats_event_relations");
			$beta = $db->loadObjectList();
		}
		
		if ($elist && empty($beta))
			$items['eventlist'] = true;
		
		if ($elist && !empty($beta))
			$items['eventlistbeta'] = true;
		
		return $items;
	}
	
	/*
	 *	Method to get the locations list
	 */
	public function getLocations() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$default = array(HTMLHelper::_('select.option', 0, Text::_('COM_RSEVENTSPRO_IMPORT_LOCATION_DEFAULT')));
		
		$query->clear()
			->select($db->qn('id','value'))
			->select($db->qn('name','text'))
			->from($db->qn('#__rseventspro_locations'))
			->where($db->qn('published').' = 1');
		$db->setQuery($query);
		if ($locations = $db->loadObjectList()) {
			return array_merge($default, $locations);
		}
		
		return $default;
	}
	
	/**
	 * Method to get time offsets.
	 */
	public function getOffsets() {
		return array(
			HTMLHelper::_('select.option', -12, '-12:00'),
			HTMLHelper::_('select.option', -11, '-11:00'),
			HTMLHelper::_('select.option', -10, '-10:00'),
			HTMLHelper::_('select.option', -9.5, '-09:30'),
			HTMLHelper::_('select.option', -9, '-09:00'),
			HTMLHelper::_('select.option', -8, '-08:00'),
			HTMLHelper::_('select.option', -7, '-07:00'),
			HTMLHelper::_('select.option', -6, '-06:00'),
			HTMLHelper::_('select.option', -5, '-05:00'),
			HTMLHelper::_('select.option', -4.5, '-04:30'),
			HTMLHelper::_('select.option', -4, '-04:00'),
			HTMLHelper::_('select.option', -3.5, '-03:30'),
			HTMLHelper::_('select.option', -3, '-03:00'),
			HTMLHelper::_('select.option', -2, '-02:00'),
			HTMLHelper::_('select.option', -1, '-01:00'),
			HTMLHelper::_('select.option', 0, '00:00'),
			HTMLHelper::_('select.option', 1, '+01:00'),
			HTMLHelper::_('select.option', 2, '+02:00'),
			HTMLHelper::_('select.option', 3, '+03:00'),
			HTMLHelper::_('select.option', 3.5, '+03:30'),
			HTMLHelper::_('select.option', 4, '+04:00'),
			HTMLHelper::_('select.option', 4.5, '+04:30'),
			HTMLHelper::_('select.option', 5, '+05:00'),
			HTMLHelper::_('select.option', 5.5, '+05:30'),
			HTMLHelper::_('select.option', 5.75, '+05:45'),
			HTMLHelper::_('select.option', 6, '+06:00'),
			HTMLHelper::_('select.option', 6.5, '+06:30'),
			HTMLHelper::_('select.option', 7, '+07:00'),
			HTMLHelper::_('select.option', 8, '+08:00'),
			HTMLHelper::_('select.option', 8.75, '+08:00'),
			HTMLHelper::_('select.option', 9, '+09:00'),
			HTMLHelper::_('select.option', 9.5, '+09:30'),
			HTMLHelper::_('select.option', 10, '+10:00'),
			HTMLHelper::_('select.option', 10.5, '+10:30'),
			HTMLHelper::_('select.option', 11, '+11:00'),
			HTMLHelper::_('select.option', 11.5, '+11:30'),
			HTMLHelper::_('select.option', 12, '+12:00'),
			HTMLHelper::_('select.option', 12.75, '+12:45'),
			HTMLHelper::_('select.option', 13, '+13:00'),
			HTMLHelper::_('select.option', 14, '+14:00')
		);
	}
	
	public function jevents() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		// Get categories
		// Get all categories used in events
		$query->clear()
			->select($db->qn('catid'))
			->from($db->qn('#__jevents_vevent'));
		
		$db->setQuery($query);
		$jevcategories = $db->loadColumn();			
		
		$condition = false;
		if (!empty($jevcategories)) {
			$jevcategories = array_map('intval',$jevcategories);
			$jevcategories = array_unique($jevcategories);
			$condition = " AND id IN (".implode(',',$jevcategories).") ";
		}
		
		$query->clear()
			->select($db->qn('id'))->select($db->qn('title'))->select($db->qn('description'))
			->select($db->qn('published'))
			->from($db->qn('#__categories'))
			->where($db->qn('extension').' = '.$db->q('com_jevents'));
			
		if ($condition)
			$query->where($db->qn('id').' IN ('.implode(',',$jevcategories).')');
		
		$db->setQuery($query);
		$jecategories = $db->loadObjectList();
		
		if (empty($jecategories)) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
		}
		
		foreach ($jecategories as $i => $jecategory) {
			$registry = new Registry;
			$registry->loadString(isset($jecategory->params) ? $jecategory->params : '');
			$jecategories[$i]->color = $registry->get('catcolour','');
		}
		
		// Add the default JEvents location
		$query->clear()
			->insert($db->qn('#__rseventspro_locations'))
			->set($db->qn('name').' = '.$db->q('JEvents'))
			->set($db->qn('published').' = 1');
		
		$db->setQuery($query);
		$db->execute();
		$idlocation = $db->insertid();
		
		// Prepare events
		if (!empty($jecategories)) {
			$events = array();
			
			foreach ($jecategories as $category) {
				// Check if the category has events
				$query->clear()
					->select('COUNT(ev_id)')
					->from($db->qn('#__jevents_vevent'))
					->where($db->qn('catid').' = '.(int) $category->id);
				
				$db->setQuery($query);
				$count = $db->loadResult();
				
				if (!$count) continue;
				
				// Create the category
				$data = array();
				$data['published'] = $category->published;
				$data['title'] = $category->title;
				$data['description'] = $category->description;
				$data['parent_id'] = 1;
				$registry = new Registry;
				$registry->loadArray(array('color' => $category->color));
				$data['params'] = $registry->toString();
				
				$category_id = $this->_savecategory($data);
				
				// Get the event details
				$query->clear()
					->select($db->qn('ed.dtstart'))->select($db->qn('ed.dtend'))->select($db->qn('ed.description'))->select($db->qn('ed.summary'))
					->select($db->qn('ed.state'))->select($db->qn('e.created_by'))
					->from($db->qn('#__jevents_vevent','e'))
					->join('left', $db->qn('#__jevents_vevdetail','ed').' ON '.$db->qn('ed.evdet_id').' = '.$db->qn('e.ev_id'))
					->where($db->qn('e.catid').' = '.(int) $category->id);
				
				$db->setQuery($query);
				$jevents = $db->loadObjectList();
				
				// Add JEvents events to the $events container
				foreach ($jevents as $event) {
					$eventcontainer = new stdClass();
					$eventcontainer->name = $event->summary;
					$eventcontainer->description = $event->description;
					$eventcontainer->location = $idlocation;
					$eventcontainer->start = $event->dtstart;
					$eventcontainer->end = $event->dtend;
					$eventcontainer->published = $event->state;
					$eventcontainer->owner = $event->created_by;
					
					$events[$category_id][] = $eventcontainer;
				}
			}
		}
		
		$counter = 0;
		
		// Parse JEvents events
		if (!empty($events)) {
			foreach ($events as $category => $event) {
				foreach ($event as $jevent) {
					$start = $jevent->start + $this->_tz;
					$end = $jevent->end + $this->_tz;
					
					$data = array();
					$data['name'] = $jevent->name;
					$data['description'] = $jevent->description;
					$data['location'] = $jevent->location;
					$data['start'] = Factory::getDate($start)->toSql();
					$data['end'] = Factory::getDate($end)->toSql();
					$data['published'] = $jevent->published;
					$data['owner'] = $jevent->owner;
					$data['completed'] = 1;
					
					if ($idevent = $this->_saveevent($data)) {
						$query->clear()
							->insert($db->qn('#__rseventspro_taxonomy'))
							->set($db->qn('type').' = '.$db->q('category'))
							->set($db->qn('id').' = '.(int) $category)
							->set($db->qn('ide').' = '.(int) $idevent);
						
						$db->setQuery($query);
						$db->execute();
						
						$counter++;
					}
				}
			}
		}
		
		if ($counter) {
			return $counter;
		}

		throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
	}
	
	public function jcalpro() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		$tmplocations 	= array();
		$tmpcategories 	= array();
		$counter		= 0;
		
		// Check for events
		$query->clear()
			->select($db->qn('id'))->select($db->qn('title'))->select($db->qn('description'))->select($db->qn('start_date'))->select($db->qn('duration_type'))
			->select($db->qn('location'))->select($db->qn('end_date'))->select($db->qn('published'))->select($db->qn('created_by'))
			->from($db->qn('#__jcalpro_events'));
		
		$db->setQuery($query);
		if ($events = $db->loadObjectList()) {
			$eIDs = array();
			$lIDs = array();
			foreach ($events as $event) {
				$eIDs[] = $event->id;
				$lIDs[] = $event->location;
			}
			
			// Get available location IDs
			$lIDs = array_map('intval', $lIDs);
			$lIDs = array_unique($lIDs);
			
			// Get available event IDs
			$eIDs = array_map('intval', $eIDs);
			$eIDs = array_unique($eIDs);
			
			// Get locations
			$query->clear()
				->select($db->qn('id'))->select($db->qn('title'))->select($db->qn('address'))->select($db->qn('city'))->select($db->qn('state'))
				->select($db->qn('postal_code'))->select($db->qn('country'))->select($db->qn('latitude'))->select($db->qn('longitude'))->select($db->qn('published'))
				->from($db->qn('#__jcalpro_locations'))
				->where($db->qn('id').' IN ('.implode(',',$lIDs).')');
			$db->setQuery($query);
			$locations = $db->loadObjectList();
			
			// Get available categories
			$query->clear()
				->select($db->qn('c.id'))->select($db->qn('c.title'))
				->select($db->qn('c.description'))->select($db->qn('c.published'))
				->from($db->qn('#__categories','c'))
				->join('LEFT', $db->qn('#__jcalpro_event_categories','jc').' ON '.$db->qn('c.id').' = '.$db->qn('jc.category_id'))
				->where($db->qn('c.extension').' = '.$db->q('com_jcalpro'))
				->where($db->qn('jc.event_id').' IN ('.implode(',', $eIDs).')')
				->group($db->qn('c.id'));
			$db->setQuery($query);
			$categories = $db->loadObjectList();
			
			// Store the new categories and locations
			if ($locations) {
				foreach ($locations as $location) {
					$address = $location->address;
					if (!empty($location->postal_code)) $address .= ', '.$location->postal_code;
					if (!empty($location->city)) $address .= ', '.$location->city;
					if (!empty($location->state)) $address .= ', '.$location->state;
					if (!empty($location->country)) $address .= ', '.$location->country;
					
					$data = array();
					$data['name'] = $location->title;
					$data['address'] = $address;
					$data['published'] = $location->published;
					
					if ($location->latitude && $location->longitude) {
						try {
							$data['coordinates'] = rseventsproHelper::checkCoordinates($location->latitude.','.$location->longitude);
						} catch (Exception $e) {
							$data['coordinates'] = '';
						}
					}
					
					$newlocation = $this->_savelocation($data);
					$tmplocations[$location->id] = $newlocation;
				}
			}
			
			if ($categories) {
				foreach ($categories as $category) {
					$data = array();
					$data['published'] = $category->published;
					$data['title'] = $category->title;
					$data['description'] = $category->description;
					$data['parent_id'] = 1;
					
					$newcategory = $this->_savecategory($data);
					$tmpcategories[$category->id] = $newcategory;
				}
			}
			
			// Add events
			foreach ($events as $event) {
				$array = array();
				$query->clear()
					->select($db->qn('category_id'))
					->from($db->qn('#__jcalpro_event_categories'))
					->where($db->qn('event_id').' = '.(int) $event->id);
				
				$db->setQuery($query);
				$ecategories = $db->loadColumn();
				$ecategories = array_map('intval',$ecategories);
				
				if (!empty($ecategories)) {
					foreach ($ecategories as $cat) {
						$array[] = $tmpcategories[$cat];
					}
				}
				
				$data = array();
				$data['name'] = $event->title;
				$data['description'] = $event->description;
				$data['location'] = $tmplocations[$event->location];
				$data['start'] = $event->start_date;
				$data['end'] = $event->end_date;
				$data['published'] = $event->published;
				$data['owner'] = $event->created_by;
				$data['completed'] = 1;
				
				if ($event->duration_type == 0 || $event->duration_type == 2) {
					$data['allday'] = 1;
					$data['end'] = $db->getNullDate();
				}
				
				if ($idevent = $this->_saveevent($data)) {
					foreach ($array as $cat) {
						$query->clear()
							->insert($db->qn('#__rseventspro_taxonomy'))
							->set($db->qn('type').' = '.$db->q('category'))
							->set($db->qn('id').' = '.(int) $cat)
							->set($db->qn('ide').' = '.(int) $idevent);
						
						$db->setQuery($query);
						$db->execute();
					}
					
					$counter++;
				}
			}
		} else {
			throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
		}
		
		if ($counter) {
			return $counter;
		}

		throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
	}
	
	public function eventlistbeta() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		// Get events
		$query->clear()
			->select($db->qn('id'))->select($db->qn('locid'))->select($db->qn('dates'))
			->select($db->qn('enddates'))->select($db->qn('times'))->select($db->qn('endtimes'))->select($db->qn('title'))
			->select($db->qn('created_by'))->select($db->qn('datdescription'))->select($db->qn('published'))
			->from($db->qn('#__eventlist_events'));
		
		
		$db->setQuery($query);
		$eevents = $db->loadObjectList();
		
		if (empty($eevents)) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
			return false;
		}
		
		// Create a default location for events that don't have a location
		$query->clear()
			->select('COUNT(id)')
			->from('#__eventlist_events')
			->where('locid'.' = 0');
		
		$db->setQuery($query);
		if ($db->loadResult()) {
			$query->clear()
				->insert($db->qn('#__rseventspro_locations'))
				->set($db->qn('name').' = '.$db->q('Eventlist'))
				->set($db->qn('published').' = '.$db->q('1'));
			
			$db->setQuery($query);
			$db->execute();
			$default_location = $db->insertid();
		}
		
		$thelocations = array();
		$query->clear()
			->select($db->qn('locid'))
			->from($db->qn('#__eventlist_events'));
		
		$db->setQuery($query);
		$locations = $db->loadColumn();
		
		if (!empty($locations)) {
			$locations = array_map('intval',$locations);
			$locations = array_unique($locations);
			
			foreach ($locations as $location) {
				$query->clear()
					->select($db->qn('venue'))->select($db->qn('url'))->select($db->qn('street'))->select($db->qn('plz'))
					->select($db->qn('city'))->select($db->qn('state'))->select($db->qn('country'))->select($db->qn('locdescription'))
					->select($db->qn('published'))
					->from($db->qn('#__eventlist_venues'))
					->where($db->qn('id').' = '.(int) $location);
				
				$db->setQuery($query);
				$elocation = $db->loadObject();
				
				$address = $elocation->street;
				if (!empty($elocation->plz)) $address .= ' , '.$elocation->plz;
				if (!empty($elocation->city)) $address .= ' , '.$elocation->city;
				if (!empty($elocation->state)) $address .= ' , '.$elocation->state;
				if (!empty($elocation->country)) $address .= ' , '.$elocation->country;
				
				$data = array();
				$data['name'] = $elocation->venue;
				$data['url'] = $elocation->url;
				$data['address'] = $address;
				$data['description'] = $elocation->locdescription;
				$data['published'] = $elocation->published;
				
				$newlocation = $this->_savelocation($data);
				$thelocations[$location] = $newlocation;
			}
		}
		
		$thecategories = array();
		$query->clear()
			->select($db->qn('catid'))
			->from($db->qn('#__eventlist_cats_event_relations'));
			
		$db->setQuery($query);
		$categories = $db->loadColumn();
		
		if (!empty($categories)) {
			$categories = array_map('intval',$categories);
			$categories = array_unique($categories);
			
			foreach ($categories as $category) {
				$query->clear()
					->select($db->qn('catname'))->select($db->qn('catdescription'))->select($db->qn('published'))->select($db->qn('color'))
					->from($db->qn('#__eventlist_categories'))
					->where($db->qn('id').' = '.(int) $category);
				
				$db->setQuery($query);
				$evcategory = $db->loadObject();
				
				$data = array();
				$data['published'] = $evcategory->published;
				$data['title'] = $evcategory->catname;
				$data['description'] = $evcategory->catdescription;
				$data['parent_id'] = 1;
				$registry = new Registry;
				$registry->loadArray(array('color' => $evcategory->color));
				$data['params'] = $registry->toString();
				
				$newcategory = $this->_savecategory($data);
				$thecategories[$category] = $newcategory;
			}
		}
		
		$container = array();
		
		if (!empty($eevents)) {
			foreach ($eevents as $event) {				
				$array = array();
				$query->clear()
					->select($db->qn('catid'))
					->from($db->qn('#__eventlist_cats_event_relations'))
					->where($db->qn('itemid').' = '.(int) $event->id);
				
				$db->setQuery($query);
				$ecategories = $db->loadColumn();
				$ecategories = array_map('intval',$ecategories);
				
				if (!empty($ecategories))
					foreach ($ecategories as $cat)
						$array[] = $thecategories[$cat];
				
				
				$eventcontainer = new stdClass();
				$eventcontainer->name = $event->title;
				$eventcontainer->description = $event->datdescription;
				$eventcontainer->location = $event->locid == 0 ? $default_location : $thelocations[$event->locid];
				$eventcontainer->oldcategories = $array;
				$eventcontainer->start = $event->dates.' '.(!is_null($event->times) ? $event->times : '00:00:00');
				$eventcontainer->end = $event->enddates.' '.(!is_null($event->endtimes) ? $event->endtimes : '00:00:00') ;
				$eventcontainer->published = $event->published;
				$eventcontainer->owner = $event->created_by;
				
				$container[] = $eventcontainer;
			}
		}
		
		$counter = 0;
		
		// Parse Eventlist events
		if (!empty($container)) {
			foreach ($container as $event) {
				$data = array();
				$data['name'] = $event->name;
				$data['description'] = $event->description;
				$data['location'] = $event->location;
				$data['start'] = $event->start;
				$data['end'] = $event->end;
				$data['published'] = $event->published;
				$data['owner'] = $event->owner;
				$data['completed'] = 1;
				
				if ($idevent = $this->_saveevent($data)) {
					$categories = $event->oldcategories;
					if (!empty($categories)) {
						foreach ($categories as $category) {
							$query->clear()
							->insert($db->qn('#__rseventspro_taxonomy'))
							->set($db->qn('type').' = '.$db->q('category'))
							->set($db->qn('id').' = '.(int) $category)
							->set($db->qn('ide').' = '.(int) $idevent);
						
							$db->setQuery($query);
							$db->execute();
						}
					}
					
					$counter++;
				}
			}
		}
		
		if ($counter)
			return $counter;

		throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
	}
	
	public function eventlist() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		// Get events
		$query->clear()
			->select($db->qn('id'))->select($db->qn('locid'))->select($db->qn('catsid'))->select($db->qn('dates'))
			->select($db->qn('enddates'))->select($db->qn('times'))->select($db->qn('endtimes'))->select($db->qn('title'))
			->select($db->qn('created_by'))->select($db->qn('datdescription'))->select($db->qn('published'))
			->from($db->qn('#__eventlist_events'));
		
		$db->setQuery($query);
		$eevents = $db->loadObjectList();
		
		if (empty($eevents)) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
		}
		
		// Create a default location for events that don't have a location
		$query->clear()
			->select('COUNT(id)')
			->from('#__eventlist_events')
			->where('locid'.' = 0');
		
		$db->setQuery($query);
		if ($db->loadResult()) {
			$query->clear()
				->insert($db->qn('#__rseventspro_locations'))
				->set($db->qn('name').' = '.$db->q('Eventlist'))
				->set($db->qn('published').' = '.$db->q('1'));
			
			$db->setQuery($query);
			$db->execute();
			$default_location = $db->insertid();
		}
		
		$thelocations = array();
		$query->clear()
			->select($db->qn('locid'))
			->from($db->qn('#__eventlist_events'));
			
		$db->setQuery($query);
		$locations = $db->loadColumn();
		
		if (!empty($locations))
			foreach ($locations as $location) {
				$query->clear()
					->select($db->qn('venue'))->select($db->qn('url'))->select($db->qn('street'))->select($db->qn('plz'))
					->select($db->qn('city'))->select($db->qn('state'))->select($db->qn('country'))->select($db->qn('locdescription'))
					->select($db->qn('published'))
					->from($db->qn('#__eventlist_venues'))
					->where($db->qn('id').' = '.(int) $location);
				
				$db->setQuery($query);
				$elocation = $db->loadObject();
				
				$address = $elocation->street;
				if (!empty($elocation->plz)) $address .= ' , '.$elocation->plz;
				if (!empty($elocation->city)) $address .= ' , '.$elocation->city;
				if (!empty($elocation->state)) $address .= ' , '.$elocation->state;
				if (!empty($elocation->country)) $address .= ' , '.$elocation->country;
				
				$data = array();
				$data['name'] = $elocation->venue;
				$data['url'] = $elocation->url;
				$data['address'] = $address;
				$data['description'] = $elocation->locdescription;
				$data['published'] = $elocation->published;
				
				$newlocation = $this->_savelocation($data);
				$thelocations[$location] = $newlocation;
			}
		
		$thecategories = array();
		$query->clear()
			->select($db->qn('catsid'))
			->from($db->qn('#__eventlist_events'));
		
		$db->setQuery($query);
		$categories = $db->loadColumn();
		
		if (!empty($categories)) {
			$categories = array_map('intval',$categories);
			$categories = array_unique($categories);
			
			foreach ($categories as $category) {
				$query->clear()
					->select($db->qn('catname'))->select($db->qn('catdescription'))->select($db->qn('published'))
					->from($db->qn('#__eventlist_categories'))
					->where($db->qn('id').' = '.(int) $category);
				
				$db->setQuery($query);
				$evcategory = $db->loadObject();
				
				$data = array();
				$data['published'] = $evcategory->published;
				$data['title'] = $evcategory->catname;
				$data['description'] = $evcategory->catdescription;
				$data['parent_id'] = 1;
				
				$newcategory = $this->_savecategory($data);
				$thecategories[$category] = $newcategory;
			}
		}
		
		$container = array();
		
		if (!empty($eevents)) {
			foreach ($eevents as $event) {
				$eventcontainer = new stdClass();
				$eventcontainer->name = $event->title;
				$eventcontainer->description = $event->datdescription;
				$eventcontainer->location = $event->locid == 0 ? $default_location : $thelocations[$event->locid];
				$eventcontainer->start = $event->dates.' '.(!is_null($event->times) ? $event->times : '00:00:00');
				$eventcontainer->end = $event->enddates.' '.(!is_null($event->endtimes) ? $event->endtimes : '00:00:00') ;
				$eventcontainer->published = $event->published;
				$eventcontainer->owner = $event->created_by;
				
				$container[$event->catsid][] = $eventcontainer;
			}
		}
		
		$counter = 0;
		
		if (!empty($container)) {
			foreach ($container as $category => $event) {
				foreach ($event as $theevent) {
					$data = array();
					$data['name'] = $theevent->name;
					$data['description'] = $theevent->description;
					$data['location'] = $theevent->location;
					$data['start'] = $theevent->start;
					$data['end'] = $theevent->end;
					$data['published'] = $theevent->published;
					$data['owner'] = $theevent->owner;
					$data['completed'] = 1;
					
					if ($idevent = $this->_saveevent($data)) {
						$query->clear()
							->insert($db->qn('#__rseventspro_taxonomy'))
							->set($db->qn('type').' = '.$db->q('category'))
							->set($db->qn('id').' = '.(int) $thecategories[$category])
							->set($db->qn('ide').' = '.(int) $idevent);
						
						$db->setQuery($query);
						$db->execute();
						
						$counter++;
					}
				}
			}
		}
		
		if ($counter)
			return $counter;

		throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
	}
	
	public function ohanah() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
	
		$counter = 0;
		
		// Get events
		$query->clear()
			->select($db->qn('ohanah_event_id'))->select($db->qn('ohanah_category_id'))->select($db->qn('ohanah_venue_id'))
			->select($db->qn('title'))->select($db->qn('description'))->select($db->qn('created_by'))
			->from($db->qn('#__ohanah_events'));
		
		$db->setQuery($query);
		$events = $db->loadObjectList();
		
		if ($events) {
			// Get locations
			$query->clear()
				->select($db->qn('ohanah_venue_id'))->select($db->qn('name'))->select($db->qn('description'))
				->select($db->qn('address_1'))->select($db->qn('address_2'))->select($db->qn('city'))->select($db->qn('state'))
				->select($db->qn('zipcode'))->select($db->qn('country'))
				->select($db->qn('latitude'))->select($db->qn('longitude'))->select($db->qn('enabled'))
				->from($db->qn('#__ohanah_venues'));
			
			$db->setQuery($query);
			$locations = $db->loadObjectList();
			
			// Get categories
			$query->clear()
				->select($db->qn('ohanah_category_id'))->select($db->qn('title'))
				->select($db->qn('enabled'))->select($db->qn('description'))
				->from($db->qn('#__ohanah_categories'));
			
			$db->setQuery($query);
			$categories = $db->loadObjectList();
			
			// Store the Ohanah locations
			$thelocations = array();
			if (!empty($locations)) {
				foreach ($locations as $location) {
					$query->clear()
						->select('COUNT(ohanah_event_id)')
						->from($db->qn('#__ohanah_events'))
						->where($db->qn('ohanah_venue_id').' = '.(int) $location->ohanah_venue_id);
					
					$db->setQuery($query);
					if ($db->loadResult()) {
						// IMPORT locations
						$coordinates = !empty($location->latitude) && !empty($location->longitude) ? $location->latitude.','.$location->longitude : '';
						$address = $location->address_1.(!empty($location->address_2) ? ' '.$location->address_2 : '').(!empty($location->city) ? ', '.$location->city : '').(!empty($location->state) ? ', '.$location->state : '').(!empty($location->zipcode) ? ', '.$location->zipcode : '').(!empty($location->country) ? ', '.$location->country : '');
						
						$data = array();
						$data['name'] = $location->name;
						$data['address'] = $address;
						$data['description'] = $location->description;
						$data['published'] = $location->enabled;
						$data['coordinates'] = $coordinates;
						
						$newlocation = $this->_savelocation($data);
						$thelocations[$location->ohanah_venue_id] = $newlocation;
					}
				}
			}
			
			$thecategories = array();
			if (!empty($categories)) {
				foreach ($categories as $category) {
					$query->clear()
						->select('COUNT(ohanah_event_id)')
						->from($db->qn('#__ohanah_events'))
						->where($db->qn('ohanah_category_id').' = '.(int) $category->ohanah_category_id);
					
					$db->setQuery($query);
					if ($db->loadResult()) {
						// IMPORT categories
						$data = array();
						$data['published'] = $category->enabled;
						$data['title'] = $category->title;
						$data['description'] = $category->description;
						$data['parent_id'] = 1;
						
						$newcategory = $this->_savecategory($data);
						$thecategories[$category->ohanah_category_id] = $newcategory;
					}
				}
			}
			
			// Some events dont need a category ID
			$query->clear()
				->select('COUNT(ohanah_event_id)')
				->from($db->qn('#__ohanah_events'))
				->where($db->qn('ohanah_category_id').' = 0');
			$db->setQuery($query);
			if ((int) $db->loadResult()) {
				$data = array();
				$data['published'] = 1;
				$data['title'] = 'Ohanah Uncategorised';
				$data['description'] = '';
				$data['parent_id'] = 1;
				
				$newcategory = $this->_savecategory($data);
				$thecategories[0] = $newcategory;
			}
			
			foreach ($events as $event) {
				// Get event dates
				$query->clear()
					->select($db->qn('start'))->select($db->qn('end'))
					->select($db->qn('timezone'))->select($db->qn('all_day'))->select($db->qn('enabled'))
					->from($db->qn('#__ohanah_dates'))
					->where($db->qn('ohanah_event_id').' = '.$db->q($event->ohanah_event_id));
				$db->setQuery($query);
				
				if ($dates = $db->loadObjectList()) {
					foreach ($dates as $date) {
						// IMPORT events
						$data = array();
						$data['name'] = $event->title;
						$data['description'] = $event->description;
						$data['location'] = $thelocations[$event->ohanah_venue_id];
						$data['start'] = $date->start;
						$data['end'] = $date->all_day ? $db->getNullDate() : $date->end;
						$data['published'] = $date->enabled;
						$data['owner'] = $event->created_by;
						$data['completed'] = 1;
						
						if ($date->all_day) {
							$data['allday'] = 1;
						}
						
						if ($idevent = $this->_saveevent($data)) {
							$query->clear()
								->insert($db->qn('#__rseventspro_taxonomy'))
								->set($db->qn('type').' = '.$db->q('category'))
								->set($db->qn('id').' = '.(int) $thecategories[$event->ohanah_category_id])
								->set($db->qn('ide').' = '.(int) $idevent);
						
							$db->setQuery($query);
							$db->execute();
							
							$counter++;
						}
					}
				}
			}
		}
		
		if ($counter)
			return $counter;

		throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
	}

	public function csv() {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		$config		= Factory::getConfig();
		$format		= Factory::getApplication()->input->getString('dateformat','Y-m-d');
		$headers	= Factory::getApplication()->input->getInt('headers', 0);
		$file		= Factory::getApplication()->input->files->get('events');
		$tmp		= $config->get('tmp_path');
		$uid		= Factory::getUser()->get('id');
		$count		= 0;
		$nulldate	= $db->getNullDate();
		
		if (rseventsproHelper::getExt($file['name']) != 'csv') {
			throw new Exception(Text::_('COM_RSEVENTSPRO_INVALID_CSV_FILE'));
		}
		
		if ($file['error'] == 0 && $file['size'] > 0) {
			$upload = File::upload($file['tmp_name'],$tmp.'/rseventspro.csv');
			
			if ($upload) {
				setlocale(LC_ALL, 'en_US.UTF-8');
				$csvfile = $tmp.'/rseventspro.csv';
				$content = array();
				
				if (($handle = fopen($csvfile, 'r')) !== FALSE)  {
					$start = 0;
					while (($data = fgetcsv($handle, 4096, ',')) !== FALSE) {
						$start++;
						
						if ($start == 1 && $headers) continue;
						if (count($data) == 1 && $data[0] == '') continue;
						if (count($data) == 9 || count($data) == 11 || count($data) == 12) {
							$content[] = $data;
						} else continue;
					}
					fclose($handle);
				}
				
				if (!empty($content)) {
					$eventCategoriesCache = array();
					$eventTagsCache = array();
					
					foreach ($content as $event) {
						$allday		= 0;
						$location	= Factory::getApplication()->input->getInt('location',0);
						$category	= Factory::getApplication()->input->getInt('category',0);
						$name		= !empty($event[0]) ? $event[0] : Text::_('COM_RSEVENTSPRO_NEW_EVENT');
						$start		= !empty($event[1]) ? $event[1] : Factory::getDate()->format($format.' H:i:s');
						$end		= @$event[2];
						$description= !empty($event[3]) ? $event[3] : '';
						$url		= !empty($event[4]) ? $event[4] : '';
						$email		= !empty($event[5]) ? $event[5] : '';
						$phone		= !empty($event[6]) ? $event[6] : '';
						$lname		= !empty($event[7]) ? $event[7] : '';
						$laddress	= !empty($event[8]) ? $event[8] : '';
						$catname	= !empty($event[9]) ? $event[9] : 'Events';
						$catdesc	= !empty($event[10]) ? $event[10] : '';
						$tags		= !empty($event[11]) ? $event[11] : '';
						$start		= $this->transformDate($start,$format);
						$end		= $this->transformDate($end,$format);
						
						if ($location == 0 && !empty($lname)) {
							$data = array();
							$data['name'] = $lname;
							$data['address'] = $laddress;
							$data['published'] = 1;
							
							$query->clear()
								->select($db->qn('id'))
								->from($db->qn('#__rseventspro_locations'))
								->where($db->qn('name').' = '.$db->q($lname))
								->where($db->qn('address').' = '.$db->q($laddress));
							$db->setQuery($query);
							$location = (int) $db->loadResult();
							
							if (!$location)
								$location = $this->_savelocation($data);
						}
						
						// Categories
						$eventCategories = array();
						if ($category) {
							$eventCategories[] = $category;
						} else {
							$categories     = explode('|', $catname);
							$descriptions   = explode('|', $catdesc);

							foreach ($categories as $i => $category) {
								$data = array(
									'published'     => 1,
									'title'         => $category,
									'description'   => isset($descriptions[$i]) ? $descriptions[$i] : '',
									'parent_id'     => 1
								);

								// Cache the category results so we don't query the db often
								$hash = md5($data['title'].'|'.$data['description']);
								if (empty($eventCategoriesCache[$hash])) {
									// Do we have a category with the same details in the database?
									$query->clear()
										->select($db->qn('id'))
										->from($db->qn('#__categories'))
										->where($db->qn('title').' = '.$db->q($data['title']))
										->where($db->qn('description').' = '.$db->q($data['description']));
									$db->setQuery($query);
									if ($category = (int) $db->loadResult()) {
										// Add its ID to the list of cached categories
										$eventCategoriesCache[$hash] = $category;
									} else {
										// Add the new category
										$eventCategoriesCache[$hash] = $this->_savecategory($data);
									}
								}
								
								$eventCategories[] = $eventCategoriesCache[$hash];
							}
						}
						
						// Tags
						$eventTags	= array();
						if ($tags) {
							if ($tags = explode('|', $tags)) {
								foreach ($tags as $tag) {
									$tdata = array(
										'published' => 1,
										'name'     => $tag
									);
									
									// Cache the tag results so we don't query the db often
									$hash = md5($tdata['name']);
									if (empty($eventTagsCache[$hash])) {
										// Do we have a category with the same details in the database?
										$query->clear()
											->select($db->qn('id'))
											->from($db->qn('#__rseventspro_tags'))
											->where($db->qn('name').' = '.$db->q($tdata['name']));
										$db->setQuery($query);
										if ($tagid = (int) $db->loadResult()) {
											// Add its ID to the list of cached tags
											$eventTagsCache[$hash] = $tagid;
										} else {
											// Add the new tag
											$eventTagsCache[$hash] = $this->_savetag($tdata);
										}
									}
									
									$eventTags[] = $eventTagsCache[$hash];
								}
							}
						}
						
						$start = Factory::getDate($start);
						$start->modify($this->_tz.' seconds');
						$start = $start->toSql();
						
						if (empty($end) || $end == $nulldate) {
							$end = $nulldate;
							$allday = 1;
						} else {
							$end = Factory::getDate($end);
							$end->modify($this->_tz.' seconds');
							$end = $end->toSql();
						}
						
						if ($allday) {
							$start = explode(' ',$start);
							$start = $start[0].' 00:00:00';
						}
						
						$enc = mb_detect_encoding($description, mb_list_encodings(), true);
						
						// Save event
						$data = array();
						$data['from'] = 'import';
						$data['name'] = $name;
						$data['description'] = $enc ? iconv($enc, 'UTF-8', $description) : $description;
						$data['location'] = $location;
						$data['start'] = $start;
						$data['end'] = $end;
						$data['URL'] = $url;
						$data['email'] = $email;
						$data['phone'] = $phone;
						$data['published'] = 1;
						$data['owner'] = $uid;
						$data['completed'] = 1;
						$data['allday'] = $allday;
						
						if ($idevent = $this->_saveevent($data)) {
							foreach ($eventCategories as $eventCategory) {
								$query->clear()
									->insert($db->qn('#__rseventspro_taxonomy'))
									->set($db->qn('type').' = '.$db->q('category'))
									->set($db->qn('id').' = '.(int) $eventCategory)
									->set($db->qn('ide').' = '.(int) $idevent);

								$db->setQuery($query);
								$db->execute();
							}
							
							foreach ($eventTags as $eventTag) {
								$query->clear()
									->insert($db->qn('#__rseventspro_taxonomy'))
									->set($db->qn('type').' = '.$db->q('tag'))
									->set($db->qn('id').' = '.(int) $eventTag)
									->set($db->qn('ide').' = '.(int) $idevent);

								$db->setQuery($query);
								$db->execute();
							}
							
							$count++;
						}
					}

					if ($count) {
						return $count;
					} else {
						throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORTED_0_EVENTS'));
					}
				} else {
					throw new Exception(Text::_('COM_RSEVENTSPRO_CHECK_IMPORTED_FILE'));
				}
				
				File::delete($tmp.'/rseventspro.csv');
			} else {
				throw new Exception(Text::_('COM_RSEVENTSPRO_IMPORT_NO_DATA'));
			}
		} else {
			throw new Exception(Text::_('COM_RSEVENTSPRO_INVALID_FILE'));
		}
	}

	protected function _savecategory($data) {
		$data['extension'] = 'com_rseventspro';
		$data['language'] = '*';
		
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true)->select($db->qn('id'))->from($db->qn('#__categories'));
		
		foreach ($data as $key => $value) {
			$query->where($db->qn($key).' = '.$db->q($value));
		}
		
		$db->setQuery($query);
		if ($catID = (int) $db->loadResult()) {
			return $catID;
		} else {
			$table = Table::getInstance('Category', 'RseventsproTable');
			$alias = ApplicationHelper::stringURLSafe($data['title']);
			
			while ($table->load(array('alias' => $alias, 'parent_id' => 1, 'extension' => 'com_rseventspro'))) {
				$alias = StringHelper::increment($alias, 'dash');
			}
			
			$data['alias'] = $alias;
			
			$table = Table::getInstance('Category', 'RseventsproTable');
			$table->setLocation($data['parent_id'], 'last-child');
			$table->save($data);
			$table->rebuildPath($table->id);
			$table->rebuild($table->id, $table->lft, $table->level, $table->path);
			
			return $table->id;
		}
	}

	protected function _savelocation($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true)->select($db->qn('id'))->from($db->qn('#__rseventspro_locations'));
		
		foreach ($data as $key => $value) {
			$query->where($db->qn($key).' = '.$db->q($value));
		}
		
		$db->setQuery($query);
		if ($locationID = (int) $db->loadResult()) {
			return $locationID;
		} else {
			$table = Table::getInstance('Location', 'RseventsproTable');
			$data['description'] = '';
			$table->save($data);
			return $table->id;
		}
	}

	protected function _savetag($data) {
		$table = Table::getInstance('Tag', 'RseventsproTable');
		$table->save($data);
		return $table->id;
	}

	protected function _saveevent($data) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$model	= BaseDatabaseModel::getInstance('Event','RseventsproModel',  array('ignore_request' => true));
		
		if ($model->save($data)) {
			$eid			= $model->getState('event.id');
			$defaultOptions = rseventsproHelper::getDefaultOptions();
			
			$query->clear()
				->update($db->qn('#__rseventspro_events'))
				->set($db->qn('options').' = '.$db->q($defaultOptions))
				->where($db->qn('id').' = '.(int) $eid);
				
			if ($data['end'] == $db->getNullDate()) {
				$query->set($db->qn('end').' = '.$db->q($db->getNullDate()));
			}
			
			$db->setQuery($query);
			$db->execute();
			
			return $eid;
		} else {
			Factory::getApplication()->enqueueMessage($model->getError(), 'error');
		}
		
		return false;
	}

	protected function transformDate($date, $format) {
		$nulldate = Factory::getDbo()->getNullDate();
		if (empty($date) || $date == $nulldate || $date == '00/00/0000 00:00:00' || $date == '00.00.0000 00:00:00' || $date == '00 00 0000 00:00:00') {
			return $nulldate;
		}
		
		if ($format == 'Y-m-d') {
			return $date;
		} elseif ($format == 'Y/m/d') {
			return str_replace('/','-',$date);
		} elseif ($format == 'Y.m.d') {
			return str_replace('.','-',$date);
		} elseif ($format == 'Y m d') {
			$date = explode(' ',$date);
			$date = $date[0].'-'.$date[1].'-'.$date[2].' '.$date[3];
			return $date;
		} elseif ($format == 'd-m-Y') {
			$regex = '#(\d{1,2})\-(\d{1,2})\-(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[2].'-'.$match[1].' '.$match[4];
				}
			}
		} elseif ($format == 'd/m/Y') {
			$regex = '#(\d{1,2})\/(\d{1,2})\/(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[2].'-'.$match[1].' '.$match[4];
				}
			}
		} elseif ($format == 'd.m.Y') {
			$regex = '#(\d{1,2})\.(\d{1,2})\.(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[2].'-'.$match[1].' '.$match[4];
				}
			}
		} elseif ($format == 'd m Y') {
			$regex = '#(\d{1,2})\s(\d{1,2})\s(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[2].'-'.$match[1].' '.$match[4];
				}
			}
		} elseif ($format == 'm-d-Y') {
			$regex = '#(\d{1,2})\-(\d{1,2})\-(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[1].'-'.$match[2].' '.$match[4];
				}
			}
		} elseif ($format == 'm/d/Y') {
			$regex = '#(\d{1,2})\/(\d{1,2})\/(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[1].'-'.$match[2].' '.$match[4];
				}
			}
		} elseif ($format == 'm.d.Y') {
			$regex = '#(\d{1,2})\.(\d{1,2})\.(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[1].'-'.$match[2].' '.$match[4];
				}
			}
		} elseif ($format == 'm d Y') {
			$regex = '#(\d{1,2})\s(\d{1,2})\s(\d{1,4})\s(.*)#s';
			if (preg_match($regex,$date,$match)) {
				if (!empty($match)) {
					return $match[3].'-'.$match[1].'-'.$match[2].' '.$match[4];
				}
			}
		}
		
		return $date;
	}
	
	protected function checkVersion($string, $version, $operator = '>') {
		preg_match('#<version>(.*?)<\/version>#is',$string,$match);
		if (isset($match) && isset($match[1])) {
			return version_compare($version,$match[1],$operator);
		}
		
		return false;
	}
}