<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Router\Route;
use Joomla\Filesystem\File;
use Joomla\CMS\Factory;

class RseventsproModelSettings extends AdminModel
{
	protected $text_prefix = 'COM_RSEVENTSPRO';
	
	public function getForm($data = array(), $loadData = true) {
		$form = $this->loadForm('com_rseventspro.settings', 'settings', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		if (rseventsproHelper::isJ4()) {
			$form->setFieldAttribute('cancel_to','layout','joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('captcha_use','layout','joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('google_calendars','layout','joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('facebook_pages','layout','joomla.form.field.list-fancy-select');
			$form->setFieldAttribute('facebook_groups','layout','joomla.form.field.list-fancy-select');
		}
		
		return $form;
	}
	
	protected function loadFormData() {
		$layout = Factory::getApplication()->input->get('layout');
		
		if ($layout == 'invoice') {
			return $this->getInvoice();
		}
		
		$data = (array) $this->getConfig();
		
		if (isset($data['cancel_to'])) {
			$data['cancel_to'] = explode(',',$data['cancel_to']);
		}
		
		if (isset($data['captcha_use'])) {
			$data['captcha_use'] = explode(',',$data['captcha_use']);
		}
		
		if (isset($data['gallery_params'])) {
			try {
				$registry = new Registry;
				$registry->loadString($data['gallery_params']);
				$data['gallery'] = $registry->toArray();
			} catch (Exception $e) {
				$data['gallery'] = array();
			}
		}
		
		return $data;
	}
	
	public function getTabs() {
		$tabs = new RSEventsproAdapterTabs('settings');
		return $tabs;
	}
	
	public function getConfig() {
		return rseventsproHelper::getConfig();
	}
	
	public function getLayouts() {
		$fields = array('general', 'dashboard', 'events', 'layout', 'emails', 'maps', 'captcha', 'payments', 'sync');
		
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/rsmediagallery.php'))
			$fields[] = 'gallery';
		
		if (rseventsproHelper::pdf('1.18'))
			$fields[] = 'invoice';
		
		return $fields;
	}
	
	public function getSocial() {
		$options = array('cb' => false, 'js' => false, 'kunena' => false, 'fireboard' => false,
				'jcomments' => false, 'jomcomment' => false, 'rscomments' => false, 'k2' => false,
				'easydiscuss' => false, 'easysocial' => false
		);
		
		if (file_exists(JPATH_SITE.'/components/com_comprofiler/comprofiler.php'))
			$options['cb'] = true;
		
		if (file_exists(JPATH_SITE.'/components/com_community/community.php'))
			$options['js'] = true;
		
		if (file_exists(JPATH_SITE.'/components/com_kunena/kunena.php'))
			$options['kunena'] = true;
		
		if (file_exists(JPATH_SITE.'/components/com_fireboard/fireboard.php'))
			$options['fireboard'] = true;
			
		if (file_exists(JPATH_SITE.'/components/com_jcomments/jcomments.php'))
			$options['jcomments'] = true;
		
		if (file_exists(JPATH_SITE.'/plugins/content/jom_comment_bot/jom_comment_bot.php'))
			$options['jomcomment'] = true;
			
		if (file_exists(JPATH_SITE.'/components/com_rscomments/helpers/rscomments.php'))
			$options['rscomments'] = true;
		
		if (file_exists(JPATH_SITE.'/components/com_k2/k2.php'))
			$options['k2'] = true;
		
		if (file_exists(JPATH_SITE.'/components/com_easydiscuss/easydiscuss.php'))
			$options['easydiscuss'] = true;
		
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_easysocial/includes/foundry.php'))
			$options['easysocial'] = true;
		
		return $options;
	}
	
	public function save($data) {
		$files		= Factory::getApplication()->input->files->get('jform'); 
		$default	= isset($files['default_image']) ? $files['default_image'] : array();
		$app		= Factory::getApplication();
		
		// Check the coordinates
		try {
			$data['google_maps_center'] = rseventsproHelper::checkCoordinates($data['google_maps_center']);
		} catch(Exception $e) {
			$data['google_maps_center'] = '';
			$app->enqueueMessage($e->getMessage(),'error');
		}
		
		// Set default image
		if ($default && $default['error'] == 0 && $default['size'] > 0) {
			$extension = strtolower(rseventsproHelper::getExt($default['name']));
			if (in_array($extension, array('jpg','jpeg','png','gif'))) {
				$file = File::makeSafe($default['name']);
				if (File::upload($default['tmp_name'], JPATH_SITE.'/components/com_rseventspro/assets/images/default/'.$file)) {
					$data['default_image'] = $file;
				}
			}
		}
		
		// Save Facebook pages
		if (isset($data['facebook_pages'])) {
			$data['facebook_pages'] = is_array($data['facebook_pages']) ? implode(',', $data['facebook_pages']) : $data['facebook_pages'];
		} else {
			$data['facebook_pages'] = '';
		}
		
		// Save Facebook groups
		if (isset($data['facebook_groups'])) {
			$data['facebook_groups'] = is_array($data['facebook_groups']) ? implode(',', $data['facebook_groups']) : $data['facebook_groups'];
		} else {
			$data['facebook_groups'] = '';
		}
		
		// Save Google calendars
		if (isset($data['google_calendars'])) {
			$data['google_calendars'] = is_array($data['google_calendars']) ? implode(',', $data['google_calendars']) : $data['google_calendars'];
		} else {
			$data['google_calendars'] = '';
		}
		
		// Save cancel email status
		if (isset($data['cancel_to'])) {
			$data['cancel_to'] = is_array($data['cancel_to']) ? implode(',', $data['cancel_to']) : $data['cancel_to'];
		} else {
			$data['cancel_to'] = '';
		}
		
		// Save captcha use
		if (isset($data['captcha_use'])) {
			$data['captcha_use'] = is_array($data['captcha_use']) ? implode(',', $data['captcha_use']) : $data['captcha_use'];
		} else {
			$data['captcha_use'] = '';
		}
		
		// Save gallery params
		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/rsmediagallery.php')) {
			$gallery = isset($data['gallery']) ? $data['gallery'] : array();
			if (!empty($gallery)) {
				if (is_array($gallery['thumb_resolution']))
					$gallery['thumb_resolution'] = implode(',',$gallery['thumb_resolution']);
				
				if (is_array($gallery['full_resolution']))
					$gallery['full_resolution'] = implode(',',$gallery['full_resolution']);
				
				try {
					$registry = new Registry;
					$registry->loadArray($gallery);
					$data['gallery_params'] = $registry->toString();
				} catch (Exception $e) {
					$data['gallery_params'] = array();
				}
				unset($data['gallery']);
			}
		}
		
		// Save iDeal files
		$app->triggerEvent('onrseproIdealSaveSettings', array(array('data' => &$data)));
		
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->select('*')->from($db->qn('#__rseventspro_config'));
		$db->setQuery($query);
		$configuration = $db->loadColumn();
		
		foreach ($data as $name => $value) {
			$value = trim($value);
			
			if (in_array($name, $configuration)) {
				$query->clear()
					->update($db->qn('#__rseventspro_config'))
					->set($db->qn('value').' = '.$db->q($value))
					->where($db->qn('name').' = '.$db->q($name));
					
			} else {
				$query->clear()
					->insert($db->qn('#__rseventspro_config'))
					->set($db->qn('value').' = '.$db->q($value))
					->set($db->qn('name').' = '.$db->q($name));
			}
			
			$db->setQuery($query);
			$db->execute();
		}
		
		return true;
	}
	
	public function savetoken() {
		$db			 = Factory::getDbo();
		$query		 = $db->getQuery(true);
		$config		 = $this->getConfig();
		$redirectURI = Route::_('index.php?option=com_rseventspro&task=settings.savetoken', false, 1);
		$token		 = false;
		
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/facebook/autoload.php';
		
		try {
			$facebook = new Facebook\Facebook(array(
				'app_id' => $config->facebook_appid,
				'app_secret' => $config->facebook_secret,
				'default_graph_version' => 'v14.0',
				'pseudo_random_string_generator' => 'openssl'
			));
			
			$helper = $facebook->getRedirectLoginHelper();
			$token	= $helper->getAccessToken($redirectURI);
			
			if (isset($token)) {
				if (!$token->isLongLived()) {
					$oAuth2Client	= $facebook->getOAuth2Client();
					$token			= $oAuth2Client->getLongLivedAccessToken($token);
				}
				
				$token	= $token->getValue();
			}
		} catch (Exception $e) {
			Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
		}
		
		if ($token) {
			$query->clear()
				->update($db->qn('#__rseventspro_config'))
				->set($db->qn('value').' = '.$db->q(trim($token)))
				->where($db->qn('name').' = '.$db->q('facebook_token'));
			
			$db->setQuery($query);
			$db->execute();
			
			return true;
		}
		
		if ($error = Factory::getApplication()->input->getString('error_message')) {
			Factory::getApplication()->enqueueMessage($error, 'error');
		}
		
		return false;
	}

	public function google() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/google.php';
		
		$google	= new RSEPROGoogle();
		
		return $google->saveToken();
	}

	public function gimport() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/google.php';
		
		try {
			$google		= new RSEPROGoogle();
			$response	= $google->parse();
			
			if (!$response) {
				$this->setError(Text::_('COM_RSEVENTSPRO_NO_EVENTS_IMPORTED'));
				return false;
			}
			
			$this->setState($this->getName() . '.gcevents', $response);
			return true;
		} catch (Exception $e) {
			$this->setError($e->getMessage());
			return false;
		}
	}
	
	public function getLogs() {
		$db		= Factory::getDbo();
		$query	= $this->getLogQuery();
		
		$db->setQuery($query,Factory::getApplication()->input->getInt('limitstart', 0), Factory::getConfig()->get('list_limit'));
		return $db->loadObjectList();
	}
	
	public function getTotalLogs() {
		$db		= Factory::getDbo();
		$query	= $this->getLogQuery(true);
		
		$db->setQuery($query);
		return $db->loadResult();
	}
	
	public function getPagination() {
		return new Pagination($this->getTotalLogs(), Factory::getApplication()->input->getInt('limitstart', 0), Factory::getConfig()->get('list_limit'));
	}
	
	protected function getLogQuery($count = false) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery();
		$type	= Factory::getApplication()->input->get('from');
		$search	= Factory::getApplication()->input->getString('search');
		$select = $count ? 'COUNT('.$db->qn('id').')' : '*';
		
		$query->clear()
			->select($select)
			->from($db->qn('#__rseventspro_sync_log'))
			->where($db->qn('type').' = '.$db->q($type))
			->order($db->qn('date').' DESC')
			->order($db->qn('id').' ASC');
		
		if ($search) {
			$search = $db->q('%'.$db->escape($search, true).'%');
			$query->where('('.$db->qn('name').' LIKE '.$search.' OR '.$db->qn('from').' LIKE '.$search.')');
		}
		
		return $query;
	}
	
	public function getLogin() {
		$config 	 = $this->getConfig();
		$redirectURI = Route::_('index.php?option=com_rseventspro&task=settings.savetoken', false, 1);
		
		if ($config->facebook_appid && $config->facebook_secret) {
			try {
				require_once JPATH_SITE.'/components/com_rseventspro/helpers/facebook/autoload.php';
				
				$facebook = new Facebook\Facebook(array(
					'app_id' => $config->facebook_appid,
					'app_secret' => $config->facebook_secret,
					'default_graph_version' => 'v14.0',
					'pseudo_random_string_generator' => 'openssl'
				));
				
				$helper = $facebook->getRedirectLoginHelper();
				$permissions = array('pages_show_list');

				return $helper->getLoginUrl($redirectURI, $permissions);
				
			} catch (Exception $e) {
				Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
			}
		}
		
		return false;
	}
	
	public function importFacebookEvents() {
		$config 		= rseventsproHelper::getConfig();
		$input			= Factory::getApplication()->input;
		$init			= $input->getInt('init', 0);
		$step			= $input->getInt('step', 0);
		$total			= $input->getInt('total', 0);
		$fbpages		= $input->get('fbpages', array(), 'array');
		$owners			= $input->get('owners', array(), 'array');
		$names			= $input->get('names', array(), 'array');
		$types			= $input->get('types', array(), 'array');
		$allowedPages	= $config->facebook_pages;
		$allowedPages	= !empty($allowedPages) ? explode(',',$allowedPages) : '';
		$allowedGroups	= $config->facebook_groups;
		$allowedGroups	= !empty($allowedGroups) ? explode(',',$allowedGroups) : '';
		$profile		= isset($config->facebook_profile) ? $config->facebook_profile : 1;
		$data			= new stdClass();
		$reset			= false;
		$limit			= 50;
		
		try {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/facebook/autoload.php';
			
			$facebook = new Facebook\Facebook(array(
				'app_id' => $config->facebook_appid,
				'app_secret' => $config->facebook_secret,
				'default_graph_version' => 'v14.0',
				'default_access_token' => $config->facebook_token,
				'pseudo_random_string_generator' => 'openssl'
			));
			
			if ($init) {
				$fbRequest	= $facebook->get('me');
				$user		= $fbRequest->getDecodedBody();
				$uid 		= $user['id'];
				$fbpages	= array();
				$owners[]	= $uid;
				
				if ($profile) {
					$fbpages[] = 'me';
					$names['me'] = 'FBUSER';
					$types['me'] = 'user';
				}
				
				if (!empty($allowedPages)) {
					$fbRequest	= $facebook->get('me/accounts?fields=id,name&limit=200');
					$pages		= $fbRequest->getDecodedBody();
					
					if (!empty($pages) && !empty($pages['data'])) {
						foreach($pages['data'] as $page) {
							foreach ($allowedPages as $pid) {
								$pid = trim($pid);
								if ($pid == $page['id']) {
									$fbpages[] = $page['id'];
									$owners[] = $page['id'];
									$names[$page['id']] = $page['name'];
									$types[$page['id']] = 'page';
								}
							}
						}
					}
				}
				
				if (!empty($allowedGroups)) {
					foreach ($allowedGroups as $pid) {
						$fbRequest	= $facebook->get($pid.'?fields=id,name');
						$group	= $fbRequest->getDecodedBody();
						
						if (isset($group['id']) && $group['id'] == $pid) {
							$fbpages[] = $group['id'];
							$owners[] = $group['id'];
							$names[$group['id']] = $group['name'];
							$types[$group['id']] = 'group';
						}
					}
				}
			}
			
			if (!empty($fbpages)) {
				reset($fbpages);
				if ($pageid = current($fbpages)) {
					$stop = false;
					$response = $facebook->get('/'.$pageid.'/events?fields=id,name,start_time,end_time,timezone,description,owner,cover,place,event_times&limit='.$limit);
					
					// Get the first page
					if ($feed = $response->getGraphEdge()) {
						// Parse the first page
						if ($step == 0) {
							if ($events = $feed->asArray()) {
								foreach ($events as $event) {
									$total += rseventsproHelper::parseFacebookEvent($event, $names[$pageid], $owners, $types[$pageid]);
								}
							}
						}
						
						// Go to the next page
						if ($step > 0) {
							for($i=0;$step>$i;$i++) {
								if ($feed) {
									$feed = $facebook->next($feed);
								} else {
									$stop = true;
								}
							}
							
							// Parse events from current page
							if ($feed) {
								$events = $feed->asArray();
								foreach ($events as $event) {
									$total += rseventsproHelper::parseFacebookEvent($event, $names[$pageid], $owners, $types[$pageid]);
								}
							} else {
								$stop = true;
							}
						}
					} else {
						$stop = true;
					}
					
					if ($stop) {
						// All pages were parsed, and we remove the page ID and reset the $step value
						array_shift($fbpages);
						$reset = true;
					}
				}
			}
			
			$data->fbpages	= $fbpages;
			$data->owners	= $owners;
			$data->names	= $names;
			$data->types	= $types;
			$data->total	= $total;
			$data->step		= $reset ? 0 : $step + 1;			
		} catch (Exception $e) {
			$data->message = $e->getMessage();
		}
		
		return json_encode($data);
	}
	
	public function getInvoice() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery();
		$lang	= $this->getLanguage();
		
		$query->clear()
			->select($db->qn('title'))->select($db->qn('layout'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('ide'). ' = 0')
			->where($db->qn('lang'). ' = '.$db->q($lang));
		$db->setQuery($query);
		$invoice = $db->loadObject();
		
		if (!$invoice) {
			$query->clear()
				->select($db->qn('title'))->select($db->qn('layout'))
				->from($db->qn('#__rseventspro_invoice'))
				->where($db->qn('ide'). ' = 0')
				->where($db->qn('lang'). ' = '.$db->q('en-GB'));
			$db->setQuery($query);
			$invoice = $db->loadObject();
			
			if (!$invoice) {
				$invoice = new stdClass();
				$invoice->title = '';
				$invoice->layout = '';
			}
		}
		
		return array('invoice_language' => $lang, 'invoice_title' => $invoice->title, 'invoice_layout' => $invoice->layout);
	}
	
	public function getLanguage() {
		$jform	= Factory::getApplication()->input->get('jform',array(),'array');
		return !empty($jform['invoice_language']) ? $jform['invoice_language'] : Factory::getLanguage()->getTag();
	}
	
	public function saveinvoice($data) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery();
		$language	= isset($data['invoice_language']) ? $data['invoice_language'] : $this->getLanguage();
		$title		= isset($data['invoice_title']) ? $data['invoice_title'] : 'Invoice';
		$layout		= isset($data['invoice_layout']) ? $data['invoice_layout'] : '';
		
		$query->clear()
			->select($db->qn('id'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('lang').' = '.$db->q($language))
			->where($db->qn('ide').' = '.$db->q(0));
		
		$db->setQuery($query);
		if ($id = $db->loadResult()) {
			$query->clear()
				->update($db->qn('#__rseventspro_invoice'))
				->set($db->qn('title').' = '.$db->q($title))
				->set($db->qn('layout').' = '.$db->q($layout))
				->where($db->qn('id').' = '.$db->q($id));
		} else {
			$query->clear()
				->insert($db->qn('#__rseventspro_invoice'))
				->set($db->qn('layout').' = '.$db->q($layout))
				->set($db->qn('title').' = '.$db->q($title))
				->set($db->qn('lang').' = '.$db->q($language))
				->set($db->qn('ide').' = '.$db->q(0));
		}
		
		$db->setQuery($query);
		$db->execute();
		
		return true;
	}
}