<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\Registry\Registry;
use Joomla\Filesystem\File;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Factory;

class RseventsproModelSpeaker extends AdminModel
{
	protected $text_prefix = 'COM_RSEVENTSPRO';

	public function getTable($type = 'Speaker', $prefix = 'RseventsproTable', $config = array()) {
		return Table::getInstance($type, $prefix, $config);
	}

	public function getItem($pk = null) {
		if ($item = parent::getItem($pk)) {
			try {
				$registry = new Registry;
				$registry->loadString($item->custom ? $item->custom : '');
				$item->custom = $registry->toArray();
			} catch (Exception $e) {
				$item->custom = array();
			}
			
			RSEventsProTranslations::overrideValues($item, 'speaker');
		}
		
		return $item;
	}

	public function getForm($data = array(), $loadData = true) {
		// Get the form.
		$form = $this->loadForm('com_rseventspro.speaker', 'speaker', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		RSEventsProTranslations::setTranslateIcon($form, 'speaker');
		
		return $form;
	}

	protected function loadFormData() {
		$data = Factory::getApplication()->getUserState('com_rseventspro.edit.speaker.data', array());

		if (empty($data))
			$data = $this->getItem();

		return $data;
	}
	
	public function save($data) {
		$table	= $this->getTable();
		$pk 	= (!empty($data['id'])) ? $data['id'] : (int) $this->getState($this->getName() . '.id');
		
		// Load the row if saving an existing tag.
		if ($pk > 0) {
			$table->load($pk);
		}
		
		// Bind the data.
		if (!$table->bind($data)) {
			$this->setError($table->getError());
			return false;
		}

		// Check the data.
		if (!$table->check()) {
			$this->setError($table->getError());
			return false;
		}
		
		// Store the data.
		if (!$table->store()) {
			$this->setError($table->getError());
			return false;
		}
		
		// Upload the image
		$table->uploadImage();

		$this->setState($this->getName() . '.id', $table->id);

		return true;
	}
	
	public function deleteimage() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id',0);
		$path	= JPATH_SITE.'/components/com_rseventspro/assets/images/speakers/';
		
		$query->select($db->qn('image'))
			->from($db->qn('#__rseventspro_speakers'))
			->where($db->qn('id').' = '.$id);
		$db->setQuery($query);
		if ($image = $db->loadResult()) {
			if (file_exists($path.$image)) {
				if (File::delete($path.$image)) {
					$query->clear()
						->update($db->qn('#__rseventspro_speakers'))
						->set($db->qn('image').' = '.$db->q(''))
						->where($db->qn('id').' = '.$id);
					$db->setQuery($query);
					$db->execute();
					
					return true;
				}
			}
			
			return false;
		}
		
		return false;
	}
}