<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Factory;

class RseventsproModelSubscriptions extends ListModel
{
	public function __construct($config = array()) {
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'u.name', 'e.name', 'u.id', 'u.date',
				'u.gateway', 'u.state', 'u.confirmed', 'u.lang', 
				'state', 'event', 'ticket', 'from', 'to', 'language', 'payment'
			);
		}
		
		parent::__construct($config);
	}

	protected function getListQuery() {
		$db 	= Factory::getDBO();
		$query 	= $db->getQuery(true);
		$tz		= rseventsproHelper::getTimezone();
		$cart	= false;
		
		Factory::getApplication()->triggerEvent('onrsepro_isCart',array(array('cart'=>&$cart)));
		
		$search 	= $this->getState('filter.search');
		$ticket 	= $this->getState('filter.ticket');
		$state		= $this->getState('filter.state');
		$language	= $this->getState('filter.language');
		$payment	= $this->getState('filter.payment');
		$event		= $this->getState('filter.event');
		$from		= $this->getState('filter.from');
		$to			= $this->getState('filter.to');
		$orderby	= $this->getState('list.ordering', 'u.date');
		$orderdir	= $this->getState('list.direction', 'desc');
		
		$from	= $from ? Factory::getDate($from, $tz)->toSql() : '';
		if ($to) {
			$to = Factory::getDate($to,$tz);
			$to->setTime(23,59,59);
			$to = $to->toSql();
		}
		
		require_once JPATH_SITE. '/components/com_rseventspro/helpers/subscriptions.php';
		
		$subscriptionsClass = new RSEventsProSubscriptions;
		
		if (!empty($search)) {
			$subscriptionsClass->setFilter('search', $search);
		}
		
		if (is_numeric($ticket)) {
			$subscriptionsClass->setFilter('ticket', $ticket);
		}
		
		if (is_numeric($event)) {
			$subscriptionsClass->setFilter('ide', $event);
		}
		
		if ($state != '') {
			$subscriptionsClass->setFilter('state', $state);
		}
		
		if ($language != '') {
			$subscriptionsClass->setFilter('language', $language);
		}
		
		if ($payment != '') {
			$subscriptionsClass->setFilter('payment', $payment);
		}
		
		$subscriptionsClass->setFilter('cart', $cart);
		$subscriptionsClass->setFilter('from', $from);
		$subscriptionsClass->setFilter('to', $to);
		$subscriptionsClass->setFilter('list.order', $orderby);
		$subscriptionsClass->setFilter('list.dir', $orderdir);
		
		return $subscriptionsClass::getSubscriptionsQuery();
	}

	protected function populateState($ordering = 'u.date', $direction = 'desc') {
		$event = Factory::getApplication()->input->getInt('filter_event', 0);

		$this->setState('filter.event', $this->getUserStateFromRequest($this->context . '.filter.event', 'filter_event', '', 'cmd'));

		// List state information.
		parent::populateState($ordering, $direction);

		if ($event) {
			$this->setState('filter.event', $event);
		}
	}
	
	/**
	 *	Method to export subscribers
	 */
	public function export() {
		$query = $this->getListQuery();
		rseventsproHelper::exportSubscribersCSV($query);
	}
}