<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\CMS\Table\Table;
use Joomla\Filesystem\File;
use Joomla\CMS\Factory;

class RseventsproModelUser extends AdminModel
{
	protected $text_prefix = 'COM_RSEVENTSPRO';

	public function getTable($type = 'User', $prefix = 'RseventsproTable', $config = array()) {
		return Table::getInstance($type, $prefix, $config);
	}

	public function getItem($pk = null) {
		$item = parent::getItem($pk);
		
		if ($item) {
			RSEventsProTranslations::overrideValues($item, 'user');
		}
		
		if (empty($item)) {
			$item = new stdClass();
			$id		= Factory::getApplication()->input->getInt('id',0);
			$user	= Factory::getUser($id);
			
			$item->id	= $user->get('id');
			$item->name	= $user->get('name');
			$item->language	= Factory::getLanguage()->getTag();
		}
		
		if ($item) {
			if (!$item->id) {
				$id		= Factory::getApplication()->input->getInt('id',0);
				$user	= Factory::getUser($id);
				
				$item->id	= $user->get('id');
				$item->name	= $user->get('name');
				$item->language	= Factory::getLanguage()->getTag();
			}
		}
		
		return $item;
	}

	public function getForm($data = array(), $loadData = true) {
		// Get the form.
		$form = $this->loadForm('com_rseventspro.user', 'user', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form))
			return false;
		
		RSEventsProTranslations::setTranslateIcon($form, 'user');
		
		return $form;
	}

	protected function loadFormData() {
		$data = Factory::getApplication()->getUserState('com_rseventspro.edit.user.data', array());

		if (empty($data))
			$data = $this->getItem();

		return $data;
	}
	
	public function save($data) {
		$table	= $this->getTable();
		$pk 	= (!empty($data['id'])) ? $data['id'] : (int) $this->getState($this->getName() . '.id');
		
		// Load the row if saving an existing tag.
		if ($pk > 0) {
			$table->load($pk);
		}
		
		// Bind the data.
		if (!$table->bind($data)) {
			$this->setError($table->getError());
			return false;
		}

		// Check the data.
		if (!$table->check()) {
			$this->setError($table->getError());
			return false;
		}
		
		// Store the data.
		if (!$table->store()) {
			$this->setError($table->getError());
			return false;
		}
		
		// Upload the image
		$table->uploadImage();

		$this->setState($this->getName() . '.id', $table->id);

		return true;
	}
	
	public function deleteimage() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id',0);
		$path	= JPATH_SITE.'/components/com_rseventspro/assets/images/users/';
		
		$query->select($db->qn('image'))
			->from($db->qn('#__rseventspro_user_info'))
			->where($db->qn('id').' = '.$id);
		$db->setQuery($query);
		if ($image = $db->loadResult()) {
			if (file_exists($path.$image)) {
				if (File::delete($path.$image)) {
					$query->clear()
						->update($db->qn('#__rseventspro_user_info'))
						->set($db->qn('image').' = '.$db->q(''))
						->where($db->qn('id').' = '.$id);
					$db->setQuery($query);
					$db->execute();
					
					return true;
				}
			}
			
			return false;
		}
		
		return false;
	}
	
	public function reset($pks) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		if ($pks) {
			$query->clear()
				->delete($db->qn('#__rseventspro_taxonomy'))
				->where($db->qn('type').' = '.$db->q('userCount'))
				->where($db->qn('id').' IN ('.implode('', $pks).')');
			$db->setQuery($query);
			$db->execute();
		}
		
		return true;
	}
}