<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Factory;

class RseventsproModelWaitinglist extends ListModel
{
	public function __construct($config = array()) {
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'name', 'email', 'date', 'sent', 'confirmed'
			);
		}
		
		parent::__construct($config);
	}
	
	protected function getListQuery() {
		$db 	= Factory::getDBO();
		$query 	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id',0);
		
		// Select fields
		$query->select('*');
		
		// Select from table
		$query->from($db->qn('#__rseventspro_waitinglist'));
		
		$query->where($db->qn('ide').' = '.$db->q($id));
		
		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			$search = $db->Quote('%'.$db->escape($search, true).'%');
			$query->where($db->qn('name').' LIKE '.$search.' OR '.$db->qn('email').' LIKE '.$search);
		}
		
		// Add the list ordering clause
		$listOrdering = $this->getState('list.ordering', 'date');
		$listDirn = $db->escape($this->getState('list.direction', 'asc'));
		$query->order($db->escape($listOrdering).' '.$listDirn);
		
		return $query;
	}
	
	public function getForm() {
		$form = Form::getInstance('waitinglist', JPATH_ADMINISTRATOR.'/components/com_rseventspro/models/forms/waitinglist.xml', array('control' => 'jform')); 
		$data = $this->getItem();
		$form->bind($data);
		
		return $form;
	}
	
	public function getItem() {
		$db 	= Factory::getDBO();
		$query 	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id',0);
		
		$query->clear()
			->select('*')
			->from($db->qn('#__rseventspro_waitinglist'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		return $db->loadObject();
	}
	
	public function save($data) {
		$db	= Factory::getDBO();
		
		$form = Form::getInstance('waitinglist', JPATH_ADMINISTRATOR.'/components/com_rseventspro/models/forms/waitinglist.xml', array('control' => 'jform'));
		$form->bind($data);
		
		$data = $form->filter($data);
		$return = $form->validate($data);

		// Check for an error.
		if ($return instanceof \Exception) {
			$this->setError($return->getMessage());
			return false;
		}

		// Check the validation results.
		if ($return === false) {
			// Get the validation messages from the form.
			foreach ($form->getErrors() as $message) {
				$this->setError($message);
			}

			return false;
		}
		
		$data = (object) $data;
		$db->updateObject('#__rseventspro_waitinglist', $data, 'id');
		
		return true;
	}
	
	public function delete($pks) {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		$pks	= array_map('intval', $pks);
		
		$query->clear()
			->delete($db->qn('#__rseventspro_waitinglist'))
			->where($db->qn('id').' IN ('.implode(',',$pks).')');
		$db->setQuery($query);
		$db->execute();
	}
	
	public function approve($pks) {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		$pks	= array_map('intval', $pks);
		$now	= Factory::getDate()->toSql();
		
		$query->clear()
			->select('w.*')->select($db->qn('e.waitinglist_time'))
			->from($db->qn('#__rseventspro_waitinglist','w'))
			->join('LEFT',$db->qn('#__rseventspro_events','e').' ON '.$db->qn('w.ide').' = '.$db->qn('e.id'))
			->where($db->qn('w.id').' IN ('.implode(',', $pks).')')
			->where($db->qn('w.hash').' = '.$db->q(''));
		$db->setQuery($query);
		if ($items = $db->loadObjectList()) {
			foreach ($items as $item) {
				$hash = md5($item->id.$item->email.$item->date);
				$date = '';
				
				if ($item->waitinglist_time) {
					$date = Factory::getDate();
					$date->modify('+'.$item->waitinglist_time.' second');
					$date = rseventsproHelper::showdate($date->toSql());
				}
				
				$query->clear()
					->update($db->qn('#__rseventspro_waitinglist'))
					->set($db->qn('hash').' = '.$db->q($hash))
					->set($db->qn('sent').' = '.$db->q($now))
					->where($db->qn('id').' = '.$db->q($item->id));
				$db->setQuery($query);
				if ($db->execute()) {
					rseventsproEmails::waitinglist($item->email, $item->ide, $item->name, $hash, $date, $item->language);
				}
			}
		}
	}
	
	public function export() {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id',0);
		$wquery	= $this->getListQuery();
		$lines  = array();
		$csv	= '';


		if (rseventsproHelper::getConfig('export_headers')) {
			$line = array();

			$line[] = Text::_('COM_RSEVENTSPRO_WAITINGLIST_EXPORT_NAME');
			$line[] = Text::_('COM_RSEVENTSPRO_WAITINGLIST_EXPORT_EMAIL');
			$line[] = Text::_('COM_RSEVENTSPRO_WAITINGLIST_EXPORT_ADDED');
			$line[] = Text::_('COM_RSEVENTSPRO_WAITINGLIST_EXPORT_SENT');
			$line[] = Text::_('COM_RSEVENTSPRO_WAITINGLIST_EXPORT_CONFIRMED');

			// Escape CSV line
			rseventsproHelper::csvEscape($line);

			$lines[] = $line;
		}
		
		$db->setQuery($wquery);
		if ($users = $db->loadObjectList()) {
			foreach ($users as $user) {
				$line = array();
				$date = empty($user->date) || $user->date == $db->getNullDate() ? '-' : rseventsproHelper::showdate($user->date,'Y-m-d H:i:s');
				$sent = empty($user->sent) || $user->sent == $db->getNullDate() ? '-' : rseventsproHelper::showdate($user->sent,'Y-m-d H:i:s');
				$confirmed = empty($user->confirmed) || $user->confirmed == $db->getNullDate() ? '-' : rseventsproHelper::showdate($user->confirmed,'Y-m-d H:i:s');

				$line[] = $user->name;
				$line[] = $user->email;
				$line[] = $date;
				$line[] = $sent;
				$line[] = $confirmed;

				// Escape CSV line
				rseventsproHelper::csvEscape($line);

				$lines[] = $line;
			}
		}

		foreach ($lines as $line) {
			$csv .= rseventsproHelper::csvLine($line)."\n";
		}
		
		$query->clear()
			->select($db->qn('name'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$eventname = $db->loadResult();
		
		$file = htmlentities(Text::sprintf('COM_RSEVENTSPRO_WAITINGLIST_EXPORT_FOR',$eventname), ENT_COMPAT, 'UTF-8');
		header('Content-type: text/csv; charset=UTF-8');
		header('Content-Disposition: attachment; filename="'.$file.'.csv"');
		echo rtrim($csv,"\n");
		exit();
	}
	
	protected function populateState($ordering = null, $direction = null) {
		parent::populateState('date', 'asc');
    }
}