<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Application\ApplicationHelper;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Installer\Installer;
use Joomla\Filesystem\Folder;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Version;
use Joomla\CMS\Factory;

class com_rseventsproInstallerScript 
{	
	public function preflight($type, $parent) {
		try {
			$minJoomla = '3.9.0';

			if (!class_exists('\\Joomla\\CMS\\Version')) {
				throw new Exception(sprintf('Please upgrade to at least Joomla! %s before continuing!', $minJoomla));
			}

			$jversion = new \Joomla\CMS\Version;
			if (!$jversion->isCompatible($minJoomla)) {
				throw new Exception(sprintf('Please upgrade to at least Joomla! %s before continuing!', $minJoomla));
			}
		} catch (Exception $e) {
			if (class_exists('\Joomla\CMS\Factory')) {
				$app = \Joomla\CMS\Factory::getApplication();
			} elseif (class_exists('JFactory')) {
				$app = JFactory::getApplication();
			}

			if (!empty($app)) {
				$app->enqueueMessage($e->getMessage(), 'error');
			}
			
			return false;
		}

		return true;
	}

	public function postflight($type, $parent) {
		$this->installprocess($type, $parent);
		
		$messages = $this->checkAddons();
		
		$this->showinstall($messages);
	}
	
	public function uninstall($parent) {
		$db			= Factory::getDbo();
		$installer	= new Installer();
		
		Folder::delete(JPATH_SITE.'/layouts/rseventspro/');
		
		$db->setQuery('SELECT '.$db->qn('extension_id').' FROM '.$db->qn('#__extensions').' WHERE '.$db->qn('element').' = '.$db->q('rseventspro').' AND '.$db->qn('folder').' = '.$db->q('installer').' AND '.$db->qn('type').' = '.$db->q('plugin').' LIMIT 1');
		$plg_id = $db->loadResult();
		if ($plg_id) $installer->uninstall('plugin', $plg_id);
		
		$db->setQuery('SELECT '.$db->qn('extension_id').' FROM '.$db->qn('#__extensions').' WHERE '.$db->qn('element').' = '.$db->q('rseventspro').' AND '.$db->qn('folder').' = '.$db->q('privacy').' AND '.$db->qn('type').' = '.$db->q('plugin').' LIMIT 1');
		$priv_id = $db->loadResult();
		if ($priv_id) $installer->uninstall('plugin', $priv_id);
	}
	
	// Install - Update process
	public function installprocess($type, $parent) {
		$db	= Factory::getDbo();
		$installer = new Installer();
		
		Folder::copy($parent->getParent()->getPath('source').'/other/layouts/', JPATH_SITE.'/layouts/', '', true);
		
		$installer->install($parent->getParent()->getPath('source').'/other/plg_installer');
		$db->setQuery('UPDATE '.$db->qn('#__extensions').' SET '.$db->qn('enabled').' = 1 WHERE '.$db->qn('element').' = '.$db->q('rseventspro').' AND '.$db->qn('type').' = '.$db->q('plugin').' AND '.$db->qn('folder').' = '.$db->q('installer'));
		$db->execute();
		
		$installer->install($parent->getParent()->getPath('source').'/other/plg_rseventsproprivacy');
		$db->setQuery('UPDATE '.$db->qn('#__extensions').' SET '.$db->qn('enabled').' = 1 WHERE '.$db->qn('element').' = '.$db->q('rseventspro').' AND '.$db->qn('type').' = '.$db->q('plugin').' AND '.$db->qn('folder').' = '.$db->q('privacy'));
		$db->execute();
		
		
		if ($type == 'update') {
			// Run queries
			$sqlfile = JPATH_ADMINISTRATOR.'/components/com_rseventspro/install.mysql.sql';
			$buffer = file_get_contents($sqlfile);
			if ($buffer === false) {
				throw new Exception(Text::_('JLIB_INSTALLER_ERROR_SQL_READBUFFER'), 1);
			}

			$queries = $db->splitSql($buffer);
			if (count($queries) == 0) {
				// No queries to process
				return 0;
			}
			
			// Process each query in the $queries array (split out of sql file).
			foreach ($queries as $query) {
				$db->setQuery($db->convertUtf8mb4QueryToUtf8($query));
				if (!$db->execute()) {
					throw new Exception(Text::sprintf('JLIB_INSTALLER_ERROR_SQL_ERROR', $db->stderr(true)), 1);
				}
			}
			
			// REV 4
			
			// Check for the sync field
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'sync'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `sync` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}

			// Check for the sid field
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'sid'"); 
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `sid` VARCHAR( 255 ) NOT NULL");
				$db->execute();
			}

			// Check for the lang field
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'lang'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` ADD `lang` VARCHAR( 10 ) NOT NULL");
				$db->execute();
			}

			// Check for the coupon field
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'coupon'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` ADD `coupon` VARCHAR( 255 ) NOT NULL");
				$db->execute();
			}
			
			// Check and remove the 'code' field from the coupons table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_coupons` WHERE `Field` = 'code'");
			if ($db->loadResult()) {
				// Get coupon codes and add them in the new coupon codes table
				$db->setQuery("SELECT `id`, `code`, `used` FROM `#__rseventspro_coupons`");
				if ($coupons = $db->loadObjectList()) {
					foreach ($coupons as $coupon) {
						if (!empty($coupon->code)) {
							$codes = explode("\n",$coupon->code);
							if(!empty($codes)) {
								foreach ($codes as $code) {				
									$code = trim($code);
									$db->setQuery("INSERT INTO `#__rseventspro_coupon_codes` SET `code` = '".$db->escape($code)."', `idc` = ".(int) $coupon->id.", `used` = ".(int) $coupon->used." ");
									$db->execute();
								}
							}
						}
					}
				}
				
				$db->setQuery("ALTER TABLE `#__rseventspro_coupons` DROP `code`");
				$db->execute();
			}

			// Check and remove the 'used' field from the coupons table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_coupons` WHERE `Field` = 'used'");
			if ($db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_coupons` DROP `used`");
				$db->execute();
			}
			
			// Set the tax_value field to float
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_payments` WHERE `Field` = 'tax_value'");
			$paymentsTable = $db->loadObject();
			if ($paymentsTable->Type == 'int(11)') {
				$db->setQuery("ALTER TABLE `#__rseventspro_payments` CHANGE `tax_value` `tax_value` FLOAT NOT NULL");
				$db->execute();
			}
			
			// Check for the 'allday' field on the events table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'allday'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `allday` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			// Check for the 'notify_me_unsubscribe' field on the events table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'notify_me_unsubscribe'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `notify_me_unsubscribe` TINYINT( 1 ) NOT NULL AFTER `notify_me`");
				$db->execute();
			}
			
			// Check for the 'ideal' field on the subscribers table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'ideal'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` ADD `ideal` VARCHAR( 100 ) NOT NULL");
				$db->execute();
			}
			
			// Update groups table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_add_locations'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_add_locations` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_create_categories'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_create_categories` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_delete_events'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_delete_events` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_download'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_download` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_edit_events'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_edit_events` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_edit_locations'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_edit_locations` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_post_events'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_post_events` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_register'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_register` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_repeat_events'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_repeat_events` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_unsubscribe'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_unsubscribe` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_upload'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_upload` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'event_moderation'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `event_moderation` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'tag_moderation'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `tag_moderation` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_approve_events'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_approve_events` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_approve_tags'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_approve_tags` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			// Update groups table with data
			$tables = $db->getTableList();
			if (in_array($db->getPrefix().'rseventspro_group_permissions', $tables)) {
				$db->setQuery("SELECT * FROM `#__rseventspro_group_permissions`");
				if ($permissions = $db->loadObjectList()) {
					foreach ($permissions as $permission) {
						$db->setQuery("UPDATE #__rseventspro_groups SET `".$permission->name."` = '".$db->escape($permission->value)."' WHERE `id` = '".(int) $permission->id."' ");
						$db->execute();
					}
				}
			}
			
			// Drop groups permissions table
			$db->setQuery("DROP TABLE IF EXISTS `#__rseventspro_group_permissions`");
			$db->execute();
			
			// Update Categories
			if (in_array($db->getPrefix().'rseventspro_categories', $tables)) {
				$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_categories` WHERE `Field` = 'ordering'");
				if ($db->loadResult()) {
					$db->setQuery("SELECT `id`, `parent` FROM `#__rseventspro_categories`");
					if ($tmpCategories = $db->loadObjectList()) {
						$categories = array();
						$parents = array();
						foreach ($tmpCategories as $category) {
							$parents[$category->id] = $category->parent;
						}
						
						$tree = $levels = array();
						$this->renderTree($tmpCategories,$tree,$levels);
						$flatCateories = $this->renderFlatTree($tree);
						
						if (!empty($flatCateories)) {
							foreach ($flatCateories as $flatCategory) {
								$db->setQuery("SELECT `id`, `parent`, `name`, `color`, `description`, `published` FROM `#__rseventspro_categories` WHERE id = ".(int) $flatCategory."");
								$categories[] = $db->loadObject();
							}
						}
						
						$newids = array();
						$newids[0] = 1;
						$uid = Factory::getUser()->get('id');
						
						Table::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_categories/tables/');
						foreach ($categories as $category) {
							$table = Table::getInstance('Category', 'CategoriesTable');
							
							$table->id = null;
							$table->title = $category->name;
							$table->alias = ApplicationHelper::stringURLSafe($category->name);
							$table->extension = 'com_rseventspro';
							$table->setLocation($newids[$parents[$category->id]], 'last-child');
							$table->description = $category->description;
							$table->created_user_id = $uid;
							$table->language = '*';
							$table->published = $category->published;
							$registry = new Registry();
							$registry->loadArray(array('color' => $category->color));
							$table->params	= $registry->toString();
							
							
							$table->store();
							$table->rebuildPath($table->id);
							$table->rebuild($table->id, $table->lft, $table->level, $table->path);
							$newids[$category->id] = $table->id;
						}
						
						unset($newids[0]);
						
						if (!empty($newids)) {
							$db->setQuery("SELECT `ide`, `id` FROM `#__rseventspro_taxonomy` WHERE `type` = 'category'");
							if ($relations = $db->loadObjectList()) {
								$db->setQuery("DELETE FROM `#__rseventspro_taxonomy` WHERE `type` = 'category'");
								$db->execute();
							}
							
							foreach ($relations as $relation) {
								if (isset($newids[$relation->id])) {
									$db->setQuery("INSERT INTO #__rseventspro_taxonomy SET `ide` = ".(int) $relation->ide." , `id` = ".(int) $newids[$relation->id].", `type` = 'category'");
									$db->execute();
								}
							}
							
							// Update calendar menus
							$db->setQuery("SELECT `id`, `params` FROM #__menu WHERE `link` LIKE 'index.php?option=com_rseventspro&view=calendar'");
							if ($calendarMenus = $db->loadObjectList()) {
								foreach ($calendarMenus as $calendarMenu) {
									$registry = new Registry;
									$registry->loadString($calendarMenu->params);
		
									$categories = $registry->get('categories');
									if (!empty($categories)) {
										$categories = explode(',',$categories);
										foreach ($categories as $i => $category) {
											$categories[$i] = $newids[$category];
										}
									} else {
										$categories = '';
									}
									
									$locations = $registry->get('locations');
									$locations = !empty($locations) ? explode(',',$locations) : '';
									
									$tags = $registry->get('tags');
									$tags = !empty($tags) ? explode(',',$tags) : '';
									
									$registry->set('categories',$categories);
									$registry->set('locations',$locations);
									$registry->set('tags',$tags);
									
									$db->setQuery("UPDATE `#__menu` SET `params` = ".$db->q($registry->toString())." WHERE `id` = ".(int) $calendarMenu->id." ");
									$db->execute();
								}
							}
							
							// Update events menus
							$db->setQuery("SELECT `id`, `params` FROM #__menu WHERE `link` LIKE 'index.php?option=com_rseventspro&view=rseventspro'");
							if ($eventsMenus = $db->loadObjectList()) {
								foreach ($eventsMenus as $eventsMenu) {
									$registry = new Registry;
									$registry->loadString($eventsMenu->params);
		
									$categories = $registry->get('categories');
									if (!empty($categories)) {
										$categories = explode(',',$categories);
										foreach ($categories as $i => $category) {
											$categories[$i] = $newids[$category];
										}
									} else {
										$categories = '';
									}
									
									$locations = $registry->get('locations');
									$locations = !empty($locations) ? explode(',',$locations) : '';
									
									$tags = $registry->get('tags');
									$tags = !empty($tags) ? explode(',',$tags) : '';
									
									$registry->set('categories',$categories);
									$registry->set('locations',$locations);
									$registry->set('tags',$tags);
									
									$db->setQuery("UPDATE `#__menu` SET `params` = ".$db->q($registry->toString())." WHERE `id` = ".(int) $eventsMenu->id." ");
									$db->execute();
								}
							}
							
							// Update map menus
							$db->setQuery("SELECT `id`, `params` FROM #__menu WHERE `link` LIKE 'index.php?option=com_rseventspro&view=rseventspro&layout=map'");
							if ($mapMenus = $db->loadObjectList()) {
								foreach ($mapMenus as $mapMenu) {
									$registry = new Registry;
									$registry->loadString($mapMenu->params);
		
									$categories = $registry->get('categories');
									if (!empty($categories)) {
										$categories = explode(',',$categories);
										foreach ($categories as $i => $category) {
											$categories[$i] = $newids[$category];
										}
									} else {
										$categories = '';
									}
									
									$locations = $registry->get('locations');
									$locations = !empty($locations) ? explode(',',$locations) : '';
									
									$tags = $registry->get('tags');
									$tags = !empty($tags) ? explode(',',$tags) : '';
									
									$registry->set('categories',$categories);
									$registry->set('locations',$locations);
									$registry->set('tags',$tags);
									
									$db->setQuery("UPDATE `#__menu` SET `params` = ".$db->q($registry->toString())." WHERE `id` = ".(int) $mapMenu->id." ");
									$db->execute();
								}
							}
						}
						
						// Drop groups permissions table
						$db->setQuery("DROP TABLE IF EXISTS `#__rseventspro_categories`");
						$db->execute();
					}
				}
			}
			
			// Check for the 'enable' field in the emails table
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_emails` WHERE `Field` = 'enable'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_emails` ADD `enable` TINYINT( 1 ) NOT NULL AFTER `type`");
				$db->execute();
			}
			
			// Set enable option to the notification emails
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_registration_enable'");
			$registration = $db->loadResult();
			
			if (!is_null($registration)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $registration." WHERE `type` = 'registration'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_registration_enable'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_activation_enable'");
			$activation = $db->loadResult();
			
			if (!is_null($activation)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $activation." WHERE `type` = 'activation'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_activation_enable'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_unsubscribe_enable'");
			$unsubscribe = $db->loadResult();
			
			if (!is_null($unsubscribe)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $unsubscribe." WHERE `type` = 'unsubscribe'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_unsubscribe_enable'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_denied_enable'");
			$denied = $db->loadResult();
			
			if (!is_null($denied)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $denied." WHERE `type` = 'denied'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_denied_enable'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_reminder_enable'");
			$reminder = $db->loadResult();
			
			if (!is_null($reminder)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $reminder." WHERE `type` = 'reminder'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_reminder_enable'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_preminder_enable'");
			$preminder = $db->loadResult();
			
			if (!is_null($preminder)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $preminder." WHERE `type` = 'preminder'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_preminder_enable'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'email_invite_enable'");
			$invite = $db->loadResult();
			
			if (!is_null($invite)) {
				$db->setQuery("UPDATE `#__rseventspro_emails` SET `enable` = ".(int) $invite." WHERE `type` = 'invite'");
				$db->execute();
				$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'email_invite_enable'");
				$db->execute();
			}
			
			// UPDATE event parameters
			$db->setQuery("SELECT `id`, `repeat_also`, `payments`, `options`, `gallery_tags`, `properties` FROM `#__rseventspro_events`");
			if ($events = $db->loadObjectList()) {
				foreach ($events as $event) {
					$repeat_also = $payments = $options = $gallery_tags = $properties = '';
					
					if (!empty($event->repeat_also)) {
						if (!$this->isJSON($event->repeat_also)) {
							$repeat_also = @unserialize($event->repeat_also);
							if ($repeat_also !== false) {
								$registry = new Registry;
								$registry->loadArray($repeat_also);
								$repeat_also = $registry->toString();
							}
						}
					}
					
					if (!empty($event->options)) {
						if (!$this->isJSON($event->options)) {
							$options = @unserialize($event->options);
							if ($options !== false) {
								$registry = new Registry;
								$registry->loadArray($options);
								$options = $registry->toString();
							}
						}
					}
					
					if (!empty($event->payments)) {
						if (!$this->isJSON($event->payments)) {
							$payments = explode(',',$event->payments);
							if ($payments !== false) {
								$registry = new Registry;
								$registry->loadArray($payments);
								$payments = $registry->toString();
							}
						}
					}
					
					if (!empty($event->gallery_tags)) {
						if (!$this->isJSON($event->gallery_tags)) {
							$gallery_tags = explode(',',$event->gallery_tags);
							if ($gallery_tags !== false) {
								$registry = new Registry;
								$registry->loadArray($gallery_tags);
								$gallery_tags = $registry->toString();
							}
						}
					}
					
					if (!empty($event->properties)) {
						if (!$this->isJSON($event->properties)) {
							$properties = explode(',',$event->properties);
							if ($properties !== false) {
								$registry = new Registry;
								$registry->loadArray($properties);
								$properties = $registry->toString();
							}
						}
					}
					
					if ($repeat_also) {
						$db->setQuery("UPDATE `#__rseventspro_events` SET `repeat_also` = '".$db->escape($repeat_also)."' WHERE `id` = ".(int) $event->id." ");
						$db->execute();
					}
					
					if ($payments) {
						$db->setQuery("UPDATE `#__rseventspro_events` SET `payments` = '".$db->escape($payments)."' WHERE `id` = ".(int) $event->id." ");
						$db->execute();
					}
					
					if ($options) {
						$db->setQuery("UPDATE `#__rseventspro_events` SET `options` = '".$db->escape($options)."' WHERE `id` = ".(int) $event->id." ");
						$db->execute();
					}
					
					if ($gallery_tags) {
						$db->setQuery("UPDATE `#__rseventspro_events` SET `gallery_tags` = '".$db->escape($gallery_tags)."' WHERE `id` = ".(int) $event->id." ");
						$db->execute();
					}
					
					if ($properties) {
						$db->setQuery("UPDATE `#__rseventspro_events` SET `properties` = '".$db->escape($properties)."' WHERE `id` = ".(int) $event->id." ");
						$db->execute();
					}
				}
			}
			
			// UPDATE locations parameters
			$db->setQuery("SELECT `id`, `gallery_tags` FROM `#__rseventspro_locations`");
			if ($locations = $db->loadObjectList()) {
				foreach ($locations as $location) {
					$gallery_tags = '';
					
					if (!empty($location->gallery_tags)) {
						if (!$this->isJSON($location->gallery_tags)) {
							$gallery_tags = explode(',',$location->gallery_tags);
							if ($gallery_tags !== false) {
								$registry = new Registry;
								$registry->loadArray($gallery_tags);
								$gallery_tags = $registry->toString();
							}
						}
					}
					
					if ($gallery_tags) {
						$db->setQuery("UPDATE `#__rseventspro_locations` SET `gallery_tags` = '".$db->escape($gallery_tags)."' WHERE `id` = ".(int) $location->id." ");
						$db->execute();
					}
				}
			}
			
			// Update menu
			$db->setQuery("SELECT `id`, `link` FROM #__menu WHERE `link` LIKE '%index.php?option=com_rseventspro&view=rseventspro&layout=show&cid=%'");
			if ($eventsLinks = $db->loadObjectList()) {
				$pattern = '#cid=([0-9]+)#is';
				foreach ($eventsLinks as $eventsLink) {
					preg_match($pattern,$eventsLink->link,$matches);
					if (!empty($matches[1])) $id = $matches[1]; else $id = 0;
					$db->setQuery("UPDATE `#__menu` SET `link` = 'index.php?option=com_rseventspro&view=rseventspro&layout=show&id=".(int) $id."' WHERE `id` = ".(int) $eventsLink->id." ");
					$db->execute();
				}
			}
			
			// START REV 5 UPDATE
			$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `late_fee` `late_fee` FLOAT NOT NULL");
			$db->execute();
			
			$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `early_fee` `early_fee` FLOAT NOT NULL");
			$db->execute();
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'position'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `position` TEXT NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'groups'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `groups` TEXT NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'ticketsconfig'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `ticketsconfig` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'featured'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `featured` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'event'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `event` TEXT NOT NULL");
				$db->execute();
			}
			// END REV 5 UPDATE
			
			// START VERSION 1.6.0 UPDATE
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'hits'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `hits` INT( 11 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'repeat_on_type'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `repeat_on_type` TINYINT( 1 ) NOT NULL AFTER `repeat_also`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'repeat_on_day'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `repeat_on_day` TINYINT( 2 ) NOT NULL AFTER `repeat_on_type`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'repeat_on_day_order'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `repeat_on_day_order` TINYINT( 1 ) NOT NULL AFTER `repeat_on_day`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'repeat_on_day_type'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `repeat_on_day_type` VARCHAR( 25 ) NOT NULL AFTER `repeat_on_day_order`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'create_user'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` ADD `create_user` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'confirmed'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` ADD `confirmed` TINYINT( 1 ) NOT NULL");
				$db->execute();
			}
			
			// END VERSION 1.6.0 UPDATE
			
			// Version 1.7.1
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'timezone'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `timezone` VARCHAR( 255 ) NOT NULL");
				$db->execute();
			}
			
			// Version 1.8.0
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'aspectratio'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `aspectratio` TINYINT( 1 ) NOT NULL");
				$db->execute();
				$db->setQuery("UPDATE `#__rseventspro_events` SET `aspectratio` = 1");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'exclude_dates'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `exclude_dates` TEXT NOT NULL AFTER `repeat_on_day_type`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'price'");
			if ($columns = $db->loadObject()) {
				if ($columns->Type == 'float') {
					$db->setQuery("ALTER TABLE `#__rseventspro_tickets` CHANGE `price` `price` DECIMAL( 20, 3 ) NOT NULL");
					$db->execute();
				}
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'restricted_categories'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `restricted_categories` TEXT NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SELECT `ide` FROM `#__rseventspro_taxonomy` WHERE `type` = 'postreminder'");
			if ($preminders = $db->loadColumn()) {
				foreach ($preminders as $preminder) {
					$db->setQuery("SELECT DISTINCT `email` FROM `#__rseventspro_users` WHERE `ide` = ".(int) $preminder." AND `state` = 1");
					if ($emails = $db->loadColumn()) {
						foreach ($emails as $email) {
							$db->setQuery("INSERT IGNORE INTO `#__rseventspro_taxonomy` SET `type` = 'preminder', `ide` = ".(int) $preminder.", `id` = '1', `extra` = ".$db->q($email)." ");
							$db->execute();
						}
					}
				}
				
				$db->setQuery("DELETE FROM `#__rseventspro_taxonomy` WHERE `type` = 'postreminder'");
				$db->execute();
			}
			
			// Version 1.9.0
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'itemid'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `itemid` INT( 11 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'order'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `order` INT( 11 ) NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_groups` WHERE `Field` = 'can_change_options'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_groups` ADD `can_change_options` TINYINT( 1 ) NOT NULL AFTER `can_approve_tags`");
				$db->execute();
			}
			
			// Version 1.10.0
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'attach'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `attach` TINYINT( 1 ) NOT NULL AFTER `groups`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'layout'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `layout` LONGTEXT NOT NULL AFTER `attach`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'small_description'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD `small_description` TEXT NOT NULL AFTER `description`");
				$db->execute();
			}
			
			// Index
			$db->setQuery("SHOW INDEX FROM #__rseventspro_events WHERE Key_name = 'location'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD INDEX (`location`)");
				$db->execute();
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_events WHERE Key_name = 'owner'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD INDEX (`owner`)");
				$db->execute();
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_events WHERE Key_name = 'completed'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD INDEX (`completed`)");
				$db->execute();
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_events WHERE Key_name = 'published'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD INDEX (`published`)");
				$db->execute();
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_events WHERE Key_name = 'published_2'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` ADD INDEX (`published`, `completed`)");
				$db->execute();
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_tickets WHERE Key_name = 'ide'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD INDEX (`ide`)");
				$db->execute();
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_tickets WHERE Key_name = 'price'");
			if (!$db->loadObject()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD INDEX (`price`)");
				$db->execute();
			}
			
			// Set event ticket layout to individual tickets
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'ticket_pdf'");
			if ($db->loadResult()) {
				$db->setQuery("SELECT `id`, `ticket_pdf`, `ticket_pdf_layout` FROM `#__rseventspro_events`");
				if ($events = $db->loadObjectList()) {
					foreach ($events as $event) {
						if (empty($event->ticket_pdf) && empty($event->ticket_pdf_layout)) continue;
						
						$db->setQuery('UPDATE `#__rseventspro_tickets` SET `attach` = '.$db->q($event->ticket_pdf).', `layout` = '.$db->q($event->ticket_pdf_layout).' WHERE `ide` = '.$db->q($event->id).' ');
						$db->execute();
					}
				}
				
				// Remove table fields
				$db->setQuery("ALTER TABLE `#__rseventspro_events` DROP `ticket_pdf`");
				$db->execute();
				$db->setQuery("ALTER TABLE `#__rseventspro_events` DROP `ticket_pdf_layout`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_locations` WHERE `Field` = 'marker'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_locations` ADD `marker` VARCHAR( 255 ) NOT NULL AFTER `coordinates`");
				$db->execute();
			}
			
			$db->setQuery("SHOW TABLES LIKE '#__rseventspro_fields'");
			$fieldsTable = $db->loadResult();
			
			if (!empty($fieldsTable)) {			
				$db->setQuery("SHOW INDEX FROM #__rseventspro_fields WHERE Column_name = 'idt'");
				if ($reht = $db->loadObject()) {
					if (empty($reht->Sub_part)) {
						$db->setQuery("ALTER TABLE #__rseventspro_fields DROP INDEX `idt`, ADD INDEX `idt` (`idt`(191)) USING BTREE");
						$db->execute();
					}
				}
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_config WHERE Column_name = 'name'");
			if ($cfgidx = $db->loadObject()) {
				if (empty($cfgidx->Sub_part)) {
					$db->setQuery("ALTER TABLE #__rseventspro_config DROP PRIMARY KEY, ADD PRIMARY KEY (`name`(191)) USING BTREE");
					$db->execute();
				}
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_countries WHERE Column_name = 'name'");
			if ($couidx = $db->loadObject()) {
				if (empty($couidx->Sub_part)) {
					$db->setQuery("ALTER TABLE #__rseventspro_countries DROP INDEX `name`, ADD UNIQUE `name` (`name`(191)) USING BTREE");
					$db->execute();
				}
			}
			
			$db->setQuery("SHOW INDEX FROM #__rseventspro_discounts WHERE Column_name = 'name'");
			if ($disidx = $db->loadObject()) {
				if (empty($disidx->Sub_part)) {
					$db->setQuery("ALTER TABLE #__rseventspro_discounts DROP INDEX `name`, ADD UNIQUE `name` (`name`(191)) USING BTREE");
					$db->execute();
				}
			}
			
			$db->setQuery("SHOW TABLES LIKE '".$db->getPrefix()."rseventspro_fields'");
			if ($db->loadResult()) {
				$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_fields` WHERE `Field` = 'idt'");
				$idtFieldsField = $db->loadObject();
				if (strtolower($idtFieldsField->Type) == 'varchar(255)') {
					$db->setQuery("ALTER TABLE `#__rseventspro_fields` CHANGE `idt` `idt` VARCHAR(11) NOT NULL DEFAULT ''");
					$db->execute();
				}
			}
			
			// Do we have mb4 utf8 support?
			$hasUTF8mb4Support = $db->hasUTF8mb4Support();
			
			// Set default values on database fields
			if ($tables = $db->getTableList()) {
				foreach ($tables as $table) {
					if (substr($table, 0, strlen($db->getPrefix().'rseventspro')) == $db->getPrefix().'rseventspro') {
						
						$db->setQuery('SHOW TABLE STATUS WHERE name like '.$db->q($table));
						if ($tableDetails = $db->loadObject()) {
							// Change table collation
							if ($hasUTF8mb4Support) {						
								if (strpos(strtolower($tableDetails->Collation), 'utf8_general') !== false) {
									$db->setQuery('ALTER TABLE '.$db->qn($table).' DEFAULT CHARSET=utf8mb4 COLLATE utf8mb4_unicode_ci;');
									$db->execute();
								}
							}
						}
						
						if ($fields = $db->getTableColumns($table, false)) {
							foreach ($fields as $field) {
								$fieldType = strtolower($field->Type);
								$fieldKey  = strtolower($field->Key);
								$collation = $field->Collation ? strtolower($field->Collation) : '';
								
								if ($hasUTF8mb4Support && strpos($collation, 'utf8_general') !== false && (strpos($fieldType, 'varchar') !== false || strpos($fieldType, 'text') !== false)) {
									$db->setQuery('ALTER TABLE '.$db->qn($table).' CHANGE '.$db->qn($field->Field).' '.$db->qn($field->Field).' '.$field->Type.' CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;');
									$db->execute();
								}
								
								if (strpos($fieldType, 'int') !== false || strpos($fieldType, 'float') !== false|| strpos($fieldType, 'decimal') !== false) {
									if ($fieldKey != 'pri') {
										$db->setQuery('ALTER TABLE '.$db->qn($table).' ALTER '.$db->qn($field->Field).' SET DEFAULT '.$db->q(0));
										$db->execute();
									}
								} elseif (strpos($fieldType, 'varchar') !== false) {
									$db->setQuery('ALTER TABLE '.$db->qn($table).' ALTER '.$db->qn($field->Field).' SET DEFAULT '.$db->q(''));
									$db->execute();
								} elseif (strpos($fieldType, 'datetime') !== false) {
									$db->setQuery('ALTER TABLE '.$db->qn($table).' ALTER '.$db->qn($field->Field).' SET DEFAULT '.$db->q($db->getNullDate()));
									$db->execute();
								}
							}
						}
					}
				}
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'hash'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` ADD `hash` VARCHAR( 255 ) NOT NULL DEFAULT '' AFTER `create_user`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'from'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `from` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00' AFTER `layout`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'to'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` ADD `to` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00' AFTER `from`");
				$db->execute();
			}
			
			try {			
				$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_discounts` WHERE `Field` = 'cart_tickets'");
				if (!$db->loadResult()) {
					$db->setQuery("ALTER TABLE `#__rseventspro_discounts` ADD `cart_tickets` INT(3) NOT NULL DEFAULT '0' AFTER `different_tickets`");
					$db->execute();
				}
			} catch (Exception $e) {}
			
			// Set enable option to the notification emails
			$token = false;
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'facebook_appid'");
			if ($fbappid = $db->loadResult()) {
				if ($fbappid == '340486642645761') {
					$db->setQuery("UPDATE `#__rseventspro_config` SET `value` = '' WHERE name = 'facebook_appid'");
					$db->execute();
					$token = true;
				}
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE name = 'facebook_secret'");
			if ($fbsecret = $db->loadResult()) {
				if ($fbsecret == 'fea413f9a085e01555de0e93848c2c4a') {
					$db->setQuery("UPDATE `#__rseventspro_config` SET `value` = '' WHERE name = 'facebook_secret'");
					$db->execute();
					$token = true;
				}
			}
			
			if ($token) {
				$db->setQuery("UPDATE `#__rseventspro_config` SET `value` = '' WHERE name = 'facebook_token'");
				$db->execute();
			}
			
			$db->setQuery("SHOW TABLES LIKE '".$db->getPrefix()."rseventspro_sync_log'");
			if ($db->loadResult()) {
				$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_sync_log` WHERE `Field` = 'page'");
				if ($columns = $db->loadObject()) {
					if (strtolower($columns->Type) == 'int(2)') {
						$db->setQuery("ALTER TABLE `#__rseventspro_sync_log` CHANGE `page` `page` VARCHAR(255) NOT NULL DEFAULT ''");
						$db->execute();
					}
				}
			}
			
			$updateData = array();
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_quota', 'type' => 'INT(11)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_guests', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_start', 'type' => 'DATETIME', 'default' => '0000-00-00 00:00:00');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_end', 'type' => 'DATETIME', 'default' => '0000-00-00 00:00:00');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_going', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_interested', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsvp_notgoing', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'event_ended', 'type' => 'TEXT');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'event_full', 'type' => 'TEXT');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_select_speakers', 'type' => 'TINYINT(2)', 'default' => '1');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_add_speaker', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'ticketinfo', 'type' => 'TINYINT(2)', 'default' => '0', 'after' => 'to');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'ticketinfolayout', 'type' => 'TEXT', 'after' => 'ticketinfo');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsm_enable', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsm_when', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'rsm_lists', 'type' => 'VARCHAR(255)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'waitinglist', 'type' => 'TINYINT(2)', 'default' => '0', 'after' => 'rsm_lists');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'waitinglist_limit', 'type' => 'INT(11)', 'default' => '0', 'after' => 'waitinglist');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'waitinglist_time', 'type' => 'VARCHAR(50)', 'default' => '', 'after' => 'waitinglist_limit');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'waitinglist_user', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'waitinglist_admin', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'notify_me_paid', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'tickets_amount', 'type' => 'INT(5)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_select_sponsors', 'type' => 'TINYINT(2)', 'default' => '1');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_add_sponsor', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_attach', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_type', 'type' => 'TINYINT(2)', 'default' => '1');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_font', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_orientation', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_padding', 'type' => 'INT(5)', 'default' => '7');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_prefix', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_title', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'invoice_layout', 'type' => 'TEXT');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_confirm_tickets', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'limit_events', 'type' => 'INT(11)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_speakers', 'field' => 'custom', 'type' => 'TEXT', 'after' => 'phone');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'notify_me_emails', 'type' => 'TEXT');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'notify_me_paid_emails', 'type' => 'TEXT');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'notify_me_unsubscribe_emails', 'type' => 'TEXT');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'start', 'type' => 'INT(11)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'disabled', 'type' => 'TINYINT(5)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'disabled_available', 'type' => 'TINYINT(5)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'disable_registration', 'type' => 'TINYINT(3)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'disable_activation', 'type' => 'TINYINT(3)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'disable_denied', 'type' => 'TINYINT(3)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'disable_refund', 'type' => 'TINYINT(3)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'available', 'type' => 'TINYINT(5)', 'default' => '1');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'disabled_type', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'manual_disabled', 'type' => 'VARCHAR(255)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_rules', 'field' => 'op', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'enable_registration', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_create_tickets', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'enable_discounts', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'default_category', 'type' => 'varchar(255)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'default_location', 'type' => 'TINYINT(10)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_view_subscriptions', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_edit_subscriptions', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_delete_subscriptions', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_change_subscriptions', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_locations', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_tags', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_payments', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_speakers', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_user_info', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_waitinglist', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_rsvp_users', 'field' => 'language', 'type' => 'VARCHAR(10)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_users', 'field' => 'client', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_unsubscribers', 'field' => 'type', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_unsubscribers', 'field' => 'idu', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'feedback', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'disable_feedback', 'type' => 'TINYINT(3)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_rules', 'field' => 'email', 'type' => 'VARCHAR(255)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'before_subscribe', 'type' => 'MEDIUMTEXT', 'null' => true);
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'after_subscribe', 'type' => 'MEDIUMTEXT', 'null' => true);
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'before_unsubscribe', 'type' => 'MEDIUMTEXT', 'null' => true);
			$updateData[] = array('table' => '#__rseventspro_events', 'field' => 'after_unsubscribe', 'type' => 'MEDIUMTEXT', 'null' => true);
			$updateData[] = array('table' => '#__rseventspro_groups', 'field' => 'can_scan', 'type' => 'TINYINT(2)', 'default' => '0');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'bg_ticket', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'bg_seat', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'bg_seat_text', 'type' => 'VARCHAR(50)', 'default' => '');
			$updateData[] = array('table' => '#__rseventspro_tickets', 'field' => 'bg_seat_selected', 'type' => 'VARCHAR(50)', 'default' => '');

			foreach ($updateData as $data) {
				$checkQuery = 'SHOW COLUMNS FROM '.$db->qn($data['table']).' WHERE '.$db->qn('Field').' = '.$db->q($data['field']);
				$updateQuery = 'ALTER TABLE '.$db->qn($data['table']).' ADD '.$db->qn($data['field']).' '.$data['type'];
				
				if (isset($data['null'])) $updateQuery .= " NULL"; else $updateQuery .= " NOT NULL";
				if (isset($data['default'])) $updateQuery .= " DEFAULT '".$data['default']."'";
				if (isset($data['after'])) $updateQuery .= ' AFTER '.$db->qn($data['after']);
				
				$db->setQuery($checkQuery);
				if (!$db->loadResult()) {
					$db->setQuery($updateQuery);
					$db->execute();
				}
			}
			
			// Set default language
			$tables = array('#__rseventspro_events', '#__rseventspro_locations', '#__rseventspro_tags', '#__rseventspro_payments', '#__rseventspro_speakers', '#__rseventspro_user_info', '#__rseventspro_tickets', '#__rseventspro_waitinglist', '#__rseventspro_rsvp_users');
			$params = ComponentHelper::getParams('com_languages');
			$defaultLanguage = $params->get(ApplicationHelper::getClientInfo(1)->name, 'en-GB');
			foreach ($tables as $table) {
				$db->setQuery("UPDATE `".$table."` SET `language` = '".$db->escape($defaultLanguage)."' WHERE `language` = ''");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tickets` WHERE `Field` = 'manual_disabled'");
			$ticketsTable = $db->loadObject();
			if (strtolower($ticketsTable->Type) == 'varchar(255)') {
				$db->setQuery("ALTER TABLE `#__rseventspro_tickets` CHANGE `manual_disabled` `manual_disabled` TEXT NOT NULL");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'location'");
			$eventsTable = $db->loadObject();
			if (strtolower($eventsTable->Type) == 'int(2)') {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `location` `location` INT(12) NOT NULL DEFAULT '0'");
				$db->execute();
			}
			
			// Check for email messages
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'report'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'report', 0, 1, 0, 'New report for {EventName}', '<p>Hello,</p>\r\n<p>A new report for <strong>{EventName}</strong> has been added. Here are the details for this report:</p>\r\n<p>User: {ReportUser}</p>\r\n<p>IP: {ReportIP}</p>\r\n<p>Message: {ReportMessage}</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'approval'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'approval', 1, 1, 0, 'Your event \"{EventName}\" has been approved.', '<p>Hello {Owner},</p>\r\n<p>Your event {EventName} has been approved by one of our staff members. You can view your event by clicking <a href=\"{EventLink}\">here</a>.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'rsvpgoing'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'rsvpgoing', 1, 1, 0, 'Going to {EventName}', '<p>Hello {user},</p>\r\n<p>Thank you for your participation to <strong>{EventName}</strong> that will start on {EventStartDate}.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'rsvpinterested'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'rsvpinterested', 1, 1, 0, 'Interested in going to {EventName}', '<p>Hello {user},</p>\r\n<p>Thank you for your interest in the <strong>{EventName}</strong> event. We hope to see you at this event.</p>\r\n<p>A quick reminder: this event starts on {EventStartDate} and ends on {EventEndDate}.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'rsvpnotgoing'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'rsvpnotgoing', 1, 1, 0, 'Not going to {EventName}', '<p>Hello {user},</p>\r\n<p>We are sorry to see that you cannot come to the <strong>{EventName}</strong> event. Hope you will change your mind.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'waitinglist'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'waitinglist', 1, 1, 0, 'Claim your ticket to {EventName}', '<p>Congratulations {user}!</p>\r\n<p>A spot has opened up for you at {EventName}. In order to register please click <a href=\"{claim}\">here</a>. Please note that you can claim your ticket before {claimdate}.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'waitinglistuser'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'waitinglistuser', 1, 1, 0, 'Waiting list confirmation for \"{EventName}\"', '<p>Hello <strong>{user}</strong>,</p>\r\n<p>Thank you for joining our waiting list for the event <strong>{EventName}</strong>.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'waitinglistadmin'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'waitinglistadmin', 1, 1, 0, 'New addition to \"{EventName}\" waiting list', '<p>Hello <strong>{OwnerName}</strong>,</p>\r\n<p><strong>{user}</strong> has been added to the waiting list of <strong>{EventName}</strong>.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'cancel'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'cancel', 1, 1, 0, 'Event {EventName} has been canceled.', '<p>Hello {user},</p>\r\n<p>We are sorry to say, that <strong>{EventName}</strong> has been canceled.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'notify_me_paid'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'notify_me_paid', 1, 1, 0, 'You have a new paid subscription for {EventName} from {SubscriberName}!', '<p>Hello <strong>{OwnerName}</strong>,</p>\r\n<p>The status of <em><strong>{SubscriberName}</strong></em>\'s subscription from the <em><strong>{EventName}</strong></em> event was completed and paid.</p>\r\n<p><strong>Subscriber info:</strong></p>\r\n<ul>\r\n<li>Date: {SubscribeDate}</li>\r\n<li>Username: {SubscriberUsername}</li>\r\n<li>Name: {SubscriberName}</li>\r\n<li>Email: {SubscriberEmail}</li>\r\n<li>IP: {SubscriberIP}</li>\r\n</ul>\r\n<p><strong>Payment related info (if available):</strong></p>\r\n<ul>\r\n<li>Gateway: {PaymentGateway}</li>\r\n<li>Tickets: {TicketInfo}</li>\r\n<li>Total: {TicketsTotal}</li>\r\n<li>Discount: {TicketsDiscount}</li>\r\n</ul>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'refund'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'refund', 1, 1, 0, 'Subscription refunded for {EventName}', '<p>Hello {user},</p>\r\n<p>Your subscription to {EventName} was refunded.</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'contact'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'contact', 1, 1, 0, 'New contact from {name} on {EventName}', '<p>Hello {OwnerName},</p>\r\n<p>{name} has submitted a contact form for the event {EventName}.</p>\r\n<p>{message}</p>');");
				$db->execute();
			}
			
			$db->setQuery("SELECT `id` FROM `#__rseventspro_emails` WHERE `lang` = 'en-GB' AND `type` = 'feedback'");
			if (!$db->loadResult()) {
				$db->setQuery("INSERT IGNORE INTO `#__rseventspro_emails` (`id`, `lang`, `type`, `enable`, `mode`, `parent`, `subject`, `message`) VALUES ('', 'en-GB', 'feedback', 1, 1, 0, 'A new feedback for {EventName} was added.', '<p>Hello {OwnerName},</p>\r\n<p>A new feedback has been added to <strong>{EventName}</strong>.</p>');");
				$db->execute();
			}
			
			// Add old rating to the new rating table
			$db->setQuery("SELECT * FROM `#__rseventspro_taxonomy` WHERE `type` = 'rating'");
			if ($ratings = $db->loadObjectList()) {
				foreach ($ratings as $rating) {
					$db->setQuery('INSERT INTO `#__rseventspro_rating` SET `ide` = '.$db->q($rating->ide).', `value` = '.$db->q($rating->id).', `ip` = '.$db->q($rating->extra));
					$db->execute();
				}
				
				$db->setQuery("DELETE FROM `#__rseventspro_taxonomy` WHERE `type` = 'rating'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE `name` = 'modal'");
			$modal = $db->loadResult();
			if ($modal == 2) {
				$db->setQuery("UPDATE `#__rseventspro_config` SET `value` = '1' WHERE `name` = 'modaltype'");
				$db->execute();
				$db->setQuery("UPDATE `#__rseventspro_config` SET `value` = '1' WHERE `name` = 'modal'");
				$db->execute();
			}
			
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE `name` = 'invoice_title'");
			$invoice_title = $db->loadResult();
			$db->setQuery("SELECT `value` FROM `#__rseventspro_config` WHERE `name` = 'invoice_layout'");
			$invoice_layout = $db->loadResult();
			
			if (!is_null($invoice_title) || !is_null($invoice_layout)) {
				$query = $db->getQuery(true);
				
				$query->clear()
					->update($db->qn('#__rseventspro_invoice'))
					->where($db->qn('ide').' = 0')
					->where($db->qn('lang').' = '.$db->q('en-GB'));
				
				if (!is_null($invoice_title)) {
					$query->set($db->qn('title').' = '.$db->q($invoice_title));
				}
				
				if (!is_null($invoice_layout)) {
					$query->set($db->qn('layout').' = '.$db->q($invoice_layout));
				}
				
				$db->setQuery($query);
				if ($db->execute()) {
					$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'invoice_title'");
					$db->execute();
					$db->setQuery("DELETE FROM `#__rseventspro_config` WHERE `name` = 'invoice_layout'");
					$db->execute();
				}
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'invoice_title'");
			$invoice_title = $db->loadResult();
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'invoice_layout'");
			$invoice_layout = $db->loadResult();
			
			if ($invoice_title || $invoice_layout) {
				$db->setQuery("SELECT id, invoice_title, invoice_layout FROM #__rseventspro_events WHERE invoice_title <> '' OR invoice_layout <> ''");
				if ($events = $db->loadObjectList()) {
					$query = $db->getQuery(true);
					
					foreach ($events as $event) {
						$query->clear()
							->insert($db->qn('#__rseventspro_invoice'))
							->set($db->qn('ide').' = '.$db->q($event->id))
							->set($db->qn('title').' = '.$db->q($event->invoice_title))
							->set($db->qn('layout').' = '.$db->q($event->invoice_layout))
							->set($db->qn('lang').' = '.$db->q('en-GB'));
						$db->setQuery($query);
						$db->execute();
					}
				}
				
				$db->setQuery("ALTER TABLE `#__rseventspro_events` DROP `invoice_title`");
				$db->execute();
				
				$db->setQuery("ALTER TABLE `#__rseventspro_events` DROP `invoice_layout`");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_reports` WHERE `Field` = 'ip'");
			$ipRepField = $db->loadObject();
			if (strtolower($ipRepField->Type) == 'varchar(15)') {
				$db->setQuery("ALTER TABLE `#__rseventspro_reports` CHANGE `ip` `ip` VARCHAR(50) NOT NULL DEFAULT ''");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_users` WHERE `Field` = 'ip'");
			$ipUsrField = $db->loadObject();
			if (strtolower($ipUsrField->Type) == 'varchar(20)') {
				$db->setQuery("ALTER TABLE `#__rseventspro_users` CHANGE `ip` `ip` VARCHAR(50) NOT NULL DEFAULT ''");
				$db->execute();
			}
			
			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_tmp` WHERE `Field` = 'table'");
			$tableTmpField = $db->loadObject();
			if (strtolower($tableTmpField->Type) == 'varchar(32)') {
				$db->setQuery("ALTER TABLE `#__rseventspro_tmp` CHANGE `table` `table` VARCHAR(50) NOT NULL DEFAULT ''");
				$db->execute();
			}

			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'before_subscribe'");
			$before_subscribeField = $db->loadObject();
			if (strtolower($before_subscribeField->Null) == 'no') {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `before_subscribe` `before_subscribe` mediumtext NULL");
				$db->execute();
			}

			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'after_subscribe'");
			$after_subscribeField = $db->loadObject();
			if (strtolower($after_subscribeField->Null) == 'no') {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `after_subscribe` `after_subscribe` mediumtext NULL");
				$db->execute();
			}

			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'before_unsubscribe'");
			$before_unsubscribeField = $db->loadObject();
			if (strtolower($before_unsubscribeField->Null) == 'no') {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `before_unsubscribe` `before_unsubscribe` mediumtext NULL");
				$db->execute();
			}

			$db->setQuery("SHOW COLUMNS FROM `#__rseventspro_events` WHERE `Field` = 'after_unsubscribe'");
			$after_unsubscribeField = $db->loadObject();
			if (strtolower($after_unsubscribeField->Null) == 'no') {
				$db->setQuery("ALTER TABLE `#__rseventspro_events` CHANGE `after_unsubscribe` `after_unsubscribe` mediumtext NULL");
				$db->execute();
			}
		}
		
		$jversion = new Version();
		if ($jversion->isCompatible('3.0')) {
			if ($content = Table::getInstance('Contenttype', 'JTable')) {
				if (!$content->load(array('type_alias' => 'com_rseventspro.categories'))) {
					$content->save(array(
						'type_title' => 'RSEvents! Pro Category',
						'type_alias' => 'com_rseventspro.category',
						'table'		 => '{"special":{"dbtable":"#__categories","key":"id","type":"Category","prefix":"JTable","config":"array()"},"common":{"dbtable":"#__ucm_content","key":"ucm_id","type":"Corecontent","prefix":"JTable","config":"array()"}}',
						'field_mappings' => '{"common":{"core_content_item_id":"id","core_title":"title","core_state":"published","core_alias":"alias","core_created_time":"created_time","core_modified_time":"modified_time","core_body":"description", "core_hits":"hits","core_publish_up":"null","core_publish_down":"null","core_access":"access", "core_params":"params", "core_featured":"null", "core_metadata":"metadata", "core_language":"language", "core_images":"null", "core_urls":"null", "core_version":"version", "core_ordering":"null", "core_metakey":"metakey", "core_metadesc":"metadesc", "core_catid":"parent_id", "core_xreference":"null", "asset_id":"asset_id"}, "special":{"parent_id":"parent_id","lft":"lft","rgt":"rgt","level":"level","path":"path","extension":"extension","note":"note"}}',
					));
				}
			}
		}
		
		// Unpublish the RSMediaGallery! plugin
		$db->setQuery("SELECT `extension_id`, `name` FROM `#__extensions` WHERE `type` = 'plugin' AND `element` = 'rsmediagallery' AND `folder` = 'rseventspro'");
		if ($gallery = $db->loadObject()) {
			$db->setQuery("UPDATE `#__extensions` SET `enabled` = 0 , `name` = '".$db->escape($gallery->name.' (Plugin no longer available!) ')."' WHERE `extension_id` = ".(int) $gallery->extension_id." ");
			$db->execute();
		}
	}
	
	// Set the install message
	public function showinstall($messages) {
?>
<style type="text/css">
#rsepro-installer-left {
	float: left;
	width: 18%;
	padding: 5px;
}

#rsepro-installer-right {
	float: left;
	padding: 5px;
	width: 70%;
}

.version-history {
	margin: 0 0 2em 0;
	padding: 0;
	list-style-type: none;
}

.version-history > li {
	margin: 0 0 0.5em 0;
	padding: 0 0 0 4em;
}

.version,
.version-new,
.version-fixed,
.version-upgraded {
	float: left;
	font-size: 0.8em;
	margin-left: -4.9em;
	width: 4.5em;
	color: white;
	text-align: center;
	font-weight: bold;
	text-transform: uppercase;
	-webkit-border-radius: 4px;
	-moz-border-radius: 4px;
	border-radius: 4px;
}

.version { background: #000; }
.version-new { background: #7dc35b; }
.version-fixed { background: #e9a130; }
.version-upgraded { background: #61b3de; }

.com-rseventspro-button {
	display: inline-block;
	background: #459300 none repeat scroll 0 0;
	color: #fff !important;
	cursor: pointer;
	margin-bottom: 10px;
    padding: 7px;
	text-decoration: none !important;
}

.rsepro-messages {
	padding: 8px 35px 8px 14px;
	margin-bottom: 18px;
	text-shadow: 0 1px 0 rgba(255,255,255,0.5);
	background-color: #f2dede;
	border-color: #ebccd1;
	color: #a94442;
	-webkit-border-radius: 4px;
	-moz-border-radius: 4px;
	border-radius: 4px;
}

.rsepro-messages > p {
    margin: 0 0 5px !important;
}
</style>

<div id="rsepro-installer-left">
	<img src="<?php echo Uri::root(); ?>media/com_rseventspro/images/rseventspro-logo.png" alt="RSEvents!Pro" />
</div>
<div id="rsepro-installer-right">
	<?php if ($messages) { ?>
	<div class="rsepro-messages">
		<?php foreach ($messages as $message) { ?>
			<p><i class="icon-info"></i> <?php echo $message; ?></p>
		<?php } ?>
	</div>
	<?php } ?>
	<h2>Changelog v1.14.7</h2>
	<ul class="version-history">
		<li><span class="version-new">Add</span> Customize ticket seats box. </li>
		<li><span class="version-new">Add</span> Add new simple event modal from backend. </li>
		<li><span class="version-new">Add</span> Reset event hits. </li>
		<li><span class="version-fixed">Fix</span> Miscellaneous code improvements. </li>
	</ul>
	<a class="com-rseventspro-button" href="index.php?option=com_rseventspro">Go to RSEvents!Pro</a>
	<a class="com-rseventspro-button" href="https://www.rsjoomla.com/support/documentation/rseventspro.html" target="_blank">Read the Documentation</a>
	<a class="com-rseventspro-button" href="https://www.rsjoomla.com/customer-support/tickets.html" target="_blank">Get Support!</a>
</div>
<div style="clear: both;"></div>
<?php
	}
	
	protected function renderTree($array, &$tree=array(), &$levels=array(), $parent=0, $level=0) {
		foreach ($array as $row) {
			if ($row->parent == $parent) {
				$levels[$row->id] 	= $level;
				$tree[$row->id]		= array();
				$this->renderTree($array, $tree[$row->id], $levels, $row->id, $level+1);
			}
		}
	}
	
	protected function renderFlatTree($tree) {
		$list = array();
		foreach($tree as $key => $children) {
			$list[] = $key;
			if (count($children)) {
				$tmp_list = $this->renderFlatTree($children);
				foreach ($tmp_list as $tmp_key)
					$list[] = $tmp_key;
			}
		}

		return $list;
	}
	
	protected function isJSON($string) {
		$data 	= json_decode($string);
		
		if (version_compare(PHP_VERSION,'5.3.0','>='))
			$valid	= json_last_error() == JSON_ERROR_NONE;
		else $valid = !is_null($data);
		
		if ($valid) {
			return is_array($data) || is_object($data);
		} else return $valid;
	}
	
	protected function checkAddons() {
		$messages	= array();
		$lang		= Factory::getLanguage();
		
		$plugins = array(
			'content' => array(
				'rseventspro' => '1.5'
			),
			'finder' => array(
				'rseventspro' => '1.5'
			),
			'system' => array(
				'rseprocart' => '1.3.2',
				'rsepropdf' => '1.22',
				'rseprooffline' => '1.5',
				'rsfprseventspro' => '3.2.1',
				
				'rsepro2co' => '1.4',
				'rseproanzegate' => '1.4',
				'rseproauthorize' => '1.5',
				'rseproeway' => '1.7',
				'rseproideal' => '1.12',
				'rseproiyzico' => '1.1',
				'rsepropayfast' => '1.3',
				'rsepropaypal' => '1.6',
				'rseprosquare' => '1.3',
				'rseprostripe' => '1.10',
				'rseprovmerchant' => '1.4'
			)
		);
		
		// Check plugins version
		if ($installed = $this->getPlugins($plugins)) {
			foreach ($installed as $plugin) {
				$file = JPATH_SITE.'/plugins/'.$plugin->folder.'/'.$plugin->element.'/'.$plugin->element.'.xml';
				if (file_exists($file)) {
					$xml		= file_get_contents($file);
					$version	= $plugins[$plugin->folder][$plugin->element];
					
					if ($this->checkVersion($xml, $version, '>') || strpos($xml, '<extension') === false) {
						$lang->load('plg_'.$plugin->folder.'_'.$plugin->element, JPATH_ADMINISTRATOR);
						//$this->disableExtension($plugin->extension_id);
						$messages[] = 'Please update the plugin "'.Text::_($plugin->name).'" manually.';
					}
				}
			}
		}
		
		$modules = array(
			'mod_rseventspro_archived' => '1.7',
			'mod_rseventspro_attendees' => '1.5',
			'mod_rseventspro_calendar' => '1.13',
			'mod_rseventspro_categories' => '1.7',
			'mod_rseventspro_events' => '1.11',
			'mod_rseventspro_featured' => '1.10',
			'mod_rseventspro_location' => '1.8',
			'mod_rseventspro_locations' => '1.5',
			'mod_rseventspro_map' => '1.14',
			'mod_rseventspro_popular' => '1.10',
			'mod_rseventspro_related' => '1.1',
			'mod_rseventspro_search' => '1.9',
			'mod_rseventspro_slider' => '1.11',
			'mod_rseventspro_tags' => '1.4',
			'mod_rseventspro_tickets' => '1.4',
			'mod_rseventspro_upcoming' => '1.10'
		);
		
		// Check modules version
		if ($installed = $this->getModules($modules)) {
			foreach ($installed as $module) {
				$file = JPATH_SITE.'/modules/'.$module->element.'/'.$module->element.'.xml';
				if (file_exists($file)) {
					$xml = file_get_contents($file);
					
					if ($this->checkVersion($xml, $modules[$module->element], '>') || strpos($xml, '<install') !== false) {
						$lang->load($module->element, JPATH_SITE);
						//$this->unpublishModule($module->element);
						$messages[] = 'Please update the module "'.Text::_($module->name).'" manually.';
					}
				}
			}
		}
		
		return $messages;
	}
	
	protected function disableExtension($extension_id) {
		$db = Factory::getDbo();
		
		$query = $db->getQuery(true)->update('#__extensions')
			->set($db->qn('enabled').'='.$db->q(0))
			->where($db->qn('extension_id').'='.$db->q($extension_id));
		
		$db->setQuery($query);
		$db->execute();
	}
	
	protected function unpublishModule($module) {
		$db = Factory::getDbo();
		
		$query = $db->getQuery(true)->update('#__modules')
			->set($db->qn('published').'='.$db->q(0))
			->where($db->qn('module').'='.$db->q($module));
		
		$db->setQuery($query);
		$db->execute();
	}
	
	protected function getModules($modules) {
		$db			= Factory::getDbo();
		$elements	= array_keys($modules);
		
		$query = $db->getQuery(true)->select('*')
			->from('#__extensions')
			->where($db->qn('type').'='.$db->q('module'))
			->where($db->qn('element').' IN ('.$this->quoteImplode($elements).')');
		$db->setQuery($query);
		
		return $db->loadObjectList();
	}
	
	protected function getPlugins($plugins) {
		$db			= Factory::getDbo();
		$folders	= array_keys($plugins);
		$elements	= array();
		
		foreach ($folders as $folder) {
			$elements = array_merge($elements , array_keys($plugins[$folder]));
		}
		
		$query = $db->getQuery(true)->select('*')
			->from('#__extensions')
			->where($db->qn('type').'='.$db->q('plugin'))
			->where($db->qn('folder').' IN ('.$this->quoteImplode($folders).')')
			->where($db->qn('element').' IN ('.$this->quoteImplode($elements).')');
		$db->setQuery($query);
		
		return $db->loadObjectList();
	}
	
	protected function quoteImplode($array) {
		$db = Factory::getDbo();
		foreach ($array as $k => $v) {
			$array[$k] = $db->q($v);
		}
		
		return implode(',', $array);
	}
	
	protected function escape($string) {
		return htmlentities($string, ENT_COMPAT, 'utf-8');
	}
	
	protected function checkVersion($string, $version, $operator = '>') {
		preg_match('#<version>(.*?)<\/version>#is',$string,$match);
		if (isset($match) && isset($match[1])) {
			return version_compare($version,$match[1],$operator);
		}
		
		return false;
	}
}