<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Table\Table;
use Joomla\CMS\Factory;

class RseventsproTableSubscription extends Table
{
	public function __construct($db) {
		parent::__construct('#__rseventspro_users', 'id', $db);
	}

	public function bind($array, $ignore = '') {
		if (!isset($array['id']) || empty($array['id'])) {
			if ($fields = $this->getFields()) {
				foreach ($fields as $key => $field) {
					if (!isset($array[$key])) {
						if (strpos(strtolower($field->Type), 'int') !== false || strpos(strtolower($field->Type), 'float') !== false) {
							$array[$key] = 0;
						} elseif (strpos(strtolower($field->Type), 'datetime') !== false) {
							$array[$key] = $this->getDbo()->getNullDate();
						} else {
							$array[$key] = '';
						}
					}
				}
			}
		}
		
		return parent::bind($array, $ignore);
	}
	public function check() {
		if (!$this->id) {
			$this->date = Factory::getDate()->toSql();
			$this->verification = md5($this->ide.$this->name);
			$this->ip = rseventsproHelper::getIP();
			$this->lang = Factory::getLanguage()->getTag();
			
			if (rseventsproHelper::getConfig('create_user') == 2) {
				$this->create_user = 1;
			}

			if ($this->gateway == 'none') {
				$this->gateway = '';
			}
		}

		return true;
	}

	public function delete($pk = null, $children = false) {
		$db		= $this->getDBO();
		$query	= $db->getQuery(true);
		
		// Remove the logs
		$query->clear()
			->delete($db->qn('#__rseventspro_user_logs'))
			->where($db->qn('ids').' = '.(int) $pk);
		
		$db->setQuery($query);
		$db->execute();
		
		// Remove the tickets
		$query->clear()
			->delete($db->qn('#__rseventspro_user_tickets'))
			->where($db->qn('ids').' = '.(int) $pk);
		
		$db->setQuery($query);
		$db->execute();
		
		// Remove ticket seats
		$query->clear()
			->delete($db->qn('#__rseventspro_user_seats'))
			->where($db->qn('ids').' = '.(int) $pk);
		
		$db->setQuery($query);
		$db->execute();
		
		// Remove confirmed tickets
		$query->clear()
			->delete($db->qn('#__rseventspro_confirmed'))
			->where($db->qn('ids').' = '.(int) $pk);
		
		$db->setQuery($query);
		$db->execute();
		
		// Remove rsmail data
		$query->clear()
			->delete($db->qn('#__rseventspro_rsmail'))
			->where($db->qn('ids').' = '.(int) $pk);
		
		$db->setQuery($query);
		$db->execute();
		
		$query->clear()
			->select($db->qn('e.id'))->select($db->qn('e.sync'))
			->select($db->qn('u.SubmissionId'))->select($db->qn('u.coupon'))
			->select($db->qn('u.ide'))->select($db->qn('u.name'))->select($db->qn('u.email'))
			->from($db->qn('#__rseventspro_users','u'))
			->join('left', $db->qn('#__rseventspro_events','e').' ON '.$db->qn('e.id').' = '.$db->qn('u.ide'))
			->where($db->qn('u.id').' = '.(int) $pk);
		
		$db->setQuery($query);
		$subscription = $db->loadObject();
		
		// Remove coupon usage
		if ($subscription->coupon) {
			$found = false;
			$query->clear()
				->select($db->qn('cc.id'))->select($db->qn('cc.used'))
				->from($db->qn('#__rseventspro_coupon_codes', 'cc'))
				->join('LEFT', $db->qn('#__rseventspro_coupons', 'c').' ON '.$db->qn('c.id').' = '.$db->qn('cc.idc'))
				->where($db->qn('c.ide').' = '.$db->q($subscription->id))
				->where($db->qn('cc.code').' = '.$db->q($subscription->coupon));
			$db->setQuery($query);
			if ($couponInfo = $db->loadObject()) {
				if ($couponInfo->used > 0) {
					$newUsed = (int) $couponInfo->used - 1;
					$query->clear()
						->update($db->qn('#__rseventspro_coupon_codes'))
						->set($db->qn('used').' = '.$db->q($newUsed))
						->where($db->qn('id').' = '.$db->q($couponInfo->id));
					$db->setQuery($query);
					$db->execute();
					$found = true;
				}
			}
			
			if (!$found) {
				$query->clear()
					->select($db->qn('id'))->select($db->qn('used'))
					->from($db->qn('#__rseventspro_discounts'))
					->where($db->qn('code').' = '.$db->q($subscription->coupon));
				$db->setQuery($query);
				if ($couponInfo = $db->loadObject()) {
					if ($couponInfo->used > 0) {
						$newUsed = (int) $couponInfo->used - 1;
						$query->clear()
							->update($db->qn('#__rseventspro_discounts'))
							->set($db->qn('used').' = '.$db->q($newUsed))
							->where($db->qn('id').' = '.$db->q($couponInfo->id));
						$db->setQuery($query);
						$db->execute();
						$found = true;
					}
				}
			}
		}
		
		$userID = isset($this->location) ? $this->location : Factory::getUser()->get('id');
		
		// Add to unsubscribers table
		if ($subscription->ide) {
			$query->clear()
				->insert($db->qn('#__rseventspro_unsubscribers'))
				->set($db->qn('ide').' = '.$db->q($subscription->ide))
				->set($db->qn('name').' = '.$db->q($subscription->name))
				->set($db->qn('email').' = '.$db->q($subscription->email))
				->set($db->qn('type').' = '.$db->q(2))
				->set($db->qn('idu').' = '.$db->q($userID))
				->set($db->qn('date').' = '.$db->q(Factory::getDate()->toSql()));
			$db->setQuery($query);
			$db->execute();
		} else {
			$query->clear()
				->select($db->qn('e.id'))
				->from($db->qn('#__rseventspro_events','e'))
				->join('left',$db->qn('#__rseventspro_cart','c').' ON '.$db->qn('c.ids').' = '.$db->q($pk))
				->where('FIND_IN_SET('.$db->qn('e.id').', '.$db->qn('c.events').') <> 0');
			
			$db->setQuery($query);
			if ($events = $db->loadColumn()) {
				foreach ($events as $event) {
					$query->clear()
						->insert($db->qn('#__rseventspro_unsubscribers'))
						->set($db->qn('ide').' = '.$db->q($event))
						->set($db->qn('name').' = '.$db->q($subscription->name))
						->set($db->qn('email').' = '.$db->q($subscription->email))
						->set($db->qn('type').' = '.$db->q(2))
						->set($db->qn('idu').' = '.$db->q($userID))
						->set($db->qn('date').' = '.$db->q(Factory::getDate()->toSql()));
					$db->setQuery($query);
					$db->execute();
				}
			}
		}
		
		// Delete RSForm!Pro submission
		if (file_exists(JPATH_SITE.'/components/com_rsform/rsform.php') && $subscription->sync) {
			$query->clear()
				->delete()
				->from($db->qn('#__rsform_submission_values'))
				->where($db->qn('SubmissionId').' = '.(int) $subscription->SubmissionId);
			
			$db->setQuery($query);
			$db->execute();
			
			$query->clear()
				->delete()
				->from($db->qn('#__rsform_submissions'))
				->where($db->qn('SubmissionId').' = '.(int) $subscription->SubmissionId);
			
			$db->setQuery($query);
			$db->execute();
		}
		
		Factory::getApplication()->triggerEvent('onrsepro_beforeDeleteSubscription', array(array('id' => $pk)));
		
		return parent::delete($pk, $children);
	}
}