<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Application\ApplicationHelper;
use Joomla\CMS\Language\Multilanguage;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Http\HttpFactory;
use Joomla\CMS\Mail\MailHelper;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;

class RseventsproController extends BaseController
{
	public function display($cachable = false, $urlparams = false) {
		// Set the default view name and format from the Request.
		$views = array('rseventspro', 'calendar', 'subscriptions');
		$vName = $this->input->getCmd('view', 'rseventspro');
		$layout = $this->input->getCmd('layout', '');
		
		if (!in_array($vName, $views)) {
			$this->input->set('view', 'rseventspro');
		}
		
		if (rseventsproHelper::isJ4()) {
			if (($vName == 'calendar' && !in_array($layout, array('day', 'week', 'items', 'module', '', 'default'))) || ($vName == 'subscriptions' && !in_array($layout, array('', 'default')))) {
				$this->input->set('view', 'rseventspro');
			}
		}
		
		parent::display($cachable, $urlparams);
		return $this;
	}
	
	/**
	 *	Method to display location results
	 *
	 * @return void
	 */
	public function locations() {
		echo rseventsproHelper::filterlocations();
		exit();
	}
	
	/**
	 *	Method to verify a certain coupon code
	 *
	 * @return void
	 */
	public function verify() {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$input		= Factory::getApplication()->input;
		$id			= $input->getInt('id');
		$coupon		= $input->getString('coupon');
		$payment	= $input->getString('payment');
		$nowunix	= Factory::getDate()->toUnix();
		$available	= false;
		$data		= false;
		$tickets	= array();
		$total		= 0;
		
		$query->clear()
			->select($db->qn('ticketsconfig'))->select($db->qn('discounts'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$event = $db->loadObject();

		if (!$event->discounts) {
			echo 'RS_DELIMITER0';
			echo Text::_('COM_RSEVENTSPRO_COUPON_ERROR');
			echo 'RS_DELIMITER1';
			exit();
		}

		if ($event->ticketsconfig) {
			$thetickets	= $input->get('tickets', array(), 'array');
			$unlimited	= $input->get('unlimited', array(), 'array');
			
			foreach ($thetickets as $tid => $theticket) {
				$tickets[$tid] = count($theticket);
			}
			
			if (!empty($unlimited)) {
				$unlimited = array_map('intval',$unlimited);
				foreach ($unlimited as $unlimitedid => $quantity)
					$tickets[$unlimitedid] = $quantity;
			}
		} else {
			$tickets = $input->get('tickets',array(),'array');
		}
		
		if ($tickets) {
			foreach ($tickets as $tid => $quantity) {
				$query->clear()
					->select($db->qn('price'))
					->from($db->qn('#__rseventspro_tickets'))
					->where($db->qn('id').' = '.(int) $tid);
				
				$db->setQuery($query);
				if ($price = $db->loadResult()) {
					$total += (int) $quantity * $price;
				}
			}
		}
		
		$global = rseventsproHelper::globalDiscount($id, $total, $tickets, $payment);
		
		if ($global) {
			$available = true;
		} else {
			$query->clear()
				->select($db->qn('cc.id'))->select($db->qn('cc.used'))->select($db->qn('c.from'))
				->select($db->qn('c.to'))->select($db->qn('c.usage'))
				->from($db->qn('#__rseventspro_coupon_codes','cc'))
				->join('left', $db->qn('#__rseventspro_coupons','c').' ON '.$db->qn('cc.idc').' = '.$db->qn('c.id'))
				->where($db->qn('c.ide').' = '.$id)
				->where('BINARY '.$db->qn('cc.code').' = '.$db->q($coupon));
			
			$db->setQuery($query);
			if ($data = $db->loadObject()) {
				$available = true;
				if (!empty($data->usage) && !empty($data->used))
					if ($data->used >= $data->usage)
						$available = false;
				
				if ($available) {
					if ($data->from == $db->getNullDate()) $data->from = '';
					if ($data->to == $db->getNullDate()) $data->to = '';
					
					if (empty($data->from) && empty($data->to)) {
						$available = true;
					} elseif (!empty($data->from) && empty($data->to)) {
						$fromunix = Factory::getDate($data->from)->toUnix();
						if ($fromunix <= $nowunix)
							$available = true;
						else $available = false;
					} elseif (empty($data->from) && !empty($data->to)) {
						$tounix = Factory::getDate($data->to)->toUnix();
						if ($tounix <= $nowunix)
							$available = false;
						else $available = true;
					} else {
						$fromunix = Factory::getDate($data->from)->toUnix();
						$tounix = Factory::getDate($data->to)->toUnix();
						
						if (($fromunix <= $nowunix && $tounix >= $nowunix) || ($fromunix >= $nowunix && $tounix <= $nowunix))
							$available = true;
						else $available = false;
					}
				}
			}
		}
		
		echo 'RS_DELIMITER0';
		if ($available) {
			echo Text::_('COM_RSEVENTSPRO_COUPON_OK');
		} else echo Text::_('COM_RSEVENTSPRO_COUPON_ERROR');
		echo 'RS_DELIMITER1';
		exit();
	}
	
	/**
	 *	Method to clear filters
	 *
	 * @return void
	 */
	public function clear() {
		$app		= Factory::getApplication();
		$itemid		= $app->input->getInt('Itemid',0);
		$parent		= $app->input->getInt('parent',0);
		$from		= $app->input->get('from');
		
		$app->setUserState('com_rseventspro.events.filter_columns'.$itemid.$parent,array());
		$app->setUserState('com_rseventspro.events.filter_operators'.$itemid.$parent,array());
		$app->setUserState('com_rseventspro.events.filter_values'.$itemid.$parent,array());
		
		if ($from == 'map')
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=map',false));
		else
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false));
	}
	
	/**
	 *	Method to load search results
	 *
	 * @return void
	 */
	public function filter() {
		$method = Factory::getApplication()->input->get('method','');
		if (!$method) echo 'RS_DELIMITER0';
		echo rseventsproHelper::filter();
		if (!$method) echo 'RS_DELIMITER1';
		exit();
	}
	
	/**
	 *	Method to get the number of allowed tickets a users can purchase
	 *
	 * @return string
	 */
	public function tickets() {
		$id = Factory::getApplication()->input->getInt('id');
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('ide'))->select($db->qn('description'))
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($ticket = $db->loadObject()) {
			// Replace translations
			if ($translation = RSEventsProTranslations::getTranslation('ticket', $id, 'description')) {
				$ticket->description = $translation;
			}
		}
		
		$w8list	 = rseventsproHelper::validWaitingList($ticket->ide);
		$seats	 = $w8list ? 1 : rseventsproHelper::checkticket($id);
		
		echo 'RS_DELIMITER0'.$seats.'|'.$ticket->description.'RS_DELIMITER1';
		exit();
	}
	
	/**
	 *	Method to generate the captcha image
	 *
	 * @return image
	 */
	public function captcha() {
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/captcha/image.php';
		$captcha = new RSCaptcha();
	}
	
	/**
	 *	Method to check captcha
	 *
	 * @return int
	 */
	public function checkcaptcha() {
		$session	= Factory::getSession();
		$input		= Factory::getApplication()->input;
		$secret		= $input->getString('secret');
		$response	= $input->getString('recaptcha');
		$ip		  	= $input->server->get('REMOTE_ADDR');
		$config		= rseventsproHelper::getConfig();
		$captcha_use= $config->captcha_use ? explode(',',$config->captcha_use) : array();
		$key		= $config->recaptcha_secret_key;
		
		echo 'RS_DELIMITER0';
		
		if (in_array(1,$captcha_use)) {
			if ($config->captcha == 1) {
				echo ($session->get('security_number') == $secret) ? 1 : 0;
			} else {
				try {
					$http = HttpFactory::getHttp();
					if ($request = $http->get('https://www.google.com/recaptcha/api/siteverify?secret='.urlencode($key).'&response='.urlencode($response).'&remoteip='.urlencode($ip))) {
						$json = json_decode($request->body);
						$captcha_response = $json->success;
					}
				} catch (Exception $e) {
					$captcha_response = false;
				}
				
				echo (int) $captcha_response;
			}
		} else {
			echo 1;
		}
		
		echo 'RS_DELIMITER1';
		exit();
	}
	
	/**
	 *	Method to show payment form
	 *
	 * @return 
	 */
	public function payment() {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$app		= Factory::getApplication();
		$method 	= $app->input->getString('method');
		$hash		= $app->input->getString('hash');
		$currency	= rseventsproHelper::getConfig('payment_currency');
		$uri		= Uri::getInstance();
		$total		= 0;
		$info		= array();
		$cart		= false;
		
		$query->clear()
			->select($db->qn('u.id'))->select($db->qn('u.ide'))->select($db->qn('u.idu'))->select($db->qn('u.name'))
			->select($db->qn('u.email'))->select($db->qn('u.discount'))->select($db->qn('u.early_fee'))->select($db->qn('u.late_fee'))
			->select($db->qn('u.tax'))->select($db->qn('u.verification'))->select($db->qn('u.state'))
			->from($db->qn('#__rseventspro_users','u'))
			->where('MD5(CONCAT('.$db->qn('u.id').','.$db->qn('u.name').','.$db->qn('u.email').')) = '.$db->q($hash));
		
		$db->setQuery($query);
		$details = $db->loadObject();
		
		if (empty($details)) {
			$this->setMessage(Text::_('COM_RSEVENTSPRO_INVALID_SUBSCRIPTION'),'error');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro',false));
		}
		
		if ($details->state == 1 || $details->state == 2) {
			if ($details->state == 1) {
				$this->setMessage(Text::_('COM_RSEVENTSPRO_SUBSCRIPTION_STATE_COMPLETE'));
			} else {
				$this->setMessage(Text::_('COM_RSEVENTSPRO_SUBSCRIPTION_STATE_DENIED'));
			}
			
			if ($details->ide) {
				$query->clear()
					->select($db->qn('id'))->select($db->qn('name'))
					->from($db->qn('#__rseventspro_events'))
					->where($db->qn('id').' = '.$db->q($details->ide));
				$db->setQuery($query);
				$event = $db->loadObject();
				return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id, $event->name),false,rseventsproHelper::itemid($event->id)));
			}
			
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro',false));
		}
		
		if ($app->input->server->get('REQUEST_METHOD', 'POST') == 'GET' && !$uri->isSSL()) {
			$uri->setScheme('https');
			$app->redirect($uri->toString());
		}
		
		$query->clear()
			->select($db->qn('ut.quantity'))->select($db->qn('t.id'))
			->select($db->qn('t.name'))->select($db->qn('t.price'))
			->from($db->qn('#__rseventspro_user_tickets','ut'))
			->join('left', $db->qn('#__rseventspro_tickets','t').' ON '.$db->qn('t.id').' = '.$db->qn('ut.idt'))
			->where($db->qn('ut.ids').' = '.(int) $details->id);
		
		$db->setQuery($query);
		$tickets = $db->loadObjectList();
		
		if ($details->ide) {
			foreach ($tickets as $ticket) {
				if ($ticket->price > 0) {
					$info[] = $ticket->quantity. ' x '.$ticket->name.' ('.rseventsproHelper::currency($ticket->price). ')';
					$total += $ticket->price * $ticket->quantity;
				} else {
					$info[] = $ticket->quantity. ' x '.$ticket->name.' ('.Text::_('COM_RSEVENTSPRO_GLOBAL_FREE'). ')';
				}
			}
			
			if (!empty($details->discount) && !empty($total)) {
				$total = $total - $details->discount;
			}
			
			if (!empty($details->early_fee) && !empty($total)) {
				$total = $total - $details->early_fee;
			}
			
			if (!empty($details->late_fee) && !empty($total)) {
				$total = $total + $details->late_fee;
			}
			
			if (!empty($details->tax)) {
				$total = $total + $details->tax;
			}
		} else {
			$app->triggerEvent('onrsepro_paymentForm', array(array('id' => $details->id, 'total' => &$total, 'info' => &$info)));
			$cart = true;
		}
		
		$app->triggerEvent('onrsepro_showForm', array(array('method' => &$method, 'details' => &$details, 'tickets' => &$tickets, 'total' => $total, 'info' => $info, 'cart' => $cart, 'currency' => &$currency)));
	}
	
	/**
	 *	Method to process the payment form
	 *
	 * @return 
	 */
	public function process() {
		$app	= Factory::getApplication();
		$data	= $app->input->get->request;
		
		$app->triggerEvent('onrsepro_processForm',array(array('data' => &$data)));
	}
	
	/**
	 *	Method to calculate event repeats
	 *
	 * @return int
	 */
	public function repeats() {
		require_once JPATH_SITE . '/components/com_rseventspro/helpers/recurring.php';
		
		$input		= Factory::getApplication()->input;
		$registry	= new Registry;
		
		$registry->set('interval', $input->getInt('interval',0));
		$registry->set('type', $input->getInt('type',0));
		$registry->set('start', $input->getString('start'));
		$registry->set('end', $input->getString('end'));
		$registry->set('days', $input->get('days',array(),'array'));
		$registry->set('also', $input->get('also',array(),'array'));
		$registry->set('exclude', $input->get('exclude',array(),'array'));
		
		$registry->set('repeat_on_type', $input->getInt('repeat_on_type',0));
		$registry->set('repeat_on_day', $input->getInt('repeat_on_day',0));
		$registry->set('repeat_on_day_order', $input->getInt('repeat_on_day_order',0));
		$registry->set('repeat_on_day_type', $input->getInt('repeat_on_day_type',0));
		
		$recurring = RSEventsProRecurring::getInstance($registry);
		$dates = $recurring->getDates(true);
		
		echo 'RS_DELIMITER0';
		echo count($dates);
		echo 'RS_DELIMITER1';
		exit();
	}
	
	/**
	 *	Method to get ajax search results
	 *
	 * @return string
	 */
	public function ajax() {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$search 	= Factory::getApplication()->input->getString('search');
		$itemid 	= Factory::getApplication()->input->getInt('iid');
		$opener		= Factory::getApplication()->input->getInt('opener',0);
		$canceled	= Factory::getApplication()->input->getInt('canceled',1);
		$lang		= Factory::getLanguage()->getTag();
		$state		= $canceled ? '1,3' : '1';
		$where		= array();
		
		$where[] = $db->qn('e.name').' LIKE '.$db->q('%'.$search.'%');
		$where[] = $db->qn('e.description').' LIKE '.$db->q('%'.$search.'%');
		
		$query->clear()
			->select($db->qn('e.id'))->select($db->qn('e.name'))->select($db->qn('e.published'))
			->from($db->qn('#__rseventspro_events','e'))
			->where($db->qn('e.completed').' = 1')
			->where($db->qn('e.published').' IN ('.$state.')');
		
		if (Multilanguage::isEnabled()) {
			$query->join('LEFT', $db->qn('#__rseventspro_translations','ten').' ON '.$db->qn('ten.reference_id').' = '.$db->qn('e.id').' AND '.$db->qn('ten.reference').' = '.$db->q('event').' AND '.$db->qn('ten.property').' = '.$db->q('name').' AND '.$db->qn('ten.language').' = '.$db->q($lang));
			
			$query->join('LEFT', $db->qn('#__rseventspro_translations','ted').' ON '.$db->qn('ted.reference_id').' = '.$db->qn('e.id').' AND '.$db->qn('ted.reference').' = '.$db->q('event').' AND '.$db->qn('ted.property').' = '.$db->q('description').' AND '.$db->qn('ted.language').' = '.$db->q($lang));
			
			$where[] = $db->qn('ten.value').' LIKE '.$db->q('%'.$search.'%');
			$where[] = $db->qn('ted.value').' LIKE '.$db->q('%'.$search.'%');
		}
		
		$query->where(implode(' OR ', $where));
		
		$db->setQuery($query);
		$events = $db->loadObjectList();
		
		$open = !$opener ? 'target="_blank"' : '';
		
		$html = 'RS_DELIMITER0';
		if (!empty($events)) {
			$html .= '<li class="rsepro_ajax_close"><a href="javascript:void(0);" onclick="rsepro_ajax_close();">x</a></li>';
			foreach ($events as $event) {
				if (!rseventsproHelper::canview($event->id)) 
					continue;
				
				$iid	= rseventsproHelper::itemid($event->id);
				$iid	= empty($iid) ? $itemid : $iid;
				
				// Replace translations
				if ($translation = RSEventsProTranslations::getTranslation('event', $event->id, 'name')) {
					$event->name = $translation;
				}
				
				$html .= '<li><a '.$open.' href="'.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),true,$iid).'">'.$event->name.($event->published == 3 ? ' <small class="text-alert">('.Text::_('COM_RSEVENTSPRO_EVENT_CANCELED_TEXT').')</small>' : '').'</a></li>';
			}
		}
		$html .= 'RS_DELIMITER1';
		
		echo $html;
		exit();
	}
	
	/**
	 *	Method to publish a moderated event
	 *
	 * @return
	 */
	public function activate() {
		$db				= Factory::getDbo();
		$query			= $db->getQuery(true);
		$key			= Factory::getApplication()->input->getString('key','');
		$permissions	= rseventsproHelper::permissions();
		$admin			= rseventsproHelper::admin();
		$juser			= Factory::getUser();
		$lang			= Factory::getLanguage();
		$sid			= Factory::getSession()->getId();
		$userid			= (int) $juser->get('id');
		
		if (!empty($key)) {
			$query->clear()
				->select($db->qn('id'))->select($db->qn('name'))->select($db->qn('sid'))
				->select($db->qn('location'))->select($db->qn('owner'))
				->from($db->qn('#__rseventspro_events'))
				->where('MD5(CONCAT('.$db->q('event').','.$db->qn('id').')) = '.$db->q($key));
			
			$db->setQuery($query,0,1);
			$event = $db->loadObject();
			
			// Do not allow a event owner to approve its own event
			if ($event->sid == $sid || (int) $event->owner == $userid) {
				Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_CANNOT_APPROVE_OWN_EVENT'),'error');
				return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
			}
			
			if ($admin || !empty($permissions['can_edit_events']) || !empty($permissions['can_approve_events'])) {
				if (!empty($event)) {
					$query->clear()
						->update($db->qn('#__rseventspro_locations'))
						->set($db->qn('published').' = 1')
						->where($db->qn('id').' = '.(int) $event->location);
					
					$db->setQuery($query);
					$db->execute();
					
					$query->clear()
						->update($db->qn('#__rseventspro_events'))
						->set($db->qn('published').' = 1')
						->set($db->qn('approved').' = 0')
						->where($db->qn('id').' = '.(int) $event->id);
					
					$db->setQuery($query);
					if ($db->execute()) {
						// Send approval email
						$owner	= Factory::getUser($event->owner);
						$to		= $owner->get('email');
						$name	= $owner->get('name');
						rseventsproEmails::approval($to, $event->id, $name, $lang->getTag());
						
						// Update user events added count
						rseventsproHelper::addUserEventCount($event->id, $event->owner);
						
						Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_EVENT_PUBLISHED'));
						return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
					}
				}
			}
		}
		
		Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'), 'error');
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false));
	}
	
	/**
	 *	Method to publish a moderated tag
	 *
	 * @return
	 */
	public function tagactivate() {
		$db				= Factory::getDbo();
		$query			= $db->getQuery(true);
		$key			= Factory::getApplication()->input->getString('key','');
		$permissions	= rseventsproHelper::permissions();
		$admin			= rseventsproHelper::admin();
		
		if (!empty($key)) {
			$query->clear()
				->select('*')
				->from('#__rseventspro_tags')
				->where('MD5(CONCAT('.$db->q('tag').','.$db->qn('id').')) = '.$db->q($key));
			
			$db->setQuery($query,0,1);
			$tag = $db->loadObject();
			
			if (($admin || !empty($permissions['can_approve_tags'])) && $tag) {
				$query->clear()
					->update($db->qn('#__rseventspro_tags'))
					->set($db->qn('published').' = 1')
					->where($db->qn('id').' = '.(int) $tag->id);
				
				$db->setQuery($query);
				if ($db->execute()) {
					Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_EVENT_TAG_PUBLISHED'));
					return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false));
				}
			}
		}
		
		Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'), 'error');
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false));
	}
	
	/**
	 *	Method to send reminders
	 *
	 * @return
	 */
	public function reminder() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$user	= Factory::getUser();
		$lang	= Factory::getLanguage();
		$id		= Factory::getApplication()->input->getInt('id');
		$sid	= Factory::getSession()->getId();
		
		$msg = Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED');
		
		$query->clear()
			->select($db->qn('id'))->select($db->qn('name'))
			->select($db->qn('sid'))->select($db->qn('owner'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($id));
		
		$db->setQuery($query);
		$event = $db->loadObject();
		
		if (rseventsproHelper::admin() || ($user->get('id') == $event->owner && !$user->get('guest')) || $sid == $event->sid) {
			$this->reminderSend($id);
			$msg = Text::_('COM_RSEVENTSPRO_EVENT_REMINDERS_SENT');
		}
		
		Factory::getApplication()->enqueueMessage($msg);
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
	}
	
	/**
	 *	Method to send auto reminders
	 *
	 * @return
	 */
	public function autoreminder() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$squery	= $db->getQuery(true);
		$lang	= Factory::getLanguage();
		
		// number of days - you can change this to the number of days that you require
		$days			= rseventsproHelper::getConfig('email_reminder_days','int');
		$now			= Factory::getDate()->toSql();
		$days_offset	= $days * 86400;
		
		$squery->clear()
			->select('DISTINCT '.$db->qn('ide'))
			->from($db->qn('#__rseventspro_taxonomy'))
			->where($db->qn('type').' = '.$db->q('reminder'));
		
		$query->clear()
			->select($db->qn('id'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('published').' = 1')
			->where($db->qn('completed').' = 1');
		
		if (!rseventsproHelper::getConfig('email_reminder_run','int')) {
			//before the event will end
			$query->where($db->q($now).' > DATE_SUB('.$db->qn('end').', INTERVAL '.$days_offset.' SECOND)');
			$query->where($db->q($now).' < '.$db->qn('end'));
			$query->where($db->qn('id').' NOT IN ('.$squery.')');
		} else {
			//before the event will start
			$query->where($db->q($now).' > DATE_SUB('.$db->qn('start').', INTERVAL '.$days_offset.' SECOND)');
			$query->where($db->q($now).' < '.$db->qn('start'));
			$query->where($db->qn('id').' NOT IN ('.$squery.')');
		}
		
		$db->setQuery($query);
		$events = $db->loadColumn();
		if (empty($events))
			exit();
		
		foreach ($events as $cid) {
			$query->clear()
				->select($db->qn('id'))->select($db->qn('name'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.(int) $cid);
			
			$db->setQuery($query);
			$row = $db->loadObject();
			if (empty($row)) continue;
			
			echo Text::sprintf('COM_RSEVENTSPRO_EVENT_SENDING_REMINDERS',$row->name);
			
			$query->clear()
				->insert($db->qn('#__rseventspro_taxonomy'))
				->set($db->qn('type').' = '.$db->q('reminder'))
				->set($db->qn('ide').' = '.(int) $row->id)
				->set($db->qn('id').' = 1');
			
			$db->setQuery($query);
			$db->execute();
			
			$this->reminderSend($row->id);
		}
		
		exit();
	}
	
	/**
	 *	Method to send post reminders
	 *
	 * @return
	 */
	public function postreminder() {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		$user	= Factory::getUser();
		$lang	= Factory::getLanguage();
		$id		= Factory::getApplication()->input->getInt('id');
		$sid	= Factory::getSession()->getId();
		$msg	= Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED');
		
		$query->clear()
			->select($db->qn('id'))->select($db->qn('name'))->select($db->qn('end'))
			->select($db->qn('sid'))->select($db->qn('owner'))
			->select($db->qn('start'))->select($db->qn('allday'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('published').' IN (1,2)')
			->where($db->qn('completed').' = 1')
			->where($db->qn('id').' = '.$db->q($id));
		
		$db->setQuery($query);
		if ($event = $db->loadObject()) {
			$now = Factory::getDate()->toUnix();
			
			if ($event->allday) {
				$date = Factory::getDate($event->start);
				$date->modify('+1 days');
				$endunix = $date->toUnix();
			} else {
				$endunix = Factory::getDate($event->end)->toUnix();
			}
			
			if ($endunix < $now && (rseventsproHelper::admin() || ($user->get('id') == $event->owner && !$user->get('guest')) || $event->sid == $sid)) {
				$this->postreminderSend($id);
				$msg = Text::_('COM_RSEVENTSPRO_EVENT_POSTREMINDERS_SENT');
			}
		
			Factory::getApplication()->enqueueMessage($msg);
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
		}
		
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro'));
	}
	
	/**
	 *	Method to send auto post reminders
	 *
	 * @return
	 */
	public function autopostreminder() {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		$lang	= Factory::getLanguage();
		$config	= rseventsproHelper::getConfig();
		$hash	= Factory::getApplication()->input->getString('hash');
		$now	= Factory::getDate()->toUnix();
		
		if ($config->auto_postreminder) {
			$secret = $config->postreminder_hash;
			
			if ($hash == $secret) {
				$query->clear()
					->select($db->qn('id'))->select($db->qn('name'))
					->select($db->qn('start'))->select($db->qn('end'))
					->select($db->qn('allday'))
					->from($db->qn('#__rseventspro_events'))
					->where($db->qn('published').' IN (1,2)')
					->where($db->qn('completed').' = 1');
				
				$db->setQuery($query);
				$events = $db->loadObjectList();
				
				foreach ($events as $event) {
					if ($event->allday) {
						$date = Factory::getDate($event->start);
						$date->modify('+1 days');
						$endunix = $date->toUnix();
					} else {
						$endunix = Factory::getDate($event->end)->toUnix();
					}
					
					if ($endunix < $now) {
						$this->postreminderSend($event->id);
					}
				}
			}
		}
		
		exit();
	}
	
	/**
	 *	Method to calculate the total
	 *
	 * @return
	 */
	public function total() {
		$app 		= Factory::getApplication();
		$jinput		= $app->input;
		$db 		= Factory::getDBO();
		$query		= $db->getQuery(true);
		$quantity	= $jinput->getInt('quantity',1);
		$tickets	= $jinput->get('tickets',array(),'array');
		$payment	= $jinput->getString('payment');
		$coupon		= $jinput->getString('coupon');
		$idevent	= $jinput->getInt('idevent',0);
		$w8list		= rseventsproHelper::validWaitingList($idevent);
		$type		= $jinput->getCmd('type','');
		$now		= Factory::getDate();
		$nowunix	= $now->toUnix();
		$total		= 0;
		$discount	= 0;
		$info		= array();
		$return		= array();
		$discounts	= array();
		$cname		= '';
		$couponid	= 0;
		
		$eventtickets = array();
		
		if (!empty($tickets)) {
			// Get event
			$query->clear()
				->select($db->qn('discounts'))->select($db->qn('early_fee'))->select($db->qn('early_fee_type'))
				->select($db->qn('early_fee_end'))->select($db->qn('late_fee'))->select($db->qn('late_fee_type'))
				->select($db->qn('late_fee_start'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.(int) $idevent);
			
			$db->setQuery($query);
			$event = $db->loadObject();
			
			foreach ($tickets as $tid => $quantity) {
				$checkticket = rseventsproHelper::checkticket($tid);
				$checkticket = $w8list ? 1 : $checkticket;
				
				if ($checkticket == RSEPRO_TICKETS_NOT_AVAILABLE) continue;
				
				$query->clear()
					->select($db->qn('price'))
					->from($db->qn('#__rseventspro_tickets'))
					->where($db->qn('id').' = '.(int) $tid);
				
				$db->setQuery($query);
				$price = $db->loadResult();
				
				if ($checkticket > RSEPRO_TICKETS_UNLIMITED && $quantity > $checkticket) $quantity = $checkticket;
				
				$eventtickets[$tid] = $quantity;
				
				// Calculate the total
				if ($price > 0) {
					$price = $price * $quantity;
					if ($event->discounts) {
						$eventdiscount = rseventsproHelper::discount($idevent,$price);
						
						if (is_array($eventdiscount)) {
							$query->clear()
								->select($db->qn('c.action'))->select($db->qn('c.type'))
								->from($db->qn('#__rseventspro_coupons','c'))
								->join('left', $db->qn('#__rseventspro_coupon_codes','cc').' ON '.$db->qn('cc.idc').' = '.$db->qn('c.id'))
								->where($db->qn('cc.id').' = '.(int) $eventdiscount['id']);
							
							$db->setQuery($query);
							$thecoupon = $db->loadObject();
							
							if ($thecoupon->action == 0) {
								if ($thecoupon->type == 0) {
									$discount += $eventdiscount['discount'] * $quantity;
								} else {
									$discount += $eventdiscount['discount'];
								}
							}
							
							$couponid = $eventdiscount['id'];
						}
					}
					$total += $price;
				}
			}
			
			if ($event->discounts) {
				$eventdiscount = rseventsproHelper::discount($idevent,$total);
				
				if (is_array($eventdiscount)) {
					$query->clear()
						->select($db->qn('c.action'))
						->from($db->qn('#__rseventspro_coupons','c'))
						->join('left', $db->qn('#__rseventspro_coupon_codes','cc').' ON '.$db->qn('cc.idc').' = '.$db->qn('c.id'))
						->where($db->qn('cc.id').' = '.(int) $eventdiscount['id']);
					
					$db->setQuery($query);
					$couponaction = $db->loadResult();
					
					if ($couponaction == 1)
						$discount += $eventdiscount['discount'];
					$couponid = $eventdiscount['id'];
				}
			}
			
			if ($event->discounts && $couponid) {
				$query->clear()
					->select($db->qn('c.name'))
					->from($db->qn('#__rseventspro_coupons','c'))
					->join('left',$db->qn('#__rseventspro_coupon_codes','cc').' ON '.$db->qn('cc.idc').' = '.$db->qn('c.id'))
					->where($db->qn('cc.id').' = '.(int) $couponid);
				
				$db->setQuery($query);
				$cname = $db->loadResult();
			}
			
			if ($event->discounts && $discount) {
				$discounts[] = (object) array('discount' => $discount, 'name' => $cname);
			}
			
			// Check for a global discount, and if found ignore the event discount
			if ($event->discounts) {
				if ($globalDiscount = rseventsproHelper::globalDiscount($idevent, $total, $eventtickets, $payment)) {
					$discounts[] = (object) array('discount' => $globalDiscount['discount'], 'name' => $globalDiscount['name']);
				}
			}
			
			// Sort discounts
			usort($discounts, array('rseventsproHelper', 'sort_discounts'));
			
			if (is_array($discounts) && isset($discounts[0])) {
				$discount	= $discounts[0]->discount;
				$cname		= $discounts[0]->name;
			}
			
			if ($discount) {
				// Update the total after the discount
				$total = $total - $discount;
				
				$info[] = Text::sprintf('COM_RSEVENTSPRO_DISCOUNT_ADDED',rseventsproHelper::currency($discount));
				$return['discount'] = rseventsproHelper::currency($discount);
				$return['discountname'] = $cname;
			}
			
			// Apply early fee
			if ($total > 0 && $event->discounts) {
				if (!empty($event->early_fee_end) && $event->early_fee_end != $db->getNullDate()) {
					$early_fee_unix = Factory::getDate($event->early_fee_end)->toUnix();
					if ($early_fee_unix > $nowunix) {
						$early = rseventsproHelper::setTax($total,$event->early_fee_type,$event->early_fee);
						$total = $total - $early;
						
						if ($early) {
							$info[] = Text::sprintf('COM_RSEVENTSPRO_EARLY_FEE_ADDED',rseventsproHelper::currency($early));
							$return['earlybooking'] = rseventsproHelper::currency($early);
						}
					}
				}
			}
			
			// Apply late fee
			if ($total > 0 && $event->discounts) {
				if (!empty($event->late_fee_start) && $event->late_fee_start != $db->getNullDate()) {
					$late_fee_unix = Factory::getDate($event->late_fee_start)->toUnix();
					if ($late_fee_unix < $nowunix) {
						$late = rseventsproHelper::setTax($total,$event->late_fee_type,$event->late_fee);
						$total = $total + $late;
						
						if ($late) {
							$info[] = Text::sprintf('COM_RSEVENTSPRO_LATE_FEE_ADDED',rseventsproHelper::currency($late));
							$return['latefee'] = rseventsproHelper::currency($late);
						}
					}
				}
			}
			
			// Apply tax
			// Check to see if the selected payment type is a wire payment
			$query->clear()
				->select($db->qn('id'))->select($db->qn('name'))
				->select($db->qn('tax_type'))->select($db->qn('tax_value'))
				->from($db->qn('#__rseventspro_payments'))
				->where($db->qn('id').' = '.(int) $payment);
			
			$db->setQuery($query);
			$wire = $db->loadObject();
			
			if ($total > 0) {
				if (!empty($wire)) {
					$tax = rseventsproHelper::setTax($total,$wire->tax_type,$wire->tax_value);
					$total = $total + $tax;
					
					if ($tax) {
						$info[] = Text::sprintf('COM_RSEVENTSPRO_TAX_ADDED',rseventsproHelper::currency($tax));
						$return['tax'] = rseventsproHelper::currency($tax);
					}
					
				} else {
					$tax = 0;
					$plugintaxes = $app->triggerEvent('onrsepro_tax',array(array('method'=>&$payment, 'total'=>$total)));
					
					if (!empty($plugintaxes))
						foreach ($plugintaxes as $plugintax)
							if (!empty($plugintax)) $tax = $plugintax;
					
					$total = $total + $tax;
					
					if ($tax) {
						$info[] = Text::sprintf('COM_RSEVENTSPRO_TAX_ADDED',rseventsproHelper::currency($tax));
						$return['tax'] = rseventsproHelper::currency($tax);
					}
				}
			}
		}
		
		$total 	= $total < 0 ? 0 : $total;
		$total 	= rseventsproHelper::currency($total);
		$info	= '|'.implode('<br />',$info);
		$return['total'] = $total;
		
		if ($type == 'json') {
			echo json_encode($return);
		} else {
			header('Content-type: text/html; charset=utf-8');
			echo 'RS_DELIMITER0'.$total.$info.'RS_DELIMITER1';
		}
		
		exit();
	}
	
	public function loadfile() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id');
		
		$query->select('*')
			->from($db->qn('#__rseventspro_files'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($file = $db->loadObject()) {
			if ($file->permissions == '') {
				$file->permissions = '000000';
			}
		}
		
		echo json_encode($file);
		exit();
	}
	
	public function ticket() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$id		= Factory::getApplication()->input->getInt('id');
		
		$query->select('*')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		if ($ticket = $db->loadObject()) {
			// Replace translations
			if ($translations = RSEventsProTranslations::getTranslations('ticket', $ticket->id)) {
				foreach ($translations as $field => $translation) {
					if (isset($ticket->$field)) {
						$ticket->$field = $translation;
					}
				}
			}
			
			$response = new stdClass();
			$response->name			= $ticket->name;
			$response->price		= rseventsproHelper::currency($ticket->price);
			$response->tprice		= $ticket->price;
			$response->mask			= rseventsproHelper::currency(0,true);
			$response->description	= $ticket->description;
			
			$response->payment_decimals	= rseventsproHelper::getConfig('payment_decimals','int');
			$response->payment_decimal	= rseventsproHelper::getConfig('payment_decimal');
			$response->payment_thousands	= rseventsproHelper::getConfig('payment_thousands');
			
			echo json_encode($response);
		}
		
		exit();
	}
	
	public function singleticket() {
		$db		  = Factory::getDbo();
		$query	  = $db->getQuery(true);
		$id		  = Factory::getApplication()->input->getInt('id');
		$quantity = Factory::getApplication()->input->getInt('quantity',1);
		$return	  = new stdClass();
		
		$query->select('*')
			->from($db->qn('#__rseventspro_tickets'))
			->where($db->qn('id').' = '.$id);
		
		$db->setQuery($query);
		$ticket = $db->loadObject();
		
		$w8list	  = rseventsproHelper::validWaitingList($ticket->ide);
		
		$return->seats = $w8list ? 1 : rseventsproHelper::checkticket($id);
		
		if ($ticket) {
			// Replace translations
			if ($translations = RSEventsProTranslations::getTranslations('ticket', $ticket->id)) {
				foreach ($translations as $field => $translation) {
					if (isset($ticket->$field)) {
						$ticket->$field = $translation;
					}
				}
			}
			
			$return->name			= $ticket->name;
			$return->price			= rseventsproHelper::currency($ticket->price);
			$return->tprice			= $ticket->price;
			$return->mask			= rseventsproHelper::currency(0,true);
			$return->description	= $ticket->description;
			
			$return->payment_decimals	= rseventsproHelper::getConfig('payment_decimals','int');
			$return->payment_decimal	= rseventsproHelper::getConfig('payment_decimal');
			$return->payment_thousands	= rseventsproHelper::getConfig('payment_thousands');
		}
		
		echo json_encode($return);
		exit();
	}
	
	public function image() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$app	= Factory::getApplication();
		$input	= $app->input;
		$width	= $input->getString('width','small');
		$height	= $input->getInt('height',0);
		
		if ($width == 'big') {
			$width = (int) rseventsproHelper::getConfig('icon_big_width');
		} elseif ($width == 'small') {
			$width = (int) rseventsproHelper::getConfig('icon_small_width');
		} else {
			$width = (int) $width;
		}
		
		$query->select($db->qn('name'))
			->select($db->qn('icon'))->select($db->qn('properties'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($input->getInt('id')));
		$db->setQuery($query);
		if ($event = $db->loadObject()) {
			$cache = Factory::getCache('com_rseventspro');
			$cache->setCaching(true);
			if ($data = $cache->get(array('rseventsproHelper', 'createImage'), array($event, $width, $height))) {
				@ob_end_clean();
				require_once JPATH_SITE.'/components/com_rseventspro/helpers/phpthumb/phpthumb.functions.php';
				header('Content-Type: '.phpthumb_functions::ImageTypeToMIMEtype($data['ext']));
				header('Content-Disposition: inline; filename="' . ApplicationHelper::stringURLSafe($event->name) . '"');
				echo $data['content'];
				$app->close();
			}
		}
		
		return false;
	}
	
	public function timezone() {
		$input		= Factory::getApplication()->input;
		$timezone	= $input->getString('timezone');
		$return		= base64_decode($input->getString('return'));
		$session	= Factory::getSession();
		
		$session->set('rsepro.timezone', $timezone);
		
		$this->setRedirect($return);
	}
	
	// Trigger plugin functions
	public function trigger() {
		Factory::getApplication()->triggerEvent('onrsepro_frontTrigger');
	}
	
	// Cron for rules
	public function rules() {
		rseventsproHelper::rules();
	}
	
	// Auto-Sync Google calendar and Facebook events
	public function autosync() {
		$config = rseventsproHelper::getConfig();
		
		// Syng Google Calendar
		if ($config->google_client_id && $config->google_secret) {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/google.php';
			
			try {
				$google	= new RSEPROGoogle();
				$google->parse();
			} catch(Exception $e) {}
		}
		
		// Sync Facebook events
		if (!empty($config->facebook_token)) {
			try {
				rseventsproHelper::facebookEvents();
			} catch(Exception $e) {}
		}
	}
	
	public function sendsubscription() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$email	= Factory::getApplication()->input->getString('email');
		
		if (empty($email) || !MailHelper::isEmailAddress($email)) {
			$this->setMessage(Text::_('COM_RSEVENTSPRO_INVALID_EMAIL'),'error');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscriptions',false));
		}
		
		$email = strtolower($email);
		
		$query->clear()
			->select($db->qn('id'))->select($db->qn('date'))
			->select($db->qn('verification'))
			->from($db->qn('#__rseventspro_users'))
			->where('LOWER('.$db->qn('email').') = '.$db->q($email));
		$db->setQuery($query);
		$subscriber = $db->loadObject();
		
		if (!$subscriber) {
			$query->clear()
				->select($db->qn('r.id'))->select($db->qn('r.date'))
				->from($db->qn('#__rseventspro_rsvp_users','r'))
				->join('LEFT', $db->qn('#__users','u').' ON '.$db->qn('r.uid').' = '.$db->qn('u.id'))
				->where('LOWER('.$db->qn('u.email').') = '.$db->q($email));
			$db->setQuery($query);
			$subscriber = $db->loadObject();
		}
		
		if ($subscriber) {
			$config		= rseventsproHelper::getConfig();
			$hash		= isset($subscriber->verification) ? md5($subscriber->date.$subscriber->id.$email.$subscriber->verification) : md5($subscriber->date.$subscriber->id.$email);
			$url		= Uri::getInstance()->toString(array('scheme','host')).rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscriptions&code='.$hash,false);
			$subject	= Text::_('COM_RSEVENTSPRO_SUBSCRIPTION_SUBJECT');
			$body		= Text::sprintf('COM_RSEVENTSPRO_SUBSCRIPTION_BODY',$url);
			$replyto	= isset($config->email_replyto) && !empty($config->email_replyto) ? explode(',',$config->email_replyto) : null;
			$replyname	= isset($config->email_replytoname) && !empty($config->email_replytoname) ? explode(',',$config->email_replytoname) : null;
			
			try {
				Factory::getMailer()->sendMail($config->email_from, $config->email_fromname, $email, $subject, $body, 1, null, null, null, $replyto, $replyname);
			} catch (Exception $e) {}
			
			$this->setMessage(Text::_('COM_RSEVENTSPRO_EMAIL_SENT'));
		} else {
			$this->setMessage(Text::_('COM_RSEVENTSPRO_NO_RECORDS_FOUND'), 'error');
		}
		
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscriptions',false));
	}
	
	public function confirm() {
		$db				= Factory::getDbo();
		$query			= $db->getQuery(true);
		$hash			= Factory::getApplication()->input->getString('hash');
		$secret 		= Factory::getConfig()->get('secret');
		$permissions	= rseventsproHelper::permissions();
		$admin			= rseventsproHelper::admin();
		$ids			= 0;
		$found			= false;
		$code           = false;
		
		if ($admin || !empty($permissions['can_confirm_tickets'])) {
			if (substr($hash, 0 , 2) == 'cr') {
				$hash = str_replace('_', ' ', $hash);
				$hash = substr_replace($hash,'',0,2);
				
				try {
					require_once JPATH_SITE.'/components/com_rseventspro/helpers/crypt.php';
					
					$hash = RseventsproCrypt::decrypt($hash, $secret);
				} catch (Exception $e) {}
			}
			
			if (preg_match('#\|([0-9]*)\|#is', $hash, $match)) {
				if (isset($match) && isset($match[1])) {
					$ids	= $match[1];
					$hash	= str_replace($match[0], '', $hash);
				}
			}
			
			if ($ids) {
				$query->clear()
					->select($db->qn('t.id'))->select($db->qn('t.ide'))->select($db->qn('t.name'))
					->select($db->qn('t.price'))->select($db->qn('ut.quantity'))
					->from($db->qn('#__rseventspro_tickets','t'))
					->join('left',$db->qn('#__rseventspro_user_tickets','ut').' ON '.$db->qn('t.id').' = '.$db->qn('ut.idt'))
					->where($db->qn('ut.ids').' = '.(int) $ids);
				$db->setQuery($query);
				if ($tickets = $db->loadObjectList()) {
					foreach ($tickets as $ticket) {
						for ($i=1;$i<=$ticket->quantity;$i++) {
							$tcode	= md5($ids.$ticket->id.$i);
							
							if (strtolower($tcode) == strtolower($hash)) {
								$found	= true;
								$code	= rseventsproHelper::getBarcodeOptions('barcode_prefix', 'RST-').$ids.'-'.substr($tcode,0,4).substr($tcode,-4);
								
								$query->clear()
									->select($db->qn('id'))
									->from('#__rseventspro_confirmed')
									->where($db->qn('ids').' = '.$db->q($ids))
									->where($db->qn('code').' = '.$db->q($code));
								$db->setQuery($query);
								if (!$db->loadResult()) {
									$query->clear()
										->insert('#__rseventspro_confirmed')
										->set($db->qn('ids').' = '.$db->q($ids))
										->set($db->qn('code').' = '.$db->q($code));
									$db->setQuery($query);
									if ($db->execute()) {
										Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_TICKET_CONFIRMED'));
									}
								} else {
									Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_TICKET_ALREADY_CONFIRMED'), 'error');
								}
								continue 2;
							}
						}
					}
				}
			}
			
			if (!$found) {
				Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_TICKET_CONFIRMED_ERROR'), 'error');
			}
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'), 'error');
		}

		$view = new HtmlView(array(
			'name' => 'rseventspro',
			'layout' => 'confirm',
			'base_path' => JPATH_SITE.'/components/com_rseventspro'
		));

		$view->addTemplatePath(JPATH_THEMES.'/'.Factory::getApplication()->getTemplate().'/html/com_rseventspro/rseventspro');

		$view->found = $found;
		$view->code = $code;

		echo $view->loadTemplate();
	}
	
	public function rsvp() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$app	= Factory::getApplication();
		$id		= $app->input->getInt('id', 0);
		$rsvp	= $app->input->get('rsvp');
		$uid	= Factory::getUser()->get('id');
		$to		= Factory::getUser()->get('email');
		$lang	= Factory::getLanguage()->getTag();
		$data	= array();
		
		if ($id && $uid) {
			$options = rseventsproHelper::getRSVPOptions($id);
			
			// Check if the user can RSVP to this event
			if (!$options->canRSVP && $rsvp == 'going') {
				$data['success'] = false;
				$data['message'] = $options->message;
				
				echo json_encode($data);
				$app->close();
			}
			
			$query->clear()
				->select($db->qn('id'))->select($db->qn('rsvp'))->select($db->qn('language'))
				->from($db->qn('#__rseventspro_rsvp_users'))
				->where($db->qn('ide').' = '.$db->q($id))
				->where($db->qn('uid').' = '.$db->q($uid));
			$db->setQuery($query);
			if ($rsvpData = $db->loadObject()) {
				if ($rsvpData->rsvp == $rsvp) {
					$query->clear()
						->delete($db->qn('#__rseventspro_rsvp_users'))
						->where($db->qn('id').' = '.$db->q($rsvpData->id));
					$db->setQuery($query);
					$db->execute();
					$data['remove'] = true;
				} else {
					$query->clear()
						->update($db->qn('#__rseventspro_rsvp_users'))
						->set($db->qn('rsvp').' = '.$db->q($rsvp))
						->set($db->qn('date').' = '.$db->q(Factory::getDate()->toSql()))
						->where($db->qn('id').' = '.$db->q($rsvpData->id));
					$db->setQuery($query);
					$db->execute();
					
					$func = 'rsvp'.$rsvp;
					rseventsproEmails::$func($to, $id, $rsvpData->language);
				}
			} else {
				$query->clear()
					->insert($db->qn('#__rseventspro_rsvp_users'))
					->set($db->qn('ide').' = '.$db->q($id))
					->set($db->qn('uid').' = '.$db->q($uid))
					->set($db->qn('rsvp').' = '.$db->q($rsvp))
					->set($db->qn('language').' = '.$db->q($lang))
					->set($db->qn('date').' = '.$db->q(Factory::getDate()->toSql()));
				$db->setQuery($query);
				$db->execute();
				
				$func = 'rsvp'.$rsvp;
				rseventsproEmails::$func($to, $id, $lang);
			}
			
			$data['success'] = true;
			$data['info'] = Text::_('COM_RSEVENTSPRO_RSVP_INFO');
		} else {
			$data['success'] = false;
			$data['message'] = Text::_('COM_RSEVENTSPRO_RSVP_PLEASE_LOGIN');
		}
		
		echo json_encode($data);
		$app->close();
	}
	
	protected function reminderSend($ide) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$lang	= Factory::getLanguage();
		$data	= array();
		
		// Get subscribers
		$query->clear()
			->select('DISTINCT '.$db->qn('u.email'))->select($db->qn('u.name'))->select($db->qn('u.id'))
			->from($db->qn('#__rseventspro_users','u'))
			->where($db->qn('u.ide').' = '.(int) $ide)
			->where($db->qn('u.state').' IN (0,1)');
		
		Factory::getApplication()->triggerEvent('onrsepro_subscriptionsQuery', array(array('query' => &$query)));
		
		$db->setQuery($query);
		if ($subscribers = $db->loadObjectList()) {
			$data = array_merge($subscribers, $data);
		}
		
		// Get RSVP subscribers
		$query->clear()
			->select($db->qn('u.email'))->select($db->qn('u.name'))->select($db->qn('r.id'))
			->from($db->qn('#__rseventspro_rsvp_users','r'))
			->join('LEFT',$db->qn('#__users','u').' ON '.$db->qn('r.uid').' = '.$db->qn('u.id'))
			->where($db->qn('r.rsvp').' IN ("going","interested")')
			->where($db->qn('r.ide').' = '.(int) $ide);
		$db->setQuery($query);
		if ($RSVPsubscribers = $db->loadObjectList()) {
			foreach ($RSVPsubscribers as $s) {
				$s->id = null;
			}
			
			$data = array_merge($data, $RSVPsubscribers);
		}
		
		if (!empty($data)) {
			$sent = array();
			foreach ($data as $subscriber) {
				$hash = md5($subscriber->email.$ide);
				
				if (!isset($sent[$hash])) {
					rseventsproEmails::reminder($subscriber, $ide, $lang->getTag());
					$sent[$hash] = true;
				}
			}
		}
	}
	
	protected function postreminderSend($ide) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$type	= rseventsproHelper::getConfig('postreminder','int');
		$data	= array();
		
		// Get event name
		$query->clear()
			->select($db->qn('name'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		$name = $db->loadResult();
		
		// Get subscribers
		$query->clear()
			->select('DISTINCT '.$db->qn('u.email'))->select($db->qn('u.name'))->select($db->qn('u.lang'))
			->from($db->qn('#__rseventspro_users','u'))
			->where($db->qn('u.ide').' = '.(int) $ide)
			->where($db->qn('u.state').' = 1');
		
		Factory::getApplication()->triggerEvent('onrsepro_subscriptionsQuery', array(array('query' => &$query)));
		
		$db->setQuery($query);
		if ($subscribers = $db->loadObjectList()) {
			$data = array_merge($subscribers, $data);
		}
		
		// Get RSVP subscribers
		$query->clear()
			->select('DISTINCT '.$db->qn('u.email'))->select($db->qn('u.name'))
			->select($db->qn('r.language', 'lang'))
			->from($db->qn('#__rseventspro_rsvp_users','r'))
			->join('LEFT',$db->qn('#__users','u').' ON '.$db->qn('r.uid').' = '.$db->qn('u.id'))
			->where($db->qn('r.rsvp').' = '.$db->q('going'))
			->where($db->qn('r.ide').' = '.(int) $ide);
		$db->setQuery($query);
		if ($RSVPsubscribers = $db->loadObjectList()) {
			$data = array_merge($data, $RSVPsubscribers);
		}
		
		if (!empty($data)) {
			foreach ($data as $subscriber) {
				if ($type == 0) {
					$query->clear()
						->select($db->qn('id'))
						->from($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q('preminder'))
						->where($db->qn('ide').' = '.$db->q($ide))
						->where($db->qn('extra').' = '.$db->q($subscriber->email));
					$db->setQuery($query);
					if ($db->loadResult()) {
						continue;
					}
				}
				
				echo 'Event ('.$name.') - Email ('.$subscriber->email.') <br />';
				rseventsproEmails::postreminder($subscriber->email, $ide, $subscriber->name, $subscriber->lang);
				
				if ($type == 0) {
					$query->clear()
						->select('MAX('.$db->qn('id').')')
						->from($db->qn('#__rseventspro_taxonomy'))
						->where($db->qn('type').' = '.$db->q('preminder'))
						->where($db->qn('ide').' = '.$db->q($ide));
					$db->setQuery($query);
					$id = (int) $db->loadResult() + 1;
					
					$query->clear()
						->insert($db->qn('#__rseventspro_taxonomy'))
						->set($db->qn('type').' = '.$db->q('preminder'))
						->set($db->qn('ide').' = '.$db->q($ide))
						->set($db->qn('id').' = '.$db->q($id))
						->set($db->qn('extra').' = '.$db->q($subscriber->email));
					$db->setQuery($query);
					$db->execute();
				}
			}
		}
	}
	
	public function claim() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$app	= Factory::getApplication();
		$hash	= $app->input->getString('hash', '');
		$now	= Factory::getDate()->toUnix();
		
		if ($hash) {
			$query->clear()
				->select('w.*')->select($db->qn('e.name','ename'))->select($db->qn('e.allday'))
				->select($db->qn('e.start'))->select($db->qn('e.end'))->select($db->qn('e.start_registration'))
				->select($db->qn('e.end_registration'))->select($db->qn('e.waitinglist_time'))
				->from($db->qn('#__rseventspro_waitinglist','w'))
				->join('LEFT', $db->qn('#__rseventspro_events','e').' ON '.$db->qn('w.ide').' = '.$db->qn('e.id'))
				->where($db->qn('w.hash').' = '.$db->q($hash));
			$db->setQuery($query);
			if ($data = $db->loadObject()) {
				
				$itemid = rseventsproHelper::itemid($data->ide);
				$itemid = $itemid ? $itemid : $data->itemid;
				
				// Check claim expiration date
				if (!empty($data->waitinglist_time)) {
					$nowd	= Factory::getDate();
					$date	= Factory::getDate($data->sent);
					$date->modify('+ '.$data->waitinglist_time.' second');
					
					if ($nowd > $date) {
						$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_ERROR5'), 'error');
						$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($data->ide,$data->ename),false,$itemid));
					}
				}
				
				// The event has ended
				$enddate = $data->allday && $data->end == $db->getNullDate() ? $data->start : $data->end;
				if ($now > Factory::getDate($enddate)->toUnix()) {
					$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_ERROR3'), 'error');
					$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($data->ide,$data->ename),false,$itemid));
				}
				
				$show = true;
				if ($data->start_registration == $db->getNullDate()) $data->start_registration = '';
				if ($data->end_registration == $db->getNullDate()) $data->end_registration = '';
				
				$start_registration = empty($data->start_registration) ? false : Factory::getDate($data->start_registration)->toUnix();
				$end_registration = empty($data->end_registration) ? false : Factory::getDate($data->end_registration)->toUnix();
				
				if (!empty($start_registration) && !empty($end_registration)) {
					if ($start_registration <= $now && $end_registration >= $now || $start_registration >= $now && $end_registration <= $now) {
						$show = true;
					} else {
						$show = false;
					}
				} elseif (empty($start_registration) && !empty($end_registration)) {
					if ($end_registration >= $now) {
						$show = true;
					} else {
						$show = false;
					}
				} elseif (!empty($start_registration) && empty($end_registration)) {
					if ($start_registration <= $now) {
						$show = true;
					} else { 
						$show = false;
					}
				} elseif (empty($start_registration) && empty($end_registration)) {
					$show = true;
				}
				
				if (!$show) {
					$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_ERROR4'), 'error');
					$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($data->ide,$data->ename),false,$itemid));
				}
				
				if ($data->used) {
					$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_ERROR1'), 'error');
					$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($data->ide,$data->ename),false,$itemid));
				}
				
				Factory::getSession()->set('rsepro_waitinglist'.$data->ide, $hash);
				$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_CLAIM_TICKET'));
				$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($data->ide,$data->ename),false,$itemid));
			} else {
				$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_ERROR2'), 'error');
				$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro',false));
			}
		} else {
			$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro',false));
		}
	}
}