<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;

class RseventsproControllerRseventspro extends BaseController
{
	protected $_app;
	
	protected $permissions;
	
	/**
	 *	Main constructor
	 */
	public function __construct() {
		parent::__construct();
		$this->_app			= Factory::getApplication();
		$this->permissions	= rseventsproHelper::permissions();
		
		$this->registerTask('approve',	'status');
		$this->registerTask('pending',	'status');
		$this->registerTask('denied',	'status');
		$this->registerTask('refund',	'status');
		$this->registerTask('getFeedback', 'loadData');
		$this->registerTask('getFaq', 'loadData');
		$this->registerTask('getItinerary', 'loadData');
		$this->registerTask('getTimeline', 'loadData');
		$this->registerTask('saveFeedback', 'saveData');
		$this->registerTask('saveFaq', 'saveData');
		$this->registerTask('saveItinerary', 'saveData');
		$this->registerTask('saveTimeline', 'saveData');
		$this->registerTask('removeFeedback', 'removeData');
		$this->registerTask('removeFaq', 'removeData');
		$this->registerTask('removeItinerary', 'removeData');
		$this->registerTask('removeTimeline', 'removeData');
		$this->registerTask('saveOrderFeedbackAjax', 'orderData');
		$this->registerTask('saveOrderFaqAjax', 'orderData');
		$this->registerTask('saveOrderFaqAjax', 'orderData');
		$this->registerTask('saveOrderTimelineAjax', 'orderData');
	}
	
	// Subscribers - export subscribers
	public function exportguests() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->exportguests();
		} else {
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)), Text::_('COM_RSEVENTSPRO_ERROR_EXPORT_SUBSCRIBERS'));
		}
	}
	
	// Subscribers - change state
	public function status() {
		$model	= $this->getModel('rseventspro');
		$event	= $model->getEvent();
		$admin	= rseventsproHelper::admin();
		$user	= $model->getUser();
		$from	= Factory::getApplication()->input->get('from', '');
		$msg	= '';
		
		if ($admin || (isset($event->owner) && $event->owner == $user) || (isset($event->sid) && $event->sid == $user) || !empty($this->permissions['can_change_subscriptions'])) {
			$id		= Factory::getApplication()->input->getInt('id');
			$values	= array('approve' => 1, 'pending' => 0, 'denied' => 2, 'refund' => 3);
			$task	= $this->getTask();
			$value	= ArrayHelper::getValue($values, $task, 0, 'int');
			
			if (!$model->status($id, $value)) {
				$this->setMessage($model->getError(), 'error');
			} else {
				$msg = Text::_('COM_RSEVENTSPRO_SUBSCRIBER_STATUS_CHANGED');
			}
		} else $msg = Text::_('COM_RSEVENTSPRO_ERROR_SUBSCRIBER_STATUS');
		
		if ($from) {
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&view=subscriptions',false),$msg);
		} else {
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscribers&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		}
	}
	
	// Subscribers - save subscriber
	public function savesubscriber() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		$app = Factory::getApplication();
		
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin  = rseventsproHelper::admin();
		$user   = $model->getUser();
		$isuser = $app->input->getInt('isuser',0);
		$from	= $app->input->get('from','');
		
		if ($admin || $event->owner == $user || $event->sid == $user || $isuser || $this->permissions['can_edit_subscriptions']) {
			if (!$model->savesubscriber()) {
				$app->enqueueMessage($model->getError(), 'error');
			} else {
				$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_SUBSCRIBER_SAVED'));
			}
		} else {
			$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_ERROR_SUBSCRIBER_SAVE'));
		}
		
		if ($from == 'subscriptions') {
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&view=subscriptions',false));
		} else if ($isuser) {
			$code	= $app->input->getString('code');
			$code	= $code ? '&code='.$code : '';
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscriptions'.$code,false));
		} else {
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscribers&id='.rseventsproHelper::sef($event->id,$event->name),false));
		}
	}
	
	// Subscribers - remove
	public function removesubscriber() {
		$model	= $this->getModel('rseventspro');
		$event	= $model->getEvent();
		$from	= Factory::getApplication()->input->get('from', '');
		$admin	= rseventsproHelper::admin();
		$user 	= $model->getUser();
		
		if ($admin || (isset($event->owner) && $event->owner == $user) || (isset($event->sid) && $event->sid == $user) || !empty($this->permissions['can_delete_subscriptions'])) {
			if (!$model->removesubscriber()) {
				$this->setMessage($model->getError(), 'error');
			} else {
				$msg = Text::_('COM_RSEVENTSPRO_SUBSCRIBER_DELETED');
			}
		} else $msg = Text::_('COM_RSEVENTSPRO_ERROR_SUBSCRIBER_DELETE');
		
		if ($from) {
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&view=subscriptions',false),$msg);
		} else {
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscribers&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		}
	}
	
	// Subscriber - remove
	public function deletesubscriber() {
		// Get the model
		$model	= $this->getModel('rseventspro');
		$code	= Factory::getApplication()->input->getString('code');
		$code	= $code ? '&code='.$code : '';
		
		if (!$model->deletesubscriber())
			$this->setMessage($model->getError(), 'error');
		else
			$this->setMessage(Text::_('COM_RSEVENTSPRO_SUBSCRIBER_DELETED'));
		
		$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=subscriptions'.$code,false),$msg);
	}
	
	// Event - send message to guests
	public function message() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			if ($model->message()) {
				$url = rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id));
				echo rseventsproHelper::redirect(true,Text::_('COM_RSEVENTSPRO_MESSAGES_SENT'),$url,true);
			}
		}
		else 
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)), Text::_('COM_RSEVENTSPRO_ERROR_SEND_MESSAGE_GUESTS'));
	}
	
	// Invite people to event
	public function invite() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$nowunix = Factory::getDate()->toUnix();
		$endunix = Factory::getDate($event->end)->toUnix();
		
		if ($nowunix > $endunix)
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)), Text::_('COM_RSEVENTSPRO_ERROR_INVITE_1'));
		
		Factory::getApplication()->input->set('tmpl', 'component');
		
		if ($model->invite()) {
			$url = rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id));
			echo rseventsproHelper::redirect(true,Text::_('COM_RSEVENTSPRO_INVITATIONS_SENT'),$url,true);
		} else {
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=invite&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)), Text::_('COM_RSEVENTSPRO_SUBSCRIBER_CAPTCHA_ERROR'));
		}
	}
	
	// Event - export event
	public function export() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Export event
		if (!$model->export())
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false));
	}
	
	// Event - rate event
	public function rate() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		echo $model->rate();
		exit();
	}
	
	// Event - delete vote
	public function deletevote() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		if ($model->deletevote()) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_VOTE_REMOVED'));
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_REMOVE_VOTE_ERROR'), 'error');
		}
		
		$url = rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id));
		Factory::getApplication()->redirect($url);
	}
	
	// Events - save edited location
	public function savelocations() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$admin = rseventsproHelper::admin();
		if (empty($this->permissions['can_edit_locations']) && !$admin)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_EDIT_LOCATION'), 500);
		
		// Save location
		if (!$model->savelocation()) {
			$this->setMessage($model->getError(), 'error');
		} else {
			$this->setMessage(Text::_('COM_RSEVENTSPRO_LOCATION_SAVED'));
		}
		
		// Redirect
		$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=locations',false));
	}
	
	// Event - subscribe
	public function subscribe() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Subscribe user
		$result = $model->subscribe();
		
		echo rseventsproHelper::redirect(true,$result['message'],$result['url'],true);
	}
	
	// Event unsubscribe
	public function unsubscribeuser() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Unsubscribe user
		$result = $model->unsubscribeuser();
		
		if (!$result['status']) {
			$this->setRedirect($result['url'],$result['message']);
		} else {
			Factory::getApplication()->input->set('tmpl','component');
			echo '<div class="rs_message_info">'.$result['message'].'</div>';
			echo '<script type="text/javascript">window.top.setTimeout(\'window.parent.location.reload();'.addslashes(rseventsproHelper::modalClose(false,true)).'\',1200);</script>';
		}
	}
	
	// Event unsubscribe
	public function unsubscribe() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Unsubscribe user
		$result = $model->unsubscribe();
		
		$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($result['id'],$result['name']),false,rseventsproHelper::itemid($result['id'])),$result['message']);
	}
	
	// Events - add location from event
	public function savelocation() {
		// Get the model
		$model = $this->getModel('rseventspro');
		$admin = rseventsproHelper::admin();
		
		if (!$admin && empty($this->permissions['can_add_locations']))
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_ADD_LOCATION'), 500);
		
		if ($model->savelocation())
			echo $model->getState('rseventspro.lid');
		
		exit();
	}
	
	// Events - save category
	public function savecategory() {
		// Get the model
		$model = $this->getModel('rseventspro');
		$admin = rseventsproHelper::admin();
		
		if (!$admin && empty($this->permissions['can_create_categories']))
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_ADD_CATEGORY'), 500);
		
		if ($model->savecategory()) {
			echo json_encode(rseventsproHelper::getCategoriesOptions());
		}
			
		exit();
	}
	
	// Event - save ticket
	public function saveticket() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		if (empty($this->permissions['enable_registration']) || empty($this->permissions['can_create_tickets']))
			throw new Exception(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'), 500);
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$tid = $model->saveticket();
			if (Factory::getApplication()->input->get('format') == 'raw') {
				return $tid;
			} else {
				echo 'RS_DELIMITER0';
				echo $tid;
				echo 'RS_DELIMITER1';
			}
		}
		
		exit();
	}
	
	// Events - remove ticket
	public function removeticket() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			Factory::getApplication()->input->set('from','ticket');
			$user = $model->getUser();
			$owner = $model->getOwner();
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_REMOVE_TICKET'), 500);
		
		echo 'RS_DELIMITER0';
		echo (int) $model->removeticket();
		echo 'RS_DELIMITER1';
		
		exit();
	}
	
	// Event - save coupon
	public function savecoupon() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		if (empty($this->permissions['enable_registration']) || empty($this->permissions['enable_discounts']))
			throw new Exception(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'), 500);
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$cid = $model->savecoupon();
			if (Factory::getApplication()->input->get('format') == 'raw') {
				return $cid;
			} else {
				echo 'RS_DELIMITER0';
				echo $cid;
				echo 'RS_DELIMITER1';
			}
		}
		
		exit();
	}
	
	// Events - remove coupon
	public function removecoupon() {	
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			Factory::getApplication()->input->set('from','coupon');
			$user = $model->getUser();
			$owner = $model->getOwner();
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_REMOVE_COUPON'), 500);
		
		echo 'RS_DELIMITER0';
		echo (int) $model->removecoupon();
		echo 'RS_DELIMITER1';
		
		exit();
	}
	
	// Events - save file
	public function savefile() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			Factory::getApplication()->input->set('from','file');
			$user = $model->getUser();
			$owner = $model->getOwner();		
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_EDIT_FILE'), 500);
		
		// Save
		$success = $model->savefile();
		
		echo (int) $success;
		exit();
	}
	
	// Events - remove file
	public function removefile() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			Factory::getApplication()->input->set('from','file');
			$user = $model->getUser();
			$owner = $model->getOwner();		
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_REMOVE_FILE'), 500);
		
		echo 'RS_DELIMITER0';
		echo (int) $model->removefile();
		echo 'RS_DELIMITER1';
		
		exit();
	}
	
	// Delete event icon
	public function deleteicon() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user = $model->getUser();
		
		echo 'RS_DELIMITER0';
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->deleteicon();
			echo '1';
		}
		else {
			echo '0';
		}
		echo 'RS_DELIMITER1';
		exit();
	}
	
	// Events - upload
	public function upload() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			$user = $model->getUser();
			$owner = $model->getOwner();
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_UPLOAD_ICON'), 500);
		}
		
		
		if (!$model->upload()) {
			$icon = '';
			$this->setMessage($model->getError());
		} else {
			$icon = '&icon='.base64_encode($model->getState('com_rseventspro.edit.icon'));
		}
		
		return $this->setRedirect(Route::_('index.php?option=com_rseventspro&layout=upload&tmpl=component'.$icon.'&id='.Factory::getApplication()->input->getInt('id'),false));
	}
	
	// Events - crop
	public function crop() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			$user = $model->getUser();
			$owner = $model->getOwner();
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_CROP_ICON'), 500);
		}
		
		// Crop image
		$model->crop();
		
		$this->setMessage(Text::_('COM_RSEVENTSPRO_CROP_SAVED'));
		return $this->setRedirect(Route::_('index.php?option=com_rseventspro&layout=upload&tmpl=component&icon='.base64_encode($model->getState('rseventspro.crop.icon')).'&id='.Factory::getApplication()->input->getInt('id'),false));
	}
	
	// Events - remove
	public function remove() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			$user  = $model->getUser();
			$owner = $model->getOwner();
			if (($user != $owner && empty($this->permissions['can_delete_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_REMOVE_EVENT'), 500);
		
		$model->remove();
		
		Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_EVENT_DELETED'));
		
		// Hack for Joomla! SEF 
		$menus		= $this->_app->getMenu();
		$menu		= $menus->getActive(); 
		
		if (!empty($menu)) {
			$query = $menu->query;
			
			if (isset($query['option']) && $query['option'] == 'com_rseventspro' && isset($query['layout']) && $query['layout'] == 'edit') {
				if ($eventsItemid = RseventsproHelperRoute::getEventsItemid()) {
					return $this->_app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false,$eventsItemid));
				} else {
					return $this->_app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false,'99999'));
				}
			}
		}
		
		// Redirect
		$this->_app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false));
	}
	
	// Events - save event
	public function save() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get the id of the event
		$input	= Factory::getApplication()->input;
		$jform	= $input->get('jform', array(), 'array');
		$show	= $input->getInt('show',0);
		$id		= (int) $jform['id'];
		$tab	= $input->getInt('tab');
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			if (!$id) {
				if (empty($this->permissions['can_post_events']))
					$permission_denied = true;
			} else {
				$user = $model->getUser();
				$owner = $model->getOwner();
				if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
					$permission_denied = true;
			}
		}
		
		if ($permission_denied) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'), 500);
		}
		
		// Save
		if ($model->save()) {
			$moderated	= $model->getState('moderated');
			$eventID	= $model->getState('eventid');
			$eventName	= $model->getState('eventname');
			$link		= 'index.php?option=com_rseventspro&layout=edit&id='.rseventsproHelper::sef($eventID,$eventName).($tab ? '&tab='.$tab : '');
			$msg		= $moderated ? Text::_('COM_RSEVENTSPRO_EVENT_SAVED_WITH_MODERATION') : Text::_('COM_RSEVENTSPRO_EVENT_SAVED_OK');
			
			if ($show) {
				$link = 'index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($eventID,$eventName);
			}
			
			return $this->setRedirect(rseventsproHelper::route($link,false), $msg);
		} else {
			$msg	= $model->getError() ? $model->getError() : Text::_('COM_RSEVENTSPRO_EVENT_SAVED_ERROR');
			$link	= Uri::getInstance()->toString();
			
			return $this->setRedirect(Route::_($link,false), $msg, 'error');
		}
	}
	
	public function cancel() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		$model = $this->getModel('rseventspro');
		$event = $model->getEvent();
		
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
	}
	
	/**
	 * Method to save tickets position
	 *
	 * @return	javascript
	 */
	public function tickets() {
		// Get the model
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			// Save the tickets configuration
			$model->tickets();
		}
		
		echo '<script type="text/javascript">'."\n";
		echo rseventsproHelper::modalClose(false,true)."\n";
		echo '</script>'."\n";
		exit();
	}
	
	/**
	 * Method to save the report message
	 *
	 * @return	javascript
	 */
	public function report() {
		// Check for request forgeries
		Session::checkToken() or jexit('Invalid Token');
		
		// Get the model
		$model = $this->getModel();
		
		// Report 
		if ($model->getCanreport()) {
			$model->report();
			echo '<div class="rs_message_info">'.Text::_('COM_RSEVENTSPRO_REPORT_ADDED').'</div>';
			echo '<script type="text/javascript">window.top.setTimeout(\''.addslashes(rseventsproHelper::modalClose(false,true)).'\',1200);</script>';
		} else {
			echo '<script type="text/javascript">'."\n";
			echo rseventsproHelper::modalClose(false,true)."\n";
			echo '</script>'."\n";
		}
	}
	
	// Delete reports
	public function deletereports() {
		Session::checkToken() or jexit(Text::_('JINVALID_TOKEN'));
		
		// Get the selected items
		$pks = Factory::getApplication()->input->get('cid', array(0), 'array');
		
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		// Force array elements to be integers
		$pks = array_map('intval',$pks);
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		// Delete reports
		if ($model->getCanreport()) {
			if ($admin || $event->owner == $user || $event->sid == $user) {
				$model->deletereports($pks);
			}
		}
		
		$this->setRedirect(base64_decode(Factory::getApplication()->input->getString('return')), Text::_('COM_RSEVENTSPRO_REPORTS_DELETED'));
	}
	
	// Confirm a subscriber
	public function confirm() {
		// Get the model
		$model = $this->getModel();
		// Get the subscription id
		$id = Factory::getApplication()->input->getInt('id',0);
		// Get code
		$code = Factory::getApplication()->input->getString('code');
		// Get state
		$state = Factory::getApplication()->input->getInt('state');
		
		echo $model->confirm($id, $code, $state);
		exit();
	}
	
	// Get events for the map
	public function getajaxmap() {
		// Get the model
		$model = $this->getModel();
		
		header('Content-Type: application/json');
		
		echo json_encode($model->getMapItems());
		
		exit();
	}
	
	// Order tickets
	public function ticketsorder() {
		// Get the model
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			// Save the tickets ordering
			$model->ticketsorder();
		}
		
		exit();
	}
	
	// Save user info
	public function saveuser() {
		Session::checkToken() or jexit(Text::_('JINVALID_TOKEN'));
		
		$model	= $this->getModel();
		$data	= Factory::getApplication()->input->get('jform', array(), 'array');
		
		if (Factory::getUser()->get('id') == $data['id']) {
			if ($model->saveuser($data)) {
				$this->setMessage(Text::_('COM_RSEVENTSPRO_USER_DATA_SAVED'));
			} else {
				$this->setMessage($model->getError(), 'error');
			}
		}
		
		$this->setRedirect(Route::_('index.php?option=com_rseventspro&layout=edituser&id='.rseventsproHelper::sef($data['id'], $data['name'])));
	}
	
	// Delete user image
	public function deleteimage() {
		$id		= Factory::getApplication()->input->getInt('id');
		$model	= $this->getModel();
		
		if (Factory::getUser()->get('id') == $id) {
			$model->deleteimage();
		}
		
		$this->setRedirect(Route::_('index.php?option=com_rseventspro&layout=edituser&id='.$id));
	}
	
	public function rsvp() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		$data   = array();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$id		= Factory::getApplication()->input->getInt('id');
			$rsvp	= Factory::getApplication()->input->get('rsvp');
			
			$model->rsvp($id, $rsvp);
			
			$data['success'] = true;
			$data['status'] = rseventsproHelper::RSVPStatus($rsvp);
		} else {
			$data['success'] = false;
			$data['message'] = Text::_('COM_RSEVENTSPRO_ERROR_SUBSCRIBER_STATUS');
		}
		
		header('Content-Type: application/json');
		echo json_encode($data);
		exit();
	}
	
	public function removersvp() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->removersvp();
			$msg = Text::_('COM_RSEVENTSPRO_RSVP_GUEST_REMOVED');
		} else $msg = Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED');
		
		$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=rsvp&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
	}
	
	public function exportrsvpguests() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->exportrsvpguests();
		} else {
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)), Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'));
		}
	}
	
	public function savespeaker() {
		// Get the model
		$model = $this->getModel('rseventspro');
		$admin = rseventsproHelper::admin();
		
		if (!$admin && empty($this->permissions['can_add_speaker']))
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_ADD_SPEAKER'), 500);
		
		$speakers = $model->savespeaker();
		
		echo '<script type="text/javascript">'."\n";
		echo 'var data = '.json_encode($speakers).';'."\n";
		echo 'window.parent.rsepro_update_speakers(data);'."\n";
		echo rseventsproHelper::modalClose(false, true)."\n";
		echo '</script>';
		
		exit();
	}
	
	public function savesponsor() {
		// Get the model
		$model = $this->getModel('rseventspro');
		$admin = rseventsproHelper::admin();
		
		if (!$admin && empty($this->permissions['can_add_sponsor']))
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_ADD_SPONSOR'), 500);
		
		$sponsors = $model->savesponsor();
		
		echo '<script type="text/javascript">'."\n";
		echo 'var data = '.json_encode($sponsors).';'."\n";
		echo 'window.parent.rsepro_update_sponsors(data);'."\n";
		echo rseventsproHelper::modalClose(false, true)."\n";
		echo '</script>';
		
		exit();
	}
	
	public function duplicateticket() {
		// Get the model
		$model = $this->getModel();
		
		$permission_denied = false;
		$admin = rseventsproHelper::admin();
		
		if (!$admin) {
			Factory::getApplication()->input->set('from','ticket');
			$user = $model->getUser();
			$owner = $model->getOwner();
			if (($user != $owner && empty($this->permissions['can_edit_events'])) || empty($user))
				$permission_denied = true;
		}
		
		if ($permission_denied)
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_DUPLICATE_TICKET'), 500);
		
		$response = $model->duplicateticket();
		
		echo json_encode($response);
		
		exit();
	}
	
	public function waiting() {
		$app = Factory::getApplication();
		
		// Get the model
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		if (!rseventsproHelper::hasWaitingList($event->id)) {
			echo rseventsproHelper::redirect(true,Text::_('COM_RSEVENTSPRO_WAITING_ERROR'),rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)),true);
			
			return false;
		}
		
		try {
			$model->waiting();
		} catch (Exception $e) {
			$app->enqueueMessage($e->getMessage(), 'error');
			$app->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=waiting&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
		}
		
		echo rseventsproHelper::redirect(true,Text::_('COM_RSEVENTSPRO_WAITING_ADDED'),rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)),true);
	}
	
	public function approvewaiting() {
		$app = Factory::getApplication();
		
		// Get the model
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			if ($model->approvewaiting()) {
				$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_USER_APPROVED'));
			} else {
				$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_USER_APPROVED_ERROR'),'error');
			}
		} else {
			$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'),'error');
		}
		
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=waitinglist&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
	}
	
	public function removewaiting() {
		$app = Factory::getApplication();
		
		// Get the model
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->removewaiting();
			$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_USER_REMOVED'));
		} else {
			$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'),'error');
		}
		
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=waitinglist&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
	}
	
	public function savewaitinglist() {
		$app = Factory::getApplication();
		
		// Get the model
		$model = $this->getModel();
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->savewaitinglist();
			$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_WAITINGLIST_USER_SAVED'));
		} else {
			$app->enqueueMessage(Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'),'error');
		}
		
		return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=waitinglist&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)));
	}
	
	public function exportunsubscribers() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->exportunsubscribers();
		} else {
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)), Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED'));
		}
	}
	
	public function removeunsubscriber() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->removeunsubscriber();
			$msg = Text::_('COM_RSEVENTSPRO_UNSUBSCRIBER_REMOVED');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=unsubscribers&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		} else {
			$msg = Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		}
	}
	
	public function cancelevent() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->cancel();
			$msg = Text::_('COM_RSEVENTSPRO_EVENT_CANCELED');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		} else {
			$msg = Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		}
	}
	
	// Subscribers - Invoice
	public function invoice() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$id		= Factory::getApplication()->input->getInt('id');
		$admin	= rseventsproHelper::admin();
		$email	= $model->getEmailFromCode();
		$user	= $model->getUser();
		$table	= Table::getInstance('Subscription','RseventsproTable');
		$lang	= Factory::getLanguage()->getTag();
		
		$table->load($id);
		
		if ($admin || (isset($event->owner) && $event->owner == $user) || (isset($event->sid) && $event->sid == $user) || $email == $table->email || $user == $table->idu || !empty($this->permissions['can_edit_subscriptions'])) {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/invoice.php';
			
			$invoice = RSEventsProInvoice::getInstance($id, $lang);
		
			$invoice->output();
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_ERROR_INVOICE'), 'error');
			$this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro',false));
		}
	}
	
	public function saveticketoptions() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		// Get event details
		$event = $model->getEvent();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$data = Factory::getApplication()->input->get('jform', array(), 'array');
			
			if ($model->saveticketoptions($data)) {
				echo '<script type="text/javascript">'."\n";
				echo 'window.parent.jQuery().colorbox.close();'."\n";
				echo 'window.parent.location.reload();'."\n";
				echo '</script>'."\n";
				exit();
			} else {
				Factory::getApplication()->redirect(Route::_('index.php?option=com_rseventspro&layout=ticketoptions&tmpl=component&id='.$data['id'], false));
			}
		}
	}
	
	public function contact() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		if (!empty($event->options['contact'])) {
			$data = Factory::getApplication()->input->get('jform', array(), 'array');
			
			if ($model->contact($data)) {
				Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_CONTACT_FORM_SUBMITED'));
			} else {
				Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_CONTACT_FORM_SUBMITED_ERROR'), 'error');
			}
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_RSEVENTSPRO_CONTACT_FORM_DISABLED'), 'error');
		}
		
		Factory::getApplication()->redirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false));
	}
	
	public function saveinvoice() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		// Get event details
		$event = $model->getEvent();
		
		$jinput	= Factory::getApplication()->input;
		$data	= $jinput->get('jform', array(), 'array');
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->saveinvoice($data);
		}
		
		echo '<script>';
		echo 'window.parent.jQuery(\'#rsepro-invoice\').modal(\'hide\')';
		echo '</script>';
		die;
	}
	
	public function feedback() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$canSubmit = rseventsproHelper::canSubmitFeedback($event->id);
		
		if ($canSubmit === true) {
			$data = Factory::getApplication()->input->get('jform', array(), 'array');
			$model->feedback($data);
			
			$message = Text::_('COM_RSEVENTSPRO_EVENT_FEEDBACK_FORM_SUBMITED');
		} elseif ($canSubmit === false) {
			$message = Text::_('COM_RSEVENTSPRO_ERROR_FEEDBACK_VIEW');
		} else {
			$message = $canSubmit;
		}
		
		echo rseventsproHelper::redirect(true,$message,rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)),true);
		
		Factory::getApplication()->input->set('tmpl','component');
	}
	
	public function getFeedbackForm() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->getFeedbackForm();
		}
		
		die;
	}
	
	public function loadData() {
		// Get the model
		$model = $this->getModel();
		
		$event = $model->getEvent();
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || ($event && $event->owner == $user) || ($event && $event->sid == $user)) {
			$id = Factory::getApplication()->input->getInt('id', 0);
		
			$task = $this->getTask();
			
			$response = $model->$task($id);
			
			echo json_encode($response);
		}
		
		exit();
	}
	
	public function saveData() {
		// Get the model
		$model = $this->getModel();
		
		$event = $model->getEvent();
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || ($event && $event->owner == $user) || ($event && $event->sid == $user)) {
			$task = $this->getTask();
			
			if ($task == 'saveFeedback') {
				$data = Factory::getApplication()->input->get('feedback', array(), 'array');
			} else if ($task == 'saveFaq') {
				$data = Factory::getApplication()->input->get('faq', array(), 'array');
			} else if ($task == 'saveItinerary') {
				$data = Factory::getApplication()->input->get('itinerary', array(), 'array');
			} else if ($task == 'saveTimeline') {
				$data = Factory::getApplication()->input->get('timeline', array(), 'array');
			}
			
			$response = $model->$task($data);
			
			echo json_encode($response);
		}
		
		exit();
	}
	
	public function removeData() {
		// Get the model
		$model = $this->getModel();
		
		$event = $model->getEvent();
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || ($event && $event->owner == $user) || ($event && $event->sid == $user)) {
			$id = Factory::getApplication()->input->getInt('id', 0);
			
			$task = $this->getTask();
			
			$model->$task($id);
		}
		
		exit();
	}
	
	public function orderData() {
		// Get the model
		$model = $this->getModel();
		
		$event = $model->getEvent();
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || ($event && $event->owner == $user) || ($event && $event->sid == $user)) {
			$task = $this->getTask();
			
			$model->$task();
		}
		
		exit();
	}
	
	public function deletefeedback() {
		// Get the model
		$model = $this->getModel('rseventspro');
		
		// Get event details
		$event = $model->getEvent();
		
		$admin = rseventsproHelper::admin();
		$user  = $model->getUser();
		
		if ($admin || $event->owner == $user || $event->sid == $user) {
			$model->deletefeedback();
			$msg = Text::_('COM_RSEVENTSPRO_FEEDBACK_REMOVED');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=feedbacks&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		} else {
			$msg = Text::_('COM_RSEVENTSPRO_GLOBAL_PERMISSION_DENIED');
			return $this->setRedirect(rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false),$msg);
		}
	}
}