<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Mail\MailHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\Filesystem\File;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;

class rseventsproEmails
{	
	/*
	*	Replace placeholders
	*/
	
	public static function placeholders($text, $ide, $name, $optionals = null, $ids = null, $lang = null) {
		if (!is_null($ids)) {
			$db		= Factory::getDbo();
			$query	= $db->getQuery(true);
				
				$query->clear()
					->select($db->qn('lang'))->select($db->qn('hash'))
					->from($db->qn('#__rseventspro_users'))
					->where($db->qn('id').' = '.$db->q($ids));
			$db->setQuery($query);
			$subscription = $db->loadObject();
			$lang = isset($subscription->lang) ? $subscription->lang : null;
		}
		
		$placeholders	= rseventsproEmails::getPlaceholders($ide, $name, $lang);
		$search			= $placeholders['search'];
		$replace		= $placeholders['replace'];
		$optionalsPlace = array('{TicketInfo}', '{TicketsTotal}', '{Discount}', '{Tax}', '{LateFee}', '{EarlyDiscount}', '{Gateway}', '{IP}', '{Coupon}');
		
		Factory::getApplication()->triggerEvent('onrseproBeforePlaceholders', array(array('text' => &$text, 'ide' => $ide, 'ids' => $ids, 'optionals' => $optionals)));
		
		if (isset($subscription)) {
			if ($hash = $subscription->hash) {
				$unsubscribe = Uri::getInstance()->toString(array('scheme','host')).rseventsproHelper::route('index.php?option=com_rseventspro&task=rseventspro.unsubscribe&hash='.$hash.'-'.$ide);
				
				if (Factory::getApplication()->isClient('administrator')) {
					$unsubscribe = str_replace('/administrator','',$unsubscribe);
				}
				
				array_push($search, '{UnsubscribeURL}');
				array_push($replace, $unsubscribe);
			}
		}
		
		if (is_array($text)) {
			foreach($text as $name => $value) {
				if ($value) {
					$text[$name] = str_replace($search,$replace,$value);
				}
			}
			
			if (!is_null($optionals) && is_array($optionals)) {
				$text['body'] = str_replace($optionalsPlace, $optionals, $text['body']);
			}
		} else {
			$text = str_replace($search,$replace,$text);
			
			if (!is_null($optionals) && is_array($optionals)) {
				$text = str_replace($optionalsPlace, $optionals, $text);
			}
		}
		
		Factory::getApplication()->triggerEvent('onrseproAfterPlaceholders', array(array('text' => &$text, 'ide' => $ide, 'ids' => $ids)));
		
		return $text;
	}
	
	/*
	*	Get available placeholders
	*/
	
	public static function getPlaceholders($id, $name, $lang) {
		static $cache = array();
		$hash = md5($id.$name);
		
		if (!isset($cache[$hash])) {
			// Get the site root
			$u		= Uri::getInstance();	
			$root	= $u->toString(array('scheme','host'));
			
			// Load language
			Factory::getLanguage()->load('com_rseventspro');
			
			$details	= rseventsproHelper::details($id, null, false, $lang);
			$event		= $details['event'];
			$categories	= $details['categories'];
			$tags		= $details['tags'];
			
			// The event link
			$eventlink = $root.rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id));
			
			// The location link
			$locationlink = $root.rseventsproHelper::route('index.php?option=com_rseventspro&layout=location&id='.rseventsproHelper::sef($event->locationid,$event->location));
			
			if (Factory::getApplication()->isClient('administrator')) {
				$eventlink = str_replace('/administrator','',$eventlink);
				$locationlink = str_replace('/administrator','',$locationlink);
			}

			// Event times
			$startdate	= $event->allday ? rseventsproHelper::showdate($event->start,rseventsproHelper::getConfig('global_date'),true) : rseventsproHelper::showdate($event->start,null,true);
			$sdate		= rseventsproHelper::showdate($event->start,rseventsproHelper::getConfig('global_date'));
			$sdatetime	= $event->allday ? '' : rseventsproHelper::showdate($event->start,rseventsproHelper::getConfig('global_time'));
			$enddate	= $event->allday ? '' : rseventsproHelper::showdate($event->end,null,true);
			$edate		= $event->allday ? '' : rseventsproHelper::showdate($event->end,rseventsproHelper::getConfig('global_date'));
			$edatetime	= $event->allday ? '' : rseventsproHelper::showdate($event->end,rseventsproHelper::getConfig('global_time'));
			
			$owner		= Factory::getUser($event->owner);
			$jform		= Factory::getApplication()->input->get('jform', array(), 'array');
			$message	= isset($jform['message']) ? $jform['message'] : '';
			$timezone	= rseventsproHelper::getTimezone();
			
			$search = array('{EventName}','{EventLink}','{EventDescription}','{EventStartDate}','{EventStartDateOnly}','{EventStartTime}','{EventEndDate}','{EventEndDateOnly}','{EventEndTime}','{Owner}','{OwnerUsername}','{OwnerName}','{OwnerEmail}','{EventURL}','{EventPhone}','{EventEmail}','{LocationName}','{LocationLink}','{LocationDescription}','{LocationURL}','{LocationAddress}','{EventCategories}','{EventTags}','{EventIconSmall}','{EventIconBig}', '{Message}', '{message}', '{User}', '{user}', '{timezone}', '{EventSmallDescription}');
			$replace = array($event->name, $eventlink, $event->description, $startdate, $sdate, $sdatetime, $enddate, $edate, $edatetime, $event->ownername, $owner->get('username'), $owner->get('name'), $owner->get('email'), $event->URL, $event->phone, $event->email, $event->location, $locationlink, $event->ldescription, $event->locationlink, $event->address, $categories, $tags, $details['image_s'], $details['image_b'], $message, $message, $name, $name, $timezone, $event->small_description);
			
			$cache[$hash] = array('search' => $search, 'replace' => $replace);
		}
		
		return $cache[$hash];
	}
	
	
	/*
	*	Invite e-mail
	*/
	
	public static function invite($from, $fromName, $to, $ide, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('invite', null, null, $lang);
		
		if (empty($email) || !$email->enable)
			return false;
		
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $to);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$text['body']		= str_replace(array('{from}','{fromname}'),array($from, $fromName), $text['body']);
		
		try {
			rseventsproEmails::sendMail($from , $fromName , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	
	/*
	*	Registration e-mail
	*/
	
	public static function registration($to, $ide, $name, $optionals, $ids = null) {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('registration', $to, $ide);
		
		if (empty($email) || !$email->enable || rseventsproEmails::getDisableOption($ide, 'registration'))
			return false;
			
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		$paymentURL = '';
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		if ($ids) {
			$db = Factory::getDbo();
			
			$query = $db->getQuery(true)->select($db->qn('URL'))
					->from($db->qn('#__rseventspro_users'))
					->where($db->qn('id').' = '.$db->q($ids));
			$db->setQuery($query);
			if ($url = $db->loadResult()) {
				$paymentURL = base64_decode($url);
				$root = Uri::getInstance()->toString(array('scheme','host'));
				if (strpos($paymentURL,$root) === false) {
					$paymentURL = $root.$paymentURL;
				}
			}
		}
		
		Factory::getApplication()->triggerEvent('onrseproRegistrationEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, $optionals, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$text['body']		= str_replace('{PaymentURL}',$paymentURL,$text['body']);
		
		if ($mode) {
			$text['body'] .= rseventsproHelper::richSnippetEmail($ids);
		}
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	
	/*
	*	Activation e-mail
	*/
	
	public static function activation($to, $ide, $name, $optionals, $ids = null) {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('activation', $to, $ide);
		
		if (empty($email) || !$email->enable || rseventsproEmails::getDisableOption($ide, 'activation'))
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproActivationEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$text		 		= rseventsproEmails::placeholders($replacer, $ide, $name, $optionals, $ids);
		$text['cc']	 		= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']	 	= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$attachments 		= rseventsproEmails::pdfAttachement($to,$ide,$name,$optionals,$ids);
		
		if ($mode) {
			$text['body'] .= rseventsproHelper::richSnippetEmail($ids);
		}
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , $attachments , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		Factory::getApplication()->triggerEvent('onrsepro_activationEmailCleanup',array(array('id'=>&$ide)));
		
		return true;
	}
	
	
	/*
	*	Unsubscribe e-mail
	*/
	
	public static function unsubscribe($to, $ide, $name, $lang = 'en-GB', $ids = null) {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('unsubscribe', null, null, $lang);
		
		if (empty($email) || !$email->enable)
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproUnsubscribeEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$optionals			= rseventsproEmails::createOptionals($ids);
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, $optionals, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	Denied e-mail
	*/
	
	public static function denied($to, $ide, $name, $ids = null) {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('denied', $to, $ide);
		
		if (empty($email) || !$email->enable || rseventsproEmails::getDisableOption($ide, 'denied'))
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproDeniedEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$optionals			= rseventsproEmails::createOptionals($ids);
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, $optionals, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	Refund e-mail
	*/
	
	public static function refund($to, $ide, $name, $ids = null) {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('refund', $to, $ide);
		
		if (empty($email) || !$email->enable || rseventsproEmails::getDisableOption($ide, 'refund'))
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproRefundEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$optionals			= rseventsproEmails::createOptionals($ids);
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, $optionals, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	Reminder e-mail
	*/
	
	public static function reminder($subscriber, $ide, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('reminder', null, null, $lang);
		
		if (empty($email) || !$email->enable)
			return false;
			
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$optionals			= !empty($subscriber->id) ? rseventsproEmails::createOptionals($subscriber->id) : null;
		$text				= rseventsproEmails::placeholders($replacer, $ide, $subscriber->name, $optionals, $subscriber->id);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $subscriber->email , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	
	/*
	*	Post-reminder e-mail
	*/
	
	public static function postreminder($to, $ide, $name, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('preminder', null, null, $lang);
		$db			= Factory::getDbo();
		
		if (empty($email) || !$email->enable)
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, null, null, $lang);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		$query = $db->getQuery(true)
			->clear()
			->select($db->qn('name'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		$name = $db->loadResult();
		
		// The feedback link
		$feedbacklink = Uri::getInstance()->toString(array('scheme','host')).rseventsproHelper::route('index.php?option=com_rseventspro&layout=feedback&id='.rseventsproHelper::sef($ide, $name).'&hash='.md5($ide.$to),false,rseventsproHelper::itemid($ide));
		$text['body'] = str_replace('{FeedbackURL}', $feedbacklink, $text['body']);
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	Guests e-mail
	*/
	
	public static function guests($to, $ide, $name, $subject, $body, $lang = null) {
		$config		= rseventsproHelper::getConfig();
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= 1;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, null, null, $lang);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'], $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	
	/*
	*	Moderation email
	*/
	
	public static function moderation($to, $ide, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('moderation', null, null, $lang);
		
		if (empty($email))
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		
		$text				= rseventsproEmails::placeholders($replacer,$ide,'');
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$approve			= rseventsproHelper::route(Uri::root().'index.php?option=com_rseventspro&task=activate&key='.md5('event'.$ide));
		$text['body']		= str_replace('{EventApprove}',$approve,$text['body']);
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	public static function tag_moderation($to, $ide, $items, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('tag_moderation', null, null, $lang);
		
		if (empty($email))
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer,$ide,'');
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		// html
		if ($mode) {
			$approve = '<ul>';
			foreach ($items as $item) {
				$link = rseventsproHelper::route(Uri::root().'index.php?option=com_rseventspro&task=tagactivate&key='.md5('tag'.$item->id));
				$approve .= "\n".'<li><a href="'.$link.'">'.Text::sprintf('RSEPRO_APPROVE_TAG', $item->name).'</a></li>';
			}
			$approve .= '</ul>';
		} else // no html
		{
			$approve = '';
			foreach ($items as $item) {
				$link = rseventsproHelper::route(Uri::root().'index.php?option=com_rseventspro&task=tagactivate&key='.md5('tag'.$item->id));
				$approve .= "\n".Text::sprintf('RSEPRO_APPROVE_TAG', $item->name).': '.$link;
			}
		}
		$text['body'] = str_replace('{TagsApprove}',$approve,$text['body']);
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	Approval e-mail
	*/
	
	public static function approval($to, $ide, $name, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('approval', null, null, $lang);
		
		if (empty($email) || empty($to))
			return false;
		
		if (!$email->enable)
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	New event subscription notification email
	*/
	
	public static function notify_me($to, $ide, $additional_data = array(), $lang = 'en-GB', $optionals = null, $ids = null) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('notify_me', null, null, $lang);
		$emails		= array();
		
		if (empty($email))
			return false;
		
		$query->clear()
			->select($db->qn('notify_me'))->select($db->qn('notify_me_emails'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		$event = $db->loadObject();
		
		if ($event->notify_me) {
			$emails[] = $to;
		}
		
		if (!empty($event->notify_me_emails)) {
			if ($notify_emails = explode(',', $event->notify_me_emails)) {
				foreach ($notify_emails as $notify_email) {
					$notify_email = trim($notify_email);
					if (MailHelper::isEmailAddress($notify_email)) {
						$emails[] = $notify_email;
					}
				}
			}
		}
		
		if (empty($emails)) {
			return false;
		}
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproNotifyEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$text				= rseventsproEmails::placeholders($replacer,$ide,'',$optionals, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		if ($additional_data) {
			$text['body']		= str_replace(array_keys($additional_data), array_values($additional_data), $text['body']);
			$text['subject']	= str_replace(array_keys($additional_data), array_values($additional_data), $text['subject']);
		}
		
		try {
			foreach ($emails as $notify_email) {
				rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $notify_email , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
			}
			
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	New paid subscription notification email
	*/
	
	public static function notify_me_paid($to, $ide, $additional_data = array(), $lang = 'en-GB', $optionals = null, $ids = null) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('notify_me_paid', null, null, $lang);
		$emails		= array();
		
		if (empty($email))
			return false;
		
		$query->clear()
			->select($db->qn('notify_me_paid'))->select($db->qn('notify_me_paid_emails'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		$event = $db->loadObject();
		
		if ($event->notify_me_paid) {
			$emails[] = $to;
		}
		
		if (!empty($event->notify_me_paid_emails)) {
			if ($notify_paid_emails = explode(',', $event->notify_me_paid_emails)) {
				foreach ($notify_paid_emails as $notify_paid_email) {
					$notify_paid_email = trim($notify_paid_email);
					if (MailHelper::isEmailAddress($notify_paid_email)) {
						$emails[] = $notify_paid_email;
					}
				}
			}
		}
		
		if (empty($emails)) {
			return false;
		}
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproNotifyPaidEmail', array(array('ids' => $ids, 'ide' => $ide, 'data' => &$replacer)));
		
		$text				= rseventsproEmails::placeholders($replacer,$ide,'',$optionals, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		if ($additional_data) {
			$text['body']		= str_replace(array_keys($additional_data), array_values($additional_data), $text['body']);
			$text['subject']	= str_replace(array_keys($additional_data), array_values($additional_data), $text['subject']);
		}
		
		try {
			foreach ($emails as $notify_paid_email) {			
				rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $notify_paid_email , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
			}
		} catch(Exception $e) {}
		
		return true;
	}
	
	
	/*
	*	Event owner unsubscribe notification
	*/
	
	public static function notify_me_unsubscribe($to, $ide, $additional_data = array(), $lang = 'en-GB', $ids = null) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('notify_me_unsubscribe', null, null, $lang);
		$emails		= array();
		
		if (empty($email))
			return false;
		
		$query->clear()
			->select($db->qn('notify_me_unsubscribe'))->select($db->qn('notify_me_unsubscribe_emails'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		$event = $db->loadObject();
		
		if ($event->notify_me_unsubscribe) {
			$emails[] = $to;
		}
		
		if (!empty($event->notify_me_unsubscribe_emails)) {
			if ($notify_unsub_emails = explode(',', $event->notify_me_unsubscribe_emails)) {
				foreach ($notify_unsub_emails as $notify_unsub_email) {
					$notify_unsub_email = trim($notify_unsub_email);
					if (MailHelper::isEmailAddress($notify_unsub_email)) {
						$emails[] = $notify_unsub_email;
					}
				}
			}
		}
		
		if (empty($emails)) {
			return false;
		}
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		Factory::getApplication()->triggerEvent('onrseproNotifyUnsubscribeEmail', array(array('ids' => $ids, 'data' => &$replacer)));
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, '', null, $ids);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		if ($additional_data) {
			$text['body']		= str_replace(array_keys($additional_data), array_values($additional_data), $text['body']);
			$text['subject']	= str_replace(array_keys($additional_data), array_values($additional_data), $text['subject']);
		}
		
		try {
			foreach ($emails as $notify_unsub_email) {
				rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $notify_unsub_email , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
			}
		} catch(Exception $e) {}
		
		return true;
	}
	
	/*
	*	Report email
	*/
	public static function report($to, $ide, $additional_data = array(), $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('report', null, null, $lang);
		
		if (empty($email) || empty($to) || !$email->enable)
			return false;
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer,$ide,'');
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		if ($additional_data) {
			$text['body']		= str_replace(array_keys($additional_data), array_values($additional_data), $text['body']);
			$text['subject']	= str_replace(array_keys($additional_data), array_values($additional_data), $text['subject']);
		}
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	
	/*
	*	Rule email
	*/
	
	public static function rule($ids, $message, $ruleemail) {
		$config		= rseventsproHelper::getConfig();
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$paymentURL = '';
		$cart		= false;
		
		Factory::getApplication()->triggerEvent('onrsepro_isCart', array(array('cart' => &$cart)));
		
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		
		// Get subscription details
		$query->clear()
			->select('u.*')
			->from($db->qn('#__rseventspro_users', 'u'))
			->where($db->qn('u.id').' = '.(int) $ids);
			
		if ($cart) {
			$query->select($db->qn('c.events'));
			$query->join('LEFT', $db->qn('#__rseventspro_cart', 'c').' ON '.$db->qn('u.id').' = '.$db->qn('c.ids'));
		}
		
		$db->setQuery($query);
		if ($subscription = $db->loadObject()) {
			if ($subscription->URL) {
				$paymentURL = base64_decode($subscription->URL);
				$root = Uri::getInstance()->toString(array('scheme','host'));
				if (strpos($paymentURL,$root) === false) {
					$paymentURL = $root.$paymentURL;
				}
			}
			
			// Get the event ID's
			$events = array($subscription->ide);
			
			if ($cart) {
				$events = explode(',', $subscription->events);
				$events = array_map('intval', $events);
			}
			
			$email = rseventsproEmails::emailrule($message, $subscription->lang);
			
			if (!$email) {
				return false;
			}
			
			$mode			= @$email->mode;
			$subject		= @$email->subject;
			$body			= @$email->message;
			
			$replacer	= array(
				'from'		=> $from,
				'fromName'	=> $fromName,
				'replyto'	=> $replyto,
				'replyname' => $replyname,
				'cc'		=> $cc,
				'bcc'		=> $bcc,
				'subject'	=> $subject,
				'body'		=> $body
			);

			$toEmail = $ruleemail ? $ruleemail : $subscription->email;

			foreach ($events as $event) {
				$text				= rseventsproEmails::placeholders($replacer, $event, $subscription->name);
				$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
				$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
				$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
				$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
				$text['body']		= str_replace('{Status}',rseventsproHelper::getStatuses($subscription->state),$text['body']);
				$text['body']		= str_replace('{PaymentURL}',$paymentURL,$text['body']);
				
				try {
					rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $toEmail , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
				} catch(Exception $e) {}
			}
		}
		
		return true;
	}
	
	/*
	*	Attach the pdf to the activation email
	*/
	public static function pdfAttachement($to, $ide, $name, $optionals, $ids) {
		$attachments = null;
		
		if (rseventsproHelper::pdf()) {
			$app	= Factory::getApplication();
			$db		= Factory::getDBO();
			$query	= $db->getQuery(true);
			$now	= rseventsproHelper::showdate('now');
			$array	= array();
			
			$query->clear()
				->select($db->qn('lang'))
				->from($db->qn('#__rseventspro_users'))
				->where($db->qn('id').' = '.$db->q($ids));
			$db->setQuery($query);
			$lang = $db->loadResult();
			
			$query->clear()
				->select($db->qn('t').'.*')->select($db->qn('ut.quantity'))
				->from($db->qn('#__rseventspro_tickets','t'))
				->join('LEFT',$db->qn('#__rseventspro_user_tickets','ut').' ON '.$db->qn('t.id').' = '.$db->qn('ut.idt'))
				->where($db->qn('t.attach').' = '.$db->q(1))
				->where($db->qn('t.ide').' = '.$db->q($ide))
				->where($db->qn('ut.ids').' = '.$db->q($ids));
			$db->setQuery($query);
			if ($tickets = $db->loadObjectList()) {
				foreach ($tickets as $ticket) {
					// Replace translations
					if ($translation = RSEventsProTranslations::getTranslation('ticket', $ticket->id, 'layout', $lang)) {
						$ticket->layout = $translation;
					}
					
					if ($translation = RSEventsProTranslations::getTranslation('ticket', $ticket->id, 'name', $lang)) {
						$ticket->name = $translation;
					}
					
					for ($i = 1; $i <= $ticket->quantity; $i++) {
						$code		= md5($ids.$ticket->id.$i);
						$code		= substr($code,0,4).substr($code,-4);
						$barcodetext= rseventsproHelper::getBarcodeOptions('barcode_prefix', 'RST-').$ids.'-'.$code;
						$barcodetext= in_array(rseventsproHelper::getBarcodeOptions('barcode', 'C39'), array('C39', 'C93')) ? strtoupper($barcodetext) : $barcodetext;
						$layout		= $ticket->layout;
						
						$app->triggerEvent('onrseproTicketPDFLayout',array(array('ids' => $ids, 'ide' => $ide, 'layout' => &$layout)));
						
						$app->triggerEvent('onrsepro_beforeReplacePDFLayout', array(array('layout' => &$layout, 'ids' => $ids, 'idt' => $ticket->id, 'ide' => $ide, 'position' => $i)));
						
						$layout = rseventsproEmails::placeholders($layout, $ide, $name, $optionals, $ids);
						$layout = str_replace('{sitepath}', JPATH_SITE, $layout);
						
						if (strpos($layout,'{barcode}') !== FALSE) {
							require_once JPATH_SITE.'/components/com_rseventspro/helpers/pdf/barcodes.php';
							
							$barcodeOption = rseventsproHelper::getBarcodeOptions('barcode', 'qrcode');
							$barcodetype = rseventsproHelper::getBarcodeOptions('barcode_type', 'qrcode');
							$barcodetexttype = $barcodetext;
							
							if ($barcodeOption == 'qrcode') {
								if ($barcodetype == 'url') {
									$hash = md5($ids.$ticket->id.$i);
									$hash = substr_replace($hash, '|'.$ids.'|', 6, 0);
									$secret = Factory::getConfig()->get('secret');
									
									try {
										require_once JPATH_SITE.'/components/com_rseventspro/helpers/crypt.php';
										
										$hash = RseventsproCrypt::encrypt($hash, $secret);
										$hash	= 'cr'.$hash;
										$hash	= str_replace(' ', '_', $hash);
									} catch (Exception $e) {}
									
									$barcodetexttype = Uri::root().'index.php?option=com_rseventspro&task=confirm&hash='.$hash;
								}
							}
							
							$barcodePDF = new TCPDFBarcode($barcodetexttype, $barcodeOption);
							$size = $barcodeOption == 'qrcode' ? '100px' : '300px';
							
							ob_start();
							$barcodePDF->getBarcodePNG();
							$thecode = ob_get_contents();
							ob_end_clean();
							
							$file = JPATH_SITE.'/components/com_rseventspro/assets/barcode/rset-'.md5($barcodetext).'.png';
							$upload = File::write($file,$thecode);
							$barcodeHTML = $upload ? '<img src="'.$file.'" alt="" width="'.$size.'" />' : '';
							
							$layout = str_replace('{barcode}', $barcodeHTML, $layout);
							
							$array[] = $file;
						}
						
						$seat 	= rseventsproEmails::getSeat($ids, $ticket->id, $i);
						$layout = str_replace(array('{useremail}', '{barcodetext}', '{date}', '{seat}'), array($to, $barcodetext, $now, $seat), $layout);
						
						$app->triggerEvent('onrsepro_activationEmail', array(array('id' => &$ide, 'name' => $ticket->name.' '.$i, 'attachment' => &$attachments, 'layout' => &$layout, 'lang' => $lang)));
					}
				}
			}
			
			$query->clear()
				->select($db->qn('invoice'))->select($db->qn('invoice_attach'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.$db->q($ide));
			$db->setQuery($query);
			$invoiceDetails = $db->loadObject();
			
			if ($invoiceDetails->invoice && $invoiceDetails->invoice_attach) {
				try {
					require_once JPATH_SITE.'/components/com_rseventspro/helpers/invoice.php';
					
					$tmp	 	= Factory::getConfig()->get('tmp_path');
					$folder		= md5(Factory::getSession()->getId()).'_activation';
					$invoice 	= RSEventsProInvoice::getInstance($ids, $lang);
					$output  	= $invoice->output(false);
					$path	 	= $tmp.'/'.$folder.'/'.$output['title'];
					
					if (File::write($path, $output['buffer'])) {
						$attachments[] = $path;
					}
				} catch (Exception $e) {
					$app->enqueueMessage($e->getMessage(),'info');
				}
			}
			
			if ($array) {
				foreach ($array as $file) {
					File::delete($file);
				}
			}
		}
		
		return $attachments;
	}
	
	/*
	*	Get the subject and message text
	*/
	
	public static function email($type, $to, $ide, $ulang = 'en-GB') {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		
		if (is_null($to) && is_null($ide)) {
			$userlanguage = $ulang;
		} else {
			// Get user language
			$query->clear()
				->select($db->qn('u.lang'))
				->from($db->qn('#__rseventspro_users','u'))
				->where($db->qn('u.email').' = '.$db->q($to))
				->where($db->qn('u.ide').' = '.(int) $ide);
			
			Factory::getApplication()->triggerEvent('onrsepro_subscriptionsQuery', array(array('query' => &$query, 'rule' => 'u.ide')));
			
			$db->setQuery($query);
			$userlanguage = $db->loadResult();
			
			// If we don't find the users language, we set the language to english (en-GB)
			if (empty($userlanguage)) {
				$userlanguage = 'en-GB';
			}
		}
		
		$query->clear()
			->select($db->qn('id'))
			->from($db->qn('#__rseventspro_emails'))
			->where($db->qn('lang').' = '.$db->q($userlanguage))
			->where($db->qn('type').' = '.$db->q($type));
		$db->setQuery($query);
		$emailid = (int) $db->loadResult();
		
		if (!$emailid)
			$userlanguage = 'en-GB';
		
		// Get email details
		$query->clear()
			->select($db->qn('subject'))->select($db->qn('message'))
			->select($db->qn('enable'))->select($db->qn('mode'))
			->from($db->qn('#__rseventspro_emails'))
			->where($db->qn('lang').' = '.$db->q($userlanguage))
			->where($db->qn('type').' = '.$db->q($type));
			
		$db->setQuery($query);
		return $db->loadObject();
	}
	
	/*
	*	Get rule emails
	*/
	
	public static function emailrule($mid, $lang = 'en-GB') {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		
		// Get all emails
		$query->clear()
			->select($db->qn('mode'))->select($db->qn('subject'))
			->select($db->qn('message'))->select($db->qn('lang'))
			->from($db->qn('#__rseventspro_emails'))
			->where($db->qn('type').' = '.$db->q('rule'))
			->where('('.$db->qn('id').' = '.(int) $mid.' OR '.$db->qn('parent').' = '.(int) $mid.')');
			
		$db->setQuery($query);
		$emails = $db->loadObjectList();
		
		if (empty($emails)) 
			return false;
		
		// Search for the email that have the selected language
		foreach ($emails as $email) {
			if ($email->lang == $lang) {
				return $email;
			}
		}
		
		// If there is no email with the selected language get the first email
		return $emails[0];
	}
	
	/*
	*	RSVP Going email
	*/
	public static function rsvpgoing($to, $ide, $lang = null) {
		return rseventsproEmails::rsvpemail('rsvpgoing', $to, $ide, $lang);
	}
	
	/*
	*	RSVP Interested email
	*/
	public static function rsvpinterested($to, $ide, $lang = null) {
		return rseventsproEmails::rsvpemail('rsvpinterested', $to, $ide, $lang);
	}
	
	/*
	*	RSVP Not going email
	*/
	public static function rsvpnotgoing($to, $ide, $lang = null) {
		return rseventsproEmails::rsvpemail('rsvpnotgoing', $to, $ide, $lang);
	}
	
	public static function waitinglist($to, $ide, $name, $hash, $claimDate, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('waitinglist', null, null, $lang);
		
		if (empty($email))
			return false;
			
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$claimURL	= Uri::getInstance()->toString(array('scheme','host')).Route::_('index.php?option=com_rseventspro&task=claim&hash='.$hash, false);
		$claimURL	= str_replace('/administrator', '', $claimURL);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, null, null, $lang);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$text['body']		= str_replace(array('{claim}','{claimdate}'),array($claimURL,$claimDate),$text['body']);
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	public static function waitinglist_email($type, $to, $ide, $name, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email($type, null, null, $lang);
		
		if (empty($email))
			return false;
			
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, null, null, $lang);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	public static function cancel($ide) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$status	= rseventsproHelper::getConfig('cancel_to');
		$ide	= (int) $ide;
		
		$query->clear()
			->select($db->qn('registration'))->select($db->qn('rsvp'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		$event = $db->loadObject();
		
		if ($event->registration) {
			$query->clear()
				->select($db->qn('u.name'))->select($db->qn('u.email'))
				->select($db->qn('u.lang'))
				->from($db->qn('#__rseventspro_users','u'))
				->where($db->qn('u.ide').' = '.$ide)
				->group($db->qn('u.email'));
			
			if ($status) {
				$query->where($db->qn('u.state').' IN ('.$status.')');
			}
			
			Factory::getApplication()->triggerEvent('onrsepro_subscriptionsQuery', array(array('query' => &$query, 'rule' => 'u.ide')));	
		} else if ($event->rsvp) {
			$query->clear()
				->select($db->qn('u.name'))->select($db->qn('u.email'))
				->select($db->qn('r.language','lang'))
				->from($db->qn('#__users','u'))
				->join('LEFT', $db->qn('#__rseventspro_rsvp_users','r').' ON '.$db->qn('u.id').' = '.$db->qn('r.uid'))
				->where($db->qn('r.ide').' = '.$db->q($ide));
			
			if ($status) {
				if ($statuses = explode(',',$status)) {
					foreach ($statuses as $i => $statusID) {
						if ($statusID == 0) $statuses[$i] = $db->q('interested');
						elseif ($statusID == 1) $statuses[$i] = $db->q('going');
						elseif ($statusID == 2) $statuses[$i] = $db->q('notgoing');
					}
					
					$status = implode(',', $statuses);
					$query->where($db->qn('r.rsvp').' IN ('.$status.')');
				}
			}
		} else {
			return false;
		}
		
		$db->setQuery($query);
		if ($subscribers = $db->loadObjectList()) {
			foreach ($subscribers as $subscriber) {
				rseventsproEmails::cancelemail($subscriber->email, $ide, $subscriber->name, $subscriber->lang);
			}
		}
	}
	
	public static function contact($data, $to, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('contact', null, null, $lang);
		
		if (empty($email))
			return false;
			
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		$body		= str_replace(array('{name}', '{email}', '{message}'), array($data['name'], $data['email'], nl2br($data['message'])), $body);
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $data['id'], '');
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$text['subject']	= str_replace(array('{name}', '{email}'), array($data['name'], $data['email']), $text['subject']);
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	public static function feedback($data, $to, $lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('feedback', null, null, $lang);
		
		if (empty($email) || empty($to) || !$email->enable)
			return false;
			
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		$body		= str_replace(array('{name}', '{email}'), array($data['name'], $data['email']), $body);
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $data['ide'], '');
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		$text['subject']	= str_replace(array('{name}', '{email}'), array($data['name'], $data['email']), $text['subject']);
		
		try {
			rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {}
		
		return true;
	}
	
	protected static function cancelemail($to, $ide, $name, $lang) {
		$config		= rseventsproHelper::getConfig();
		$email		= rseventsproEmails::email('cancel', null, null, $lang);
		
		if (empty($email) || !$email->enable) {
			return false;
		}
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, null, null, $lang);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			return rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {
			return false;
		}
	}
	
	/*
	*	Get RSVP email
	*/
	
	protected static function rsvpemail($type, $to, $ide, $lang = null) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		$check	= str_replace('rsvp', 'rsvp_', $type);
		
		$query->clear()
			->select($db->qn($check))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($ide));
		$db->setQuery($query);
		if (!$db->loadResult()) {
			return false;
		}
		
		$config		= rseventsproHelper::getConfig();
		$lang		= $lang ? $lang : Factory::getLanguage()->getTag();
		$email		= rseventsproEmails::email($type, null, null, $lang);
		
		if (empty($email)) return false;
		
		// Get the name of the subscriber
		$query->clear()
			->select($db->qn('name'))
			->from($db->qn('#__users'))
			->where($db->qn('email').' = '.$db->q($to));
		$db->setQuery($query);
		$name = $db->loadResult();
		
		$from		= $config->email_from;
		$fromName	= $config->email_fromname;
		$mode		= $email->mode;
		$replyto	= $config->email_replyto;
		$replyname	= $config->email_replytoname;
		$cc			= $config->email_cc;
		$bcc		= $config->email_bcc;
		$cc			= !empty($cc) ? $cc : null;
		$bcc		= !empty($bcc) ? $bcc : null;
		$subject	= $email->subject;
		$body		= $email->message;
		
		$replacer	= array(
			'from'		=> $from,
			'fromName'	=> $fromName,
			'replyto'	=> $replyto,
			'replyname' => $replyname,
			'cc'		=> $cc,
			'bcc'		=> $bcc,
			'subject'	=> $subject,
			'body'		=> $body
		);
		
		$text				= rseventsproEmails::placeholders($replacer, $ide, $name, null, null, $lang);
		$text['cc']			= isset($text['cc']) && !empty($text['cc']) ? explode(',',$text['cc']) : null;
		$text['bcc']		= isset($text['bcc']) && !empty($text['bcc']) ? explode(',',$text['bcc']) : null;
		$text['replyto']	= isset($text['replyto']) && !empty($text['replyto']) ? explode(',',$text['replyto']) : null;
		$text['replyname']	= isset($text['replyname']) && !empty($text['replyname']) ? explode(',',$text['replyname']) : null;
		
		try {
			return rseventsproEmails::sendMail($text['from'] , $text['fromName'] , $to , $text['subject'] , $text['body'] , $mode , $text['cc'] , $text['bcc'] , null , $text['replyto'], $text['replyname']);
		} catch(Exception $e) {
			return false;
		}
	}
	
	/*
	*	Create optional placeholders
	*/
	
	protected static function createOptionals($id) {
		$db		= Factory::getDBO();
		$query	= $db->getQuery(true);
		$id		= (int) $id;
		$info	= array();
		$total	= 0;
		
		// Get subscription details
		$query->select('*')
			->from($db->qn('#__rseventspro_users'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$subscription = $db->loadObject();
		
		if ($subscription->ide) {
			// Get tickets
			$tickets = rseventsproHelper::getUserTickets($id, false, $subscription->lang);
			
			if (!empty($tickets)) {
				foreach ($tickets as $ticket) {
					if ($ticket->price > 0) {
						$total += $ticket->price * $ticket->quantity;
						$ticketInfo = $ticket->quantity . ' x ' .$ticket->name.' ('.rseventsproHelper::currency($ticket->price).') ';
					} else {
						$ticketInfo = $ticket->quantity . ' x ' .$ticket->name.' ('.Text::_('COM_RSEVENTSPRO_GLOBAL_FREE').') ';
					}
					
					$info[] = $ticketInfo.rseventsproHelper::getSeats($id,$ticket->id);
				}
			}
			
			if (!empty($subscription->discount) && !empty($total)) {
				$total = $total - $subscription->discount;
			}
			
			if (!empty($subscription->early_fee) && !empty($total)) {
				$total = $total - $subscription->early_fee;
			}
			
			if (!empty($subscription->late_fee) && !empty($total)) {
				$total = $total + $subscription->late_fee;
			}
			
			if (!empty($subscription->tax) && !empty($total)) {
				$total = $total + $subscription->tax;
			}
		} else {
			Factory::getApplication()->triggerEvent('onrsepro_paymentForm', array(array('id' => $id, 'total' => &$total, 'info' => &$info)));
		}
		
		$ticketstotal		= !empty($total) ? rseventsproHelper::currency($total) : '';
		$ticketsdiscount	= !empty($subscription->discount)	? rseventsproHelper::currency($subscription->discount) : '';
		$subscriptionTax	= !empty($subscription->tax)		? rseventsproHelper::currency($subscription->tax) : '';
		$lateFee			= !empty($subscription->late_fee)	? rseventsproHelper::currency($subscription->late_fee) : '';
		$earlyDiscount		= !empty($subscription->early_fee)	? rseventsproHelper::currency($subscription->early_fee) : '';
		$gateway			= rseventsproHelper::getPayment($subscription->gateway, $subscription->lang);
		$IP					= $subscription->ip;
		$coupon				= $subscription->coupon;
		
		return array(implode('<br />', $info), $ticketstotal, $ticketsdiscount, $subscriptionTax, $lateFee, $earlyDiscount, $gateway, $IP, $coupon);
	}
	
	protected static function getSeat($ids, $idt, $pos) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('seat'))
			->from($db->qn('#__rseventspro_user_seats'))
			->where($db->qn('idt').' = '.(int) $idt)
			->where($db->qn('ids').' = '.(int) $ids);
		
		$db->setQuery($query);
		if ($seats = $db->loadColumn()) {
			$pos = $pos - 1;
			return isset($seats[$pos]) ? $seats[$pos] : ''; 
		}
		
		return '';
	}
	
	protected static function getDisableOption($id, $type) {
		$db			= Factory::getDbo();
		$query		= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('disable_'.$type))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		return (int) $db->loadResult();
	}
	
	public static function sendMail($from, $fromName, $to, $subject, $body, $mode, $cc = null, $bcc = null, $attachments = null, $replyto = null, $replyname = null) {
		$mailer	= Factory::getMailer();
		
		if ($mode) {
			$textOnly = preg_replace('#<script(.*?)</script>#is','',$body);
			$textOnly = $textOnly ? strip_tags($textOnly) : strip_tags($body);
			$mailer->AltBody = $textOnly;
		}
		
		return $mailer->sendMail($from, $fromName, $to, $subject, $body, $mode, $cc, $bcc, $attachments, $replyto, $replyname);
	}
}