<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;

class RSEventsProInvoice {

    protected $cart = false;
    protected $order_id;
    protected $lang;
    protected $document;

    public function __construct($order_id = null, $lang = null) {
		if (empty($order_id)) {
            throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_INVOICE_NO_ORDER_ID'), 500);
        }

		$this->order_id = $order_id;
		$this->lang = $lang;
		
		// Check if the order id is valid and the payment is complete
		if (!$this->isValidOrderId()) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_INVOICE_INVALID_OR_NOT_PAYED_ORDER_ID'), 500);
		}
		
		// Check for PDF plugin
		if (!rseventsproHelper::pdf('1.18')) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_INVOICE_PDF_PLUGIN'), 500);
		}
		
		Factory::getApplication()->triggerEvent('onrsepro_isCart',array(array('cart'=>&$this->cart)));
    }

    public static function getInstance($order_id = null, $lang = null) {
        static $instances = array();

        if (!isset($instances[$order_id])) {
            $instances[$order_id] = new RSEventsProInvoice($order_id, $lang);
        }

        return $instances[$order_id];
    }
	
	public function output($downloadable = true) {
		$invoice = $this->getTemplate();
		
		if (empty($invoice)) {
			throw new Exception(Text::_('COM_RSEVENTSPRO_ERROR_INVOICE_NO_TEMPLATE'), 500);
		}
		
		require_once JPATH_SITE.'/components/com_rseventspro/helpers/pdf.php';
		
		$options = array('font' => $this->document->font, 'orientation' => $this->document->orientation);
		$pdf	 = RSEventsProPDF::getInstance($options);
		
		if ($downloadable) {
			$pdf->output($invoice, $this->document->title.'.pdf');
			die;
		} else {
			return array('buffer' => $pdf->write($invoice), 'title' => $this->document->title.'.pdf');
		}
	}
	
	protected function isValidOrderId() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('state'))
			->from($db->qn('#__rseventspro_users'))
			->where($db->qn('id').' = '.$db->q($this->order_id));
		$db->setQuery($query);
		return $db->loadResult() == 1;
	}
	
	protected function getData() {
		static $data = array();
		
		if (!isset($data[$this->order_id])) {
			$db		= Factory::getDbo();
			$query	= $db->getQuery(true);
			
			$query->clear()
				->select('*')
				->from($db->qn('#__rseventspro_users'))
				->where($db->qn('id').' = '.$db->q($this->order_id));
			$db->setQuery($query);
			$data[$this->order_id] = $db->loadObject();
		}
		
		return $data[$this->order_id];
	}
	
	protected function getEventInvoiceDetails($id, $lang = 'en-GB') {
		static $eventInvoice = array();
		
		if (!isset($eventInvoice[$id])) {
			$db		 = Factory::getDbo();
			$query	 = $db->getQuery(true);
			$invoice = $this->getInvoice($id, $lang);
			
			$query->clear()
				->select($db->qn('invoice'))->select($db->qn('invoice_attach'))->select($db->qn('invoice_type'))->select($db->qn('invoice_font'))
				->select($db->qn('invoice_orientation'))->select($db->qn('invoice_padding'))->select($db->qn('invoice_prefix'))
				->from($db->qn('#__rseventspro_events'))
				->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$eInvoice = $db->loadObject();
			$eInvoice->invoice_title = $invoice->title;
			$eInvoice->invoice_layout = $invoice->layout; 
			
			$eventInvoice[$id] = $eInvoice;
		}
		
		return $eventInvoice[$id];
	}
	
	protected function getGlobalInvoiceDetails($lang = 'en-GB') {
		$config		= rseventsproHelper::getConfig();
		$invoice	= $this->getInvoice(0, $lang);
		
		return (object) array('invoice_font' => $config->invoice_font, 'invoice_orientation' => $config->invoice_orientation, 'invoice_padding' => $config->invoice_padding, 'invoice_prefix' => $config->invoice_prefix, 'invoice_title' => $invoice->title, 'invoice_layout' => $invoice->layout);
	}
	
	protected function getInvoiceDetails($id, $lang = 'en-GB') {
		if (empty($id)) {
			$events = $this->getCartEvents();
			
			if (count($events) == 1) {
				if (isset($events[0])) {
					$object = $this->getEventInvoiceDetails($events[0], $lang)->invoice_type == 1 ? $this->getGlobalInvoiceDetails($lang) : $this->getEventInvoiceDetails($events[0], $lang);
				} else {
					$object = $this->getGlobalInvoiceDetails($lang);
				}
				
			} else {
				$object = $this->getGlobalInvoiceDetails($lang);
			}
		} else {
			$object = $this->getEventInvoiceDetails($id, $lang)->invoice_type == 1 ? $this->getGlobalInvoiceDetails($lang) : $this->getEventInvoiceDetails($id, $lang);
		}
		
		return $object;
	}
	
	protected function getTemplate() {
		$data		= $this->getData();
		$lang		= $this->lang ? $this->lang : $data->lang;
		$layout		= $this->getInvoiceDetails($data->ide, $lang)->invoice_layout;
		$prefix		= $this->getInvoiceDetails($data->ide, $lang)->invoice_prefix;
		$prefix		= trim($prefix) ? $prefix : '';
		$padding	= $this->getInvoiceDetails($data->ide, $lang)->invoice_padding;
		$title		= $this->getInvoiceDetails($data->ide, $lang)->invoice_title;
		$optionals	= array('{discount}' => '', '{tax}' => '', '{late_fee}' => '', '{early_fee}' => '');
		
		$placeholders = array(
			'{invoice_id}'		 => $prefix.str_pad($this->order_id, $padding, 0, STR_PAD_LEFT),
			'{email}' 			 => $data->email,
			'{name}' 			 => $data->name,
			'{username}' 		 => $data->idu ? Factory::getUser($data->idu)->get('username') : '',
			'{payment}' 		 => $data->gateway ? rseventsproHelper::getPayment($data->gateway, $lang) : '-',
			'{date}'	 		 => rseventsproHelper::showdate($data->date),
			'{invoice_table}'	 => $this->createInvoiceTable($optionals, $lang),
			'{site_name}' 	     => Factory::getConfig()->get('sitename'),
			'{site_url}' 	     => Uri::root(),
			'{sitepath}'         => JPATH_SITE
		);
		
		$replace	= array_keys($placeholders);
		$with		= array_values($placeholders);
		$layout		= str_replace($replace, $with, $layout);
		$layout		= str_replace(array_keys($optionals), array_values($optionals), $layout);
		$title		= str_replace($replace, $with, $title);
		
		$this->document = new stdClass();
		$this->document->title = $title;
		$this->document->font = $this->getInvoiceDetails($data->ide, $lang)->invoice_font;
		$this->document->orientation = $this->getInvoiceDetails($data->ide, $lang)->invoice_orientation;
		
		return $layout;
	}
	
	protected function createInvoiceTable(&$optionals, $lang) {
		$total	 = 0;
		$data	 = $this->getData();
		$table	 = '';
		
		if ($folder = Uri::root(true))      {
			$site_path = substr(str_replace(DIRECTORY_SEPARATOR, '/', JPATH_SITE), 0, -strlen($folder));
		} else {
			$site_path = str_replace(DIRECTORY_SEPARATOR, '/', JPATH_SITE);
		}
		
		if ($css_path = realpath($site_path.'/' . HTMLHelper::stylesheet('com_rseventspro/invoice.css', array('pathOnly' => true, 'relative' => true)))) {
			$table .= '<link rel="stylesheet" href="'.$this->escape($css_path).'" type="text/css"/>';
		}
		
		$table .= '<table width="100%" cellpadding="3" cellspacing="1" class="invoice_table">
                <thead>
                    <tr>
                        <th class="package">'.Text::_('COM_RSEVENTSPRO_INVOICE_ITEM').'</th>
                        <th class="price">'.Text::_('COM_RSEVENTSPRO_INVOICE_PRICE').'</th>
                        <th class="quantity">'.Text::_('COM_RSEVENTSPRO_INVOICE_QTY').'</th>
                        <th class="total">'.Text::_('COM_RSEVENTSPRO_INVOICE_TOTAL').'</th>
                    </tr>
                </thead>
                <tbody>';
		
		if ($data->ide) {
			$tickets = $this->getTickets($lang);
			
			$table .= '<tr><td colspan="4">'.Text::_('COM_RSEVENTSPRO_INVOICE_EVENT').' '.$this->getEventName($data->ide, $lang).'</td></tr>';
			foreach ($tickets as $ticket) {
				$total += $ticket->quantity * $ticket->price;
				
				$table .= '<tr>';
				$table .= '<td>'.$this->escape($ticket->id ? $ticket->name : Text::_('COM_RSEVENTSPRO_FREE_ENTRANCE')).'</td>';
				$table .= '<td>'.$this->escape(rseventsproHelper::currency($ticket->price)).'</td>';
				$table .= '<td>'.$this->escape($ticket->quantity).'</td>';
				$table .= '<td>'.$this->escape(rseventsproHelper::currency($ticket->quantity * $ticket->price)).'</td>';
				$table .= '</tr>';
			}
			
			$total = $total - $data->discount;
		} else {
			if ($this->cart) {
				$cart = $this->getCart();
				$events = $cart->getEvents();
				
				foreach ($events as $event) {
					$table .= '<tr><td colspan="4">'.Text::_('COM_RSEVENTSPRO_INVOICE_EVENT').' '.$this->getEventName($event, $lang).'</td></tr>';
					
					$tickets = $this->getCartTickets($event, $lang);
					
					if ($tickets) {
						foreach ($tickets as $ticket) {
							$table .= '<tr>';
							$table .= '<td>'.$this->escape($ticket->name).'</td>';
							$table .= '<td>'.$this->escape(rseventsproHelper::currency($ticket->total)).'</td>';
							$table .= '<td>'.$this->escape($ticket->quantity).'</td>';
							$table .= '<td>'.$this->escape(rseventsproHelper::currency($ticket->total * $ticket->quantity)).'</td>';
							$table .= '</tr>';
							
							if (isset($ticket->extra) && !empty($ticket->extra)) {
								foreach ($ticket->extra as $label => $price) {
									$table .= '<tr>';
									$table .= '<td>- '.$this->escape($label).'</td>';
									$table .= '<td>'.$this->escape(rseventsproHelper::currency($price)).'</td>';
									$table .= '<td>'.$this->escape(1).'</td>';
									$table .= '<td>'.$this->escape(rseventsproHelper::currency($price)).'</td>';
									$table .= '</tr>';
								}
							}
							
						}
					}
				}
				
				$total = $cart->total;
			}
		}
		
		if (!empty($data->early_fee)) {
			if ($data->ide) {
				$total = $total - $data->early_fee;
			}
			
			$optionals['{early_fee}'] = rseventsproHelper::currency($data->early_fee);
			
			$table .= '<tr>';
            $table .= '<td class="early_fee" colspan="3" align="right">'.Text::_('COM_RSEVENTSPRO_INVOICE_EARLY_FEE').'</td>';
			$table .= '<td class="early_fee">-'.$this->escape(rseventsproHelper::currency($data->early_fee)).'</td>';
			$table .= '</tr>';
		}
		
		if (!empty($data->late_fee)) {
			if ($data->ide) {
				$total = $total + $data->late_fee;
			}
			
			$optionals['{late_fee}'] = rseventsproHelper::currency($data->early_fee);
			
			$table .= '<tr>';
            $table .= '<td class="late_fee" colspan="3" align="right">'.Text::_('COM_RSEVENTSPRO_INVOICE_LATE_FEE').'</td>';
			$table .= '<td class="late_fee">'.$this->escape(rseventsproHelper::currency($data->late_fee)).'</td>';
			$table .= '</tr>';
		}
		
		if (!empty($data->tax)) {
			if ($data->ide) {
				$total = $total + $data->tax;
			}
			
			$optionals['{tax}'] = rseventsproHelper::currency($data->tax);
			
			$table .= '<tr>';
            $table .= '<td class="tax" colspan="3" align="right">'.Text::_('COM_RSEVENTSPRO_INVOICE_TAX').'</td>';
			$table .= '<td class="tax">'.$this->escape(rseventsproHelper::currency($data->tax)).'</td>';
			$table .= '</tr>';
		}
		
		if (!empty($data->discount)) {
			
			$optionals['{discount}'] = rseventsproHelper::currency($data->discount);
			
			$table .= '<tr>';
            $table .= '<td class="discount" colspan="3" align="right">'.Text::_('COM_RSEVENTSPRO_INVOICE_DISCOUNT').'</td>';
			$table .= '<td class="discount">-'.$this->escape(rseventsproHelper::currency($data->discount)).'</td>';
			$table .= '</tr>';
		}
		
		$table .= '<tr>
                        <td class="grand total" colspan="3" align="right">'.Text::_('COM_RSEVENTSPRO_INVOICE_GRAND_TOTAL').'</td>
                        <td class="grand total">'.$this->escape(rseventsproHelper::currency($total)).'</td>
                    </tr>
                </tbody>
            </table>';
		
		return $table;
	}
	
	protected function getTickets($lang) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('ut.quantity'))->select($db->qn('t').'.*')
			->from($db->qn('#__rseventspro_user_tickets','ut'))
			->join('left',$db->qn('#__rseventspro_tickets','t').' ON '.$db->qn('t.id').' = '.$db->qn('ut.idt'))
			->where($db->qn('ut.ids').' = '.$db->q($this->order_id));
		
		$db->setQuery($query);
		if ($tickets = $db->loadObjectList()) {
			foreach ($tickets as $ticket) {
				// Replace translations
				if ($translations = RSEventsProTranslations::getTranslations('ticket', $ticket->id, $lang)) {
					foreach ($translations as $field => $translation) {
						if (isset($ticket->$field)) {
							$ticket->$field = $translation;
						}
					}
				}
			}
		}
		
		return $tickets;
	}
	
	protected function getCart() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->select($db->qn('hash'))
			->from($db->qn('#__rseventspro_cart'))
			->where($db->qn('ids').' = '.$db->q($this->order_id));
		
		$db->setQuery($query);
		if ($hash = $db->loadResult()) {
			require_once JPATH_SITE.'/components/com_rseventspro/helpers/cart.php';
			return RSEventsProCart::getInstance($hash);
		}
		
		return false;
	}
	
	protected function getCartTickets($event, $lang) {
		$cart	= $this->getCart();
		$return = array();
		
		if ($tickets = $cart->getTickets($event)) {
			foreach ($tickets as $ticket) {
				$extraCost 	= array_sum((array) $ticket->prices);
				$hash		= md5($ticket->ticketID.$extraCost);
				
				if (empty($return[$hash])) {
					$ticketInfo = new stdClass();
				
					$info					= $cart->getTicket($ticket->ticketID, $lang);
					$labels					= $cart->getFieldLabels($ticket->ticketID, $lang);
					$ticketInfo->name 		= $ticket->ticketID ? $info->name : Text::_('COM_RSEVENTSPRO_FREE_ENTRANCE');
					$ticketInfo->total 		= $ticket->ticketID ? $info->price : 0;
					$ticketInfo->quantity 	= 1;
					$ticketInfo->extraCost 	= $extraCost;
					
					if ($extraCost) {
						$ticketInfo->extra = array();
						
						foreach ($ticket->prices as $name => $price) {
							if (empty($price)) continue;
							$label = isset($labels[$name]) ? $labels[$name] : $name;
							$ticketInfo->extra[$label] = $price;
						}
					}
					
					$return[$hash] = $ticketInfo;
				} else {
					$return[$hash]->quantity++;
				}
			}
		}
		
		return $return;
	}
	
	protected function getCartEvents() {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('events'))
			->from($db->qn('#__rseventspro_cart'))
			->where($db->qn('ids').' = '.$db->q($this->order_id));
		
		$db->setQuery($query);
		$events = $db->loadResult();
		
		return explode(',', $events);
	}
	
	protected function getEventName($id, $lang = 'en-GB') {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('name'))
			->from($db->qn('#__rseventspro_events'))
			->where($db->qn('id').' = '.$db->q($id));
		
		$db->setQuery($query);
		if ($event = $db->loadResult()) {
			// Replace translations
			if ($translation = RSEventsProTranslations::getTranslation('event', $id, 'name', $lang)) {
				$event = $translation;
			}
		}
		
		return $event;
	}
	
	protected function getInvoice($id, $lang) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('title'))->select($db->qn('layout'))
			->from($db->qn('#__rseventspro_invoice'))
			->where($db->qn('ide').' = '.$db->q($id))
			->where($db->qn('lang').' = '.$db->q($lang));
		$db->setQuery($query);
		$invoice = $db->loadObject();
		
		if (!$invoice) {
			$query->clear()
				->select($db->qn('title'))->select($db->qn('layout'))
				->from($db->qn('#__rseventspro_invoice'))
				->where($db->qn('ide').' = '.$db->q($id))
				->where($db->qn('lang').' = '.$db->q('en-GB'));
			$db->setQuery($query);
			$invoice = $db->loadObject();
			
			if (!$invoice) {
				$invoice = new stdClass();
				$invoice->title = '';
				$invoice->layout = '';
			}
		}
		
		return $invoice;
	}
	
	protected function escape($string) {
		return htmlentities($string, ENT_COMPAT, 'UTF-8');
	}
}