<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

$count = count($this->events);
$rss = $this->params->get('rss',1);
$ical = $this->params->get('ical',1); ?>

<div class="rsepro-events-list-container">

    <h1><?php echo Text::sprintf('COM_RSEVENTSPRO_EVENTS_FROM',$this->date); ?></h1>

    <div class="clearfix">
        <a href="<?php echo $this->prev; ?>" class="<?php echo RSEventsproAdapterGrid::styles(array('btn', 'pull-left')); ?>"><i class="fa fa-chevron-left"></i> <?php echo Text::_('COM_RSEVENTSPRO_PREVIOUS_DAY'); ?></a>
        <a href="<?php echo $this->next; ?>" class="<?php echo RSEventsproAdapterGrid::styles(array('btn', 'pull-right')); ?>"><?php echo Text::_('COM_RSEVENTSPRO_NEXT_DAY'); ?> <i class="fa fa-chevron-right"></i></a>
    </div>

    <?php if ($rss || $ical || $this->config->timezone) { ?>
    <div class="rs_rss">
        <?php if ($this->config->timezone) { ?>
        <a rel="rs_timezone" <?php if (rseventsproHelper::getConfig('modaltype','int') == 1) echo ' href="#timezoneModal" data-toggle="modal" data-bs-toggle="modal"'; else echo ' href="javascript:void(0)"'; ?> class="<?php echo rseventsproHelper::tooltipClass(); ?> rsepro-timezone" title="<?php echo rseventsproHelper::tooltipText(Text::_('COM_RSEVENTSPRO_CHANGE_TIMEZONE')); ?>">
            <i class="fa fa-clock"></i>
        </a>
        <?php } ?>

        <?php if ($rss) { ?>
        <a href="<?php echo $this->rss; ?>" class="<?php echo rseventsproHelper::tooltipClass(); ?> rsepro-rss" title="<?php echo rseventsproHelper::tooltipText(Text::_('COM_RSEVENTSPRO_RSS')); ?>">
            <i class="fa fa-rss-square"></i>
        </a>
        <?php } ?>
        <?php if ($ical) { ?>
        <a href="<?php echo $this->ical; ?>" class="<?php echo rseventsproHelper::tooltipClass(); ?> rsepro-ical" title="<?php echo rseventsproHelper::tooltipText(Text::_('COM_RSEVENTSPRO_ICS')); ?>">
            <i class="fa fa-calendar"></i>
        </a>
        <?php } ?>
    </div>
    <?php } ?>

    <?php if (!empty($this->events)) { ?>
    <ul class="<?php echo rseventsproHelper::layout('container'); ?>" id="rs_events_container">

        <?php $eventIds = rseventsproHelper::getEventIds($this->events, 'id'); ?>
        <?php $this->events = rseventsproHelper::details($eventIds); ?>
        <?php foreach($this->events as $details) { ?>
        <?php if (isset($details['event']) && !empty($details['event'])) $event = $details['event']; else continue; ?>
        <?php $full = rseventsproHelper::eventisfull($event->id); ?>
        <?php $ongoing = rseventsproHelper::ongoing($event->id); ?>
        <?php $categories = (isset($details['categories']) && !empty($details['categories'])) ? Text::_('COM_RSEVENTSPRO_GLOBAL_CATEGORIES').': '.$details['categories'] : '';  ?>
        <?php $tags = (isset($details['tags']) && !empty($details['tags'])) ? Text::_('COM_RSEVENTSPRO_GLOBAL_TAGS').': '.$details['tags'] : '';  ?>
        <?php $incomplete = !$event->completed ? ' rs_incomplete' : ''; ?>
        <?php $featured = $event->featured ? ' rs_featured' : ''; ?>
        <?php $canceled = $event->published == 3 ? ' rsepro_canceled_event_block' : ''; ?>
        <?php $canEdit = (!empty($this->permissions['can_edit_events']) || $event->owner == $this->user || $event->sid == $this->user || $this->admin) && !empty($this->user); ?>
        <?php $canDelete = (!empty($this->permissions['can_delete_events']) || $event->owner == $this->user || $event->sid == $this->user || $this->admin) && !empty($this->user); ?>

        <li class="<?php echo rseventsproHelper::layout('item-container'); ?><?php echo $incomplete.$featured.$canceled; ?>" id="rs_event<?php echo $event->id; ?>" itemscope itemtype="http://schema.org/Event">

            <?php if ($canEdit || $canDelete) { ?>
            <div class="rsepro-event-options <?php echo rseventsproHelper::layout('options'); ?>" style="display:none;">
                <?php if ($canEdit) { ?>
                    <a href="<?php echo rseventsproHelper::route('index.php?option=com_rseventspro&layout=edit&id='.rseventsproHelper::sef($event->id,$event->name)); ?>">
                        <i class="fa fa-edit"></i>
                    </a>
                <?php } ?>

                <?php if ($canDelete) { ?>
                    <a href="<?php echo rseventsproHelper::route('index.php?option=com_rseventspro&task=rseventspro.remove&id='.rseventsproHelper::sef($event->id,$event->name)); ?>" onclick="return confirm('<?php echo Text::_('COM_RSEVENTSPRO_GLOBAL_DELETE_CONFIRMATION'); ?>');">
                        <i class="fa fa-trash"></i>
                    </a>
                <?php } ?>
            </div>
            <?php } ?>

            <?php if (!empty($event->options['show_icon_list'])) { ?>
            <div class="<?php echo rseventsproHelper::layout('image-container'); ?>" itemprop="image">
                <a href="<?php echo rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)); ?>" class="rs_event_link thumbnail">
                    <img src="<?php echo rseventsproHelper::thumb($event->id, rseventsproHelper::layout('image-width')); ?>" alt="" width="<?php echo rseventsproHelper::layout('image-width'); ?>" />
                </a>
            </div>
            <?php } ?>

            <div class="<?php echo rseventsproHelper::layout('event-details-container'); ?>">
                <div itemprop="name" class="<?php echo rseventsproHelper::layout('event-title'); ?>">
                    <a itemprop="url" href="<?php echo rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($event->id,$event->name),false,rseventsproHelper::itemid($event->id)); ?>" class="rs_event_link<?php echo $full ? ' rs_event_full' : ''; ?><?php echo $ongoing ? ' rs_event_ongoing' : ''; ?>"><?php echo $event->name; ?></a> <?php if (!$event->completed) echo Text::_('COM_RSEVENTSPRO_GLOBAL_INCOMPLETE_EVENT'); ?> <?php if (!$event->published) echo Text::_('COM_RSEVENTSPRO_GLOBAL_UNPUBLISHED_EVENT'); ?> <?php if ($event->published == 3) echo '<small class="text-error">('.Text::_('COM_RSEVENTSPRO_EVENT_CANCELED_TEXT').')</small>'; ?>
                </div>
                <div class="<?php echo rseventsproHelper::layout('event-date'); ?>">
                    <?php if ($event->allday) { ?>
                    <?php if (!empty($event->options['start_date_list'])) { ?>
                    <span class="rsepro-event-on-block">
                    <?php echo Text::_('COM_RSEVENTSPRO_GLOBAL_ON'); ?> <b><?php echo rseventsproHelper::showdate($event->start,$this->config->global_date,true); ?></b>
                    </span>
                    <?php } ?>
                    <?php } else { ?>

                    <?php if (!empty($event->options['start_date_list']) || !empty($event->options['start_time_list']) || !empty($event->options['end_date_list']) || !empty($event->options['end_time_list'])) { ?>
                    <?php if (!empty($event->options['start_date_list']) || !empty($event->options['start_time_list'])) { ?>
                    <?php if ((!empty($event->options['start_date_list']) || !empty($event->options['start_time_list'])) && empty($event->options['end_date_list']) && empty($event->options['end_time_list'])) { ?>
                    <span class="rsepro-event-starting-block">
                    <?php echo Text::_('COM_RSEVENTSPRO_EVENT_STARTING_ON'); ?>
                    <?php } else { ?>
                    <span class="rsepro-event-from-block">
                    <?php echo Text::_('COM_RSEVENTSPRO_EVENT_FROM'); ?>
                    <?php } ?>
                    <b><?php echo rseventsproHelper::showdate($event->start,rseventsproHelper::showMask('list_start',$event->options),true); ?></b>
                    </span>
                    <?php } ?>
                    <?php if (!empty($event->options['end_date_list']) || !empty($event->options['end_time_list'])) { ?>
                    <?php if ((!empty($event->options['end_date_list']) || !empty($event->options['end_time_list'])) && empty($event->options['start_date_list']) && empty($event->options['start_time_list'])) { ?>
                    <span class="rsepro-event-ending-block">
                    <?php echo Text::_('COM_RSEVENTSPRO_EVENT_ENDING_ON'); ?>
                    <?php } else { ?>
                    <span class="rsepro-event-until-block">
                    <?php echo Text::_('COM_RSEVENTSPRO_EVENT_UNTIL'); ?>
                    <?php } ?>
                    <b><?php echo rseventsproHelper::showdate($event->end,rseventsproHelper::showMask('list_end',$event->options),true); ?></b>
                    </span>
                    <?php } ?>
                    <?php } ?>

                    <?php } ?>
                </div>

                <?php if (!empty($event->options['show_location_list']) || !empty($event->options['show_categories_list']) || !empty($event->options['show_tags_list'])) { ?>
                <div class="<?php echo rseventsproHelper::layout('event-taxonomy'); ?>">
                    <?php if ($event->locationid && $event->lpublished && !empty($event->options['show_location_list'])) { ?>
                    <span class="rsepro-event-location-block" itemprop="location" itemscope itemtype="http://schema.org/Place"><?php echo Text::_('COM_RSEVENTSPRO_GLOBAL_AT'); ?> <a itemprop="url" href="<?php echo rseventsproHelper::route('index.php?option=com_rseventspro&layout=location&id='.rseventsproHelper::sef($event->locationid,$event->location)); ?>"><span itemprop="name"><?php echo $event->location; ?></span></a>
                    <span itemprop="address" style="display:none;"><?php echo $event->address; ?></span>
                    </span>
                    <?php } ?>
                    <?php if (!empty($event->options['show_categories_list'])) { ?>
                    <span class="rsepro-event-categories-block"><?php echo $categories; ?></span>
                    <?php } ?>
                    <?php if (!empty($event->options['show_tags_list'])) { ?>
                    <span class="rsepro-event-tags-block"><?php echo $tags; ?></span>
                    <?php } ?>
                </div>
                <?php } ?>

                <?php if (!empty($event->small_description)) { ?>
                <div class="<?php echo rseventsproHelper::layout('event-description'); ?>">
                    <?php echo $event->small_description; ?>
                </div>
                <?php } ?>
            </div>

            <meta content="<?php echo rseventsproHelper::showdate($event->start,'Y-m-d H:i:s'); ?>" itemprop="startDate" />
            <?php if (!$event->allday) { ?><meta content="<?php echo rseventsproHelper::showdate($event->end,'Y-m-d H:i:s'); ?>" itemprop="endDate" /><?php } ?>
        </li>
        <?php } ?>
    </ul>

    <div class="rs_loader" id="rs_loader" style="display:none;">
        <?php echo HTMLHelper::image('com_rseventspro/loader.gif', '', array(), true); ?>
    </div>

    <?php if ($this->total > $count) { ?>
    <p id="rsepro_number_events"><?php echo Text::sprintf('COM_RSEVENTSPRO_SHOWING_EVENTS','<span>'.$count.'</span>',$this->total); ?></p>
    <a class="rs_read_more" id="rsepro_loadmore"><?php echo Text::_('COM_RSEVENTSPRO_GLOBAL_LOAD_MORE'); ?></a>
    <?php } ?>

    <span id="total" class="rs_hidden"><?php echo $this->total; ?></span>
    <span id="Itemid" class="rs_hidden"><?php echo Factory::getApplication()->input->getInt('Itemid'); ?></span>
    <span id="langcode" class="rs_hidden"><?php echo rseventsproHelper::getLanguageCode(); ?></span>
    <span id="date" class="rs_hidden"><?php echo Factory::getApplication()->input->getString('date'); ?></span>
    <?php } else { ?>
    <div class="alert alert-warning"><?php echo Text::_('COM_RSEVENTSPRO_GLOBAL_NO_EVENTS'); ?></div>
    <?php } ?>

    <?php if ($this->config->timezone) echo rseventsproHelper::timezoneModal(); ?>
</div>

<script type="text/javascript">
	jQuery(document).ready(function() {
		<?php if ($this->total > $count) { ?>
		jQuery('#rsepro_loadmore').on('click', function() {
			rspagination('day', jQuery('#rs_events_container > li').length);
		});
		<?php } ?>
		<?php if (!empty($count)) { ?>
		jQuery('#rs_events_container li').on({
			mouseenter: function() {
				jQuery(this).find('div.rsepro-event-options').css('display','');
			},
			mouseleave: function() {
				jQuery(this).find('div.rsepro-event-options').css('display','none');
			}
		});
		<?php } ?>
	});
</script>