<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\Document\Feed\FeedItem;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

class RseventsproViewCalendar extends HtmlView
{	
	//Creates the Event Feed
	public function display($tpl = null) {
		$doc		= Factory::getDocument();
		$params		= rseventsproHelper::getParams();
		
		// Get events
		$rows = $this->get('Events');
		
		// If the option to not show full events is enabled , then remove them from our events list
		if (!$params->get('full',1)) {
			foreach ($rows as $i => $event) {
				if (rseventsproHelper::eventisfull($event->id)) {
					unset($rows[$i]);
				}
			}
		}
		
		foreach ($rows as $row ) {
			
			if (!rseventsproHelper::canview($row->id)) 
				continue;
			
			// Get event details
			$event = $this->getEvent($row->id);
			
			// Strip html from feed item title
			$title = $this->escape($event->name);
			$title = html_entity_decode($title, ENT_COMPAT, 'UTF-8');		

			// Url link to event
			$link = rseventsproHelper::route('index.php?option=com_rseventspro&layout=show&id='.rseventsproHelper::sef($row->id,$event->name),false,rseventsproHelper::itemid($row->id));
			
			// feed item description text
			$description = Text::_('COM_RSEVENTSPRO_FEED_LOCATION').': <strong>'.$this->escape($event->locationname).'</strong> - '.$this->escape($event->address).'<br />';
			$description .= Text::_('COM_RSEVENTSPRO_FEED_DATE').': '.rseventsproHelper::showdate($event->start);
			if (!$event->allday) $description .= ' - '.rseventsproHelper::showdate($event->end);
			$description .= '<br />';
			
			if (!empty($event->description)) $description .= $event->description;

			// load individual item creator class
			$item = new FeedItem();
			$item->title 		= $title;
			$item->link 		= $link;
			$item->description 	= $description;
			$item->date			= Factory::getDate($event->start)->format('r');
			
			// loads item info into rss array
			$doc->addItem( $item );
		}
	}
	
	protected function getEvent($id) {
		$db		= Factory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear()
			->select($db->qn('e.name'))->select($db->qn('e.start'))->select($db->qn('e.end'))->select($db->qn('e.allday'))
			->select($db->qn('e.description'))->select($db->qn('l.id','lid'))
			->select($db->qn('l.name','locationname'))->select($db->qn('l.address'))
			->from($db->qn('#__rseventspro_events','e'))
			->join('left', $db->qn('#__rseventspro_locations','l').' ON '.$db->qn('e.location').' = '.$db->qn('l.id'))
			->where($db->qn('e.id').' = '.(int) $id);
		
		$db->setQuery($query);
		if ($event = $db->loadObject()) {
			// Replace translations
			if ($translations = RSEventsProTranslations::getTranslations('event', $id)) {
				foreach ($translations as $field => $translation) {
					if (isset($event->$field)) {
						$event->$field = $translation;
					}
				}
			}
			
			if ($translations = RSEventsProTranslations::getTranslations('location', $event->lid)) {
				foreach ($translations as $field => $translation) {
					if (isset($event->locationname) && $field == 'name') {
						$event->locationname = $translation;
					}
				}
			}
		}
		
		return $event;
	}
}