<?php
/**
* @package RSEvents!Pro
* @copyright (C) 2020 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;

class RseventsproViewSubscriptions extends HtmlView
{
	public function display($tpl = null) {
		$doc		= Factory::getDocument();
		$app		= Factory::getApplication();
		$pathway	= $app->getPathWay();
		$menus		= $app->getMenu();
		$menu		= $menus->getActive();
		$jconfig	= Factory::getConfig();
		$layout		= $this->getLayout();
		$title		= null;
		
		// Get menu parameters , user permission etc.
		$this->admin		= rseventsproHelper::admin();
		$this->params		= rseventsproHelper::getParams();
		$this->permissions	= rseventsproHelper::permissions();
		$this->config		= rseventsproHelper::getConfig();
		
		if (isset($this->permissions) && empty($this->permissions['can_view_subscriptions'])) {
			rseventsproHelper::error(Text::_('COM_RSEVENTSPRO_ERROR_VIEW_SUBSCRIPTIONS_PERMISSION'), rseventsproHelper::route('index.php?option=com_rseventspro&layout=default',false, RseventsproHelperRoute::getEventsItemid('999999')));
		}
		
		$theview = isset($menu->query['view']) ? $menu->query['view'] : 'rseventspro';
		if ($theview != 'subscriptions') {
			$pathway->addItem(Text::_('COM_RSEVENTSPRO_BC_ALL_SUBSCRIPTIONS'));
		}
		
		// Add Joomla! menu metadata
		if ($menu && isset($menu->title)) {
			$title = $menu->title;
		}
		
		if ($this->params->get('page_title')) {
			$title = $this->params->get('page_title');
		}
		
		if ($title) {
			if ($jconfig->get('sitename_pagetitles', 0) == 1) {
				$title = Text::sprintf('JPAGETITLE', $jconfig->get('sitename'), $title);
			} elseif ($jconfig->get('sitename_pagetitles', 0) == 2) {
				$title = Text::sprintf('JPAGETITLE', $title, $jconfig->get('sitename'));
			}
			
			$doc->setTitle($title);
		}
		
		if ($this->params->get('menu-meta_description'))
			$doc->setDescription($this->params->get('menu-meta_description'));

		if ($this->params->get('menu-meta_keywords'))
			$doc->setMetadata('keywords', $this->params->get('menu-meta_keywords'));

		if ($this->params->get('robots'))
			$doc->setMetadata('robots', $this->params->get('robots'));
		
		// Get subscriptions
		$this->items	= $this->get('Items');
		$this->pagination 	 = $this->get('Pagination');
		$this->state 		 = $this->get('State');
		$this->filterForm    = $this->get('FilterForm');
		$this->activeFilters = $this->get('ActiveFilters');
		
		parent::display($tpl);
	}
	
	public function getStatus($state) {
		if ($state == 0) {
			return '<font color="blue">'.Text::_('COM_RSEVENTSPRO_GLOBAL_STATUS_INCOMPLETE').'</font>';
		} elseif ($state == 1) {
			return '<font color="green">'.Text::_('COM_RSEVENTSPRO_GLOBAL_STATUS_COMPLETED').'</font>';
		} elseif ($state == 2) {
			return '<font color="red">'.Text::_('COM_RSEVENTSPRO_GLOBAL_STATUS_DENIED').'</font>';
		} elseif ($state == 3) {
			return '<font color="orange">'.Text::_('COM_RSEVENTSPRO_GLOBAL_STATUS_REFUNDED').'</font>';
		}
	}
	
	public function getEvents($item) {
		$names = array();
		
		if ($item->ide) {
			// Replace translations
			if ($translation = RSEventsProTranslations::getTranslation('event', $item->ide, 'name')) {
				$item->event = $translation;
			}
			
			$names[] = $item->event;
		} else {
			if (method_exists('plgSystemRseproCart', 'cartEvents')) {
				if ($events = plgSystemRseproCart::cartEvents($item->id)) {
					foreach ($events as $event) {
						// Replace translations
						if ($translation = RSEventsProTranslations::getTranslation('event', $event->id, 'name')) {
							$event->name = $translation;
						}
						
						$names[] = $event->name;
					}
				}
			}
		}
		
		return implode(', ', $names);
	}
}