<?php

/**
 * @package Helix3 Framework
 * @author JoomShaper http://www.joomshaper.com
 * @copyright Copyright (c) 2010 - 2015 JoomShaper
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;

defined('_JEXEC') or die;
$doc = Factory::getDocument();
$app = Factory::getApplication();
$params = Factory::getApplication()->getTemplate('true')->params;

//Load Helix
$helix3_path = JPATH_PLUGINS . '/system/helix3/core/helix3.php';

//Comingsoon Logo
if ($logo_image = $params->get('comingsoon_logo')) {
	$logo = Uri::base(true) . '/' . $logo_image;
} elseif ($logo_image = $params->get('logo_image')) {
	$logo = Uri::base(true) . '/' .  $logo_image;
} else {
	$logo = $this->baseurl . '/templates/' . $this->template . '/images/presets/preset1/logo@2x.png';
}

//Comingsoon BG
if ($cs_bg = $params->get('comingsoon_background')) {
	$cs_bg = Uri::base(true) . '/' . $cs_bg;

	$cs_bg_style = '.sp-comingsoon-wrap{'
		. 'background: url(' . $cs_bg . ');'
		. '}';
	$doc->addStyleDeclaration($cs_bg_style);
}


if (file_exists($helix3_path)) {
	require_once($helix3_path);
	$helix3 = Helix3::getInstance();
} else {
	die('Please install and activate helix plugin');
}

$comingsoon_title = $this->params->get('comingsoon_title');
if ($comingsoon_title) {
	$doc->setTitle($comingsoon_title . ' | ' . $app->get('sitename'));
}

$comingsoon_date = explode('-', $this->params->get("comingsoon_date"));

//Load jQuery
JHtml::_('jquery.framework');
?>
<!DOCTYPE html>
<html class="sp-comingsoon" xmlns="http://www.w3.org/1999/xhtml" xml:lang="<?php echo $this->language; ?>"
	lang="<?php echo $this->language; ?>" dir="<?php echo $this->direction; ?>">

<head>
	<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
	<meta name="viewport" content="width=device-width, initial-scale=1">
	<?php
	if ($favicon = $helix3->getParam('favicon')) {
		$doc->addFavicon(Uri::base(true) . '/' .  $favicon);
	} else {
		$doc->addFavicon($helix3->getTemplateUri() . '/images/favicon.ico');
	}
	?>
	<jdoc:include type="head" />
	<?php
	$helix3->addCSS('bootstrap.min.css, font-awesome.min.css')
		->lessInit()->setLessVariables(array(
			'preset' => $helix3->Preset(),
			'bg_color' => $helix3->PresetParam('_bg'),
			'text_color' => $helix3->PresetParam('_text'),
			'major_color' => $helix3->PresetParam('_major'),
			'hover_color' => $helix3->PresetParam('_hover'),
		))
		->addLess('master', 'template')
		->addLess('presets',  'presets/' . $helix3->Preset())
		->addJS('jquery.countdown.min.js');
	?>
</head>

<body>
	<div class="sp-comingsoon-wrap">
		<div class="container">
			<div class="text-center">
				<div id="sp-comingsoon">
					<div class="comingsoon-page-logo">
						<img class="img-resposive comingsoon-logo" alt="logo" src="<?php echo $logo; ?>" />
					</div>


					<div id="sp-comingsoon-countdown" class="sp-comingsoon-countdown">

					</div>


					<?php if ($comingsoon_title) { ?>
						<h1 class="sp-comingsoon-title">
							<?php echo $comingsoon_title; ?>
						</h1>
					<?php } ?>

					<?php if ($this->params->get('comingsoon_content')) { ?>
						<div class="sp-comingsoon-content">
							<?php echo $this->params->get('comingsoon_content'); ?>
						</div>
					<?php } ?>

					<?php if ($this->countModules('comingsoon')) { ?>
						<div class="sp-position-comingsoon">
							<jdoc:include type="modules" name="comingsoon" style="sp_xhtml" />
						</div>
					<?php } ?>

					<?php
					//Social Icons
					$facebook 	= $this->params->get('facebook');
					$twitter  	= $this->params->get('twitter');
					$pinterest 	= $this->params->get('pinterest');
					$youtube 	= $this->params->get('youtube');
					$linkedin 	= $this->params->get('linkedin');
					$dribbble 	= $this->params->get('dribbble');
					$behance 	= $this->params->get('behance');
					$skype 		= $this->params->get('skype');
					$flickr 	= $this->params->get('flickr');
					$vk 		= $this->params->get('vk');

					if ($this->params->get('show_social_icons') && ($facebook || $twitter || $pinterest || $youtube || $linkedin || $dribbble || $behance || $skype || $flickr || $vk)) {
						$html  = '<ul class="social-icons">';

						if ($facebook) {
							$html .= '<li><a target="_blank" href="' . $facebook . '"><i class="fa fa-facebook"></i></a></li>';
						}
						if ($twitter) {
							$html .= '<li><a target="_blank" href="' . $twitter . '"><i class="fa fa-twitter"></i></a></li>';
						}
						if ($pinterest) {
							$html .= '<li><a target="_blank" href="' . $pinterest . '"><i class="fa fa-pinterest"></i></a></li>';
						}
						if ($youtube) {
							$html .= '<li><a target="_blank" href="' . $youtube . '"><i class="fa fa-youtube"></i></a></li>';
						}
						if ($linkedin) {
							$html .= '<li><a target="_blank" href="' . $linkedin . '"><i class="fa fa-linkedin"></i></a></li>';
						}
						if ($dribbble) {
							$html .= '<li><a target="_blank" href="' . $dribbble . '"><i class="fa fa-dribbble"></i></a></li>';
						}
						if ($behance) {
							$html .= '<li><a target="_blank" href="' . $behance . '"><i class="fa fa-behance"></i></a></li>';
						}
						if ($flickr) {
							$html .= '<li><a target="_blank" href="' . $flickr . '"><i class="fa fa-flickr"></i></a></li>';
						}
						if ($vk) {
							$html .= '<li><a target="_blank" href="' . $vk . '"><i class="fa fa-vk"></i></a></li>';
						}
						if ($skype) {
							$html .= '<li><a href="skype:' . $skype . '?chat"><i class="fa fa-skype"></i></a></li>';
						}

						$html .= '</ul>';

						echo $html;
					}

					?>

				</div>
			</div>
		</div>
	</div>

	<script type="text/javascript">
		jQuery(function($) {
			$('#sp-comingsoon-countdown').countdown(
				'<?php echo trim($comingsoon_date[0]); ?>/<?php echo trim($comingsoon_date[1]); ?>/<?php echo trim($comingsoon_date[2]); ?>',
				function(event) {
					$(this).html(event.strftime(
						'<div class="days"><span class="number">%-D</span><span class="string">%!D:<?php echo Text::_("HELIX_DAY"); ?>,<?php echo Text::_("HELIX_DAYS"); ?>;</span></div><div class="hours"><span class="number">%H</span><span class="string">%!H:<?php echo Text::_("HELIX_HOUR"); ?>,<?php echo Text::_("HELIX_HOURS"); ?>;</span></div><div class="minutes"><span class="number">%M</span><span class="string">%!M:<?php echo Text::_("HELIX_MINUTE"); ?>,<?php echo Text::_("HELIX_MINUTES"); ?>;</span></div><div class="seconds"><span class="number">%S</span><span class="string">%!S:<?php echo Text::_("HELIX_SECOND"); ?>,<?php echo Text::_("HELIX_SECONDS"); ?>;</span></div>'
					));
				});
		});
	</script>

</body>

</html>