<?php

/**
 * @package     SP Simple Portfolio
 *
 * @copyright   Copyright (C) 2010 - 2015 JoomShaper. All rights reserved.
 * @license     GNU General Public License version 2 or later.
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

defined('_JEXEC') or die();

require_once JPATH_COMPONENT . '/helpers/helper.php';
jimport('joomla.filesystem.file');
$layout_type = $this->params->get('layout_type', 'default');

//Load Helix
$helix3_path = JPATH_PLUGINS . '/system/helix3/core/helix3.php';

if (file_exists($helix3_path)) {
    require_once($helix3_path);
    helix3::addCSS('magnific-popup.css') // CSS Files
        ->addJS('jquery.magnific-popup.min.js, spsimpleportfolio_main.js'); // JS Files
}


//Load the method jquery script.
JHtml::_('jquery.framework');

//Params
$params     = JComponentHelper::getParams('com_spsimpleportfolio');
$square     = strtolower($params->get('square', '600x600'));
$rectangle     = strtolower($params->get('rectangle', '600x400'));
$tower         = strtolower($params->get('tower', '600x800'));

//Add js and css files
$doc = Factory::getDocument();
$doc->addStylesheet(Uri::root(true) . '/components/com_spsimpleportfolio/assets/css/spsimpleportfolio.css');
$doc->addScript(Uri::root(true) . '/components/com_spsimpleportfolio/assets/js/jquery.shuffle.modernizr.min.js');
$doc->addScript(Uri::root(true) . '/components/com_spsimpleportfolio/assets/js/spsimpleportfolio.js');

$menu     = Factory::getApplication()->getMenu();
$itemId = '';
if (is_object($menu->getActive())) {
    $active = $menu->getActive();
    $itemId = '&Itemid=' . $active->id;
}

if ($this->params->get('show_page_heading') && $this->params->get('page_heading')) {
    echo "<h1 class='page-header'>" . $this->params->get('page_heading') . "</h1>";
}

$i = 0;
//Sizes
$sizes = array(
    $rectangle,
    $tower,
    $square,

    $tower,
    $rectangle,
    $square,

    $square,
    $rectangle,
    $tower,

    $square,
    $tower,
    $rectangle
);

?>

<div id="sp-simpleportfolio"
    class="sp-simpleportfolio sp-simpleportfolio-view-items layout-<?php echo str_replace('_', '-', $layout_type); ?>">


    <?php if ($this->params->get('show_filter', 1)) { ?>
        <div class="sp-simpleportfolio-filter">
            <ul>
                <li class="active" data-group="all"><a href="#"><?php echo Text::_('COM_SPSIMPLEPORTFOLIO_SHOW_ALL'); ?></a>
                </li>
                <?php
                $filters = SpsimpleportfolioHelper::getTagList($this->items);
                foreach ($filters as $filter) {
                ?>
                    <li data-group="<?php echo $filter->alias; ?>"><a href="#"><?php echo $filter->title; ?></a></li>
                <?php
                }
                ?>
            </ul>
        </div>
    <?php } ?>

    <?php
    //Videos
    foreach ($this->items as $key => $this->item) {

        if ($this->item->video) {
            $video = parse_url($this->item->video);

            switch ($video['host']) {
                case 'youtu.be':
                    $video_id     = trim($video['path'], '/');
                    $video_src     = '//www.youtube.com/embed/' . $video_id;
                    break;

                case 'www.youtube.com':
                case 'youtube.com':
                    parse_str($video['query'], $query);
                    $video_id     = $query['v'];
                    $video_src     = '//www.youtube.com/embed/' . $video_id;
                    break;

                case 'vimeo.com':
                case 'www.vimeo.com':
                    $video_id     = trim($video['path'], '/');
                    $video_src     = "//player.vimeo.com/video/" . $video_id;
            }

            echo '<iframe class="sp-simpleportfolio-lightbox" src="' . $video_src . '" width="500" height="281" id="sp-simpleportfolio-video' . $this->item->spsimpleportfolio_item_id . '" style="border:none;" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>';
        }
    }
    ?>

    <div class="sp-simpleportfolio-items sp-simpleportfolio-columns-<?php echo $this->params->get('columns', 3); ?>">
        <?php foreach ($this->items as $this->item) { ?>

            <?php
            $tags = SpsimpleportfolioHelper::getTags($this->item->spsimpleportfolio_tag_id);
            $newtags = array();
            $filter = '';
            $groups = array();
            foreach ($tags as $tag) {
                $newtags[]      = $tag->title;
                $filter     .= ' ' . $tag->alias;
                $groups[]     .= '"' . $tag->alias . '"';
            }

            $groups = implode(',', $groups);

            ?>

            <div class="sp-simpleportfolio-item" data-groups='[<?php echo $groups; ?>]'>
                <?php $this->item->url = Route::_('index.php?option=com_spsimpleportfolio&view=item&id=' . $this->item->spsimpleportfolio_item_id . ':' . $this->item->alias . $itemId); ?>

                <div class="sp-simpleportfolio-overlay-wrapper clearfix">

                    <?php if ($this->item->video) { ?>
                        <span class="sp-simpleportfolio-icon-video"></span>
                    <?php } ?>

                    <?php if ($this->params->get('thumbnail_type', 'masonry') == 'masonry') { ?>
                        <a class="img-wrapper btn-zoom" href="<?php echo $this->item->popup_img_url; ?>"><img
                                class="sp-simpleportfolio-img" src="<?php echo $this->item->thumb; ?>"
                                alt="<?php echo $this->item->title; ?>"></a>

                    <?php } else if ($this->params->get('thumbnail_type', 'masonry') == 'rectangular') { ?>
                        <a class="img-wrapper btn-zoom" href="<?php echo $this->item->popup_img_url; ?>"><img
                                class="sp-simpleportfolio-img" src="<?php echo $this->item->thumb; ?>"
                                alt="<?php echo $this->item->title; ?>"></a>
                    <?php } else { ?>
                        <a class="img-wrapper btn-zoom" href="<?php echo $this->item->popup_img_url; ?>"><img
                                class="sp-simpleportfolio-img" src="<?php echo $this->item->thumb; ?>"
                                alt="<?php echo $this->item->title; ?>"></a>
                    <?php } ?>

                    <div class="sp-simpleportfolio-overlay">
                        <div class="sp-vertical-middle">
                            <div>
                                <div class="sp-simpleportfolio-btns">
                                    <?php if ($this->item->video) { ?>
                                        <a class="btn-zoom" href="#"
                                            data-featherlight="#sp-simpleportfolio-video<?php echo $this->item->spsimpleportfolio_item_id; ?>"><?php echo Text::_('COM_SPSIMPLEPORTFOLIO_WATCH'); ?></a>
                                    <?php } else { ?>
                                        <a class="btn-link" href="<?php echo $this->item->url; ?>"><i
                                                class="fa fa-plus"></i></a>
                                    <?php } ?>
                                </div>
                                <?php //if($layout_type!='default') { 
                                ?>
                                <h3 class="sp-simpleportfolio-title">
                                    <a href="<?php echo $this->item->url; ?>">
                                        <?php echo $this->item->title; ?>
                                    </a>
                                </h3>
                                <div class="sp-simpleportfolio-tags">
                                    <?php echo implode(', ', $newtags); ?>
                                </div>
                                <?php //} 
                                ?>
                            </div>
                        </div>
                    </div>
                </div>



            </div>

            <?php
            $i++;
            if ($i == 11) {
                $i = 0;
            }
            ?>

        <?php } ?>
    </div>

    <?php if ($this->pagination->pagesTotal > 1) : ?>
        <div class="pagination">
            <?php echo $this->pagination->getPagesLinks(); ?>
        </div>
    <?php endif; ?>
</div>